#ifndef __FIELD_H
#define __FIELD_H

// -- project includes --
#include "shake_math.h"
#include "FieldShape.h"
#include "ShapeParameters.h"
#include "vec3.h"

// -- STL includes --
#include <list>
#include <string>
#include <set>

using std::list;
using std::string;
using std::set;

/*!
  \class AField
  \brief abstract base class for fields in the sublattice

  \author  Steffen Abe
  $Revision$
  $Date$
*/

class AField
{
private:
  set<string> m_possible_shapes;
  AShapeParameters *m_shape;
  
protected:
  void AddShape(const string& s){m_possible_shapes.insert(s);};

public:
  list<string> getShapes()const;
  void setShape(AShapeParameters*);
  virtual bool isScalar()const=0;
  virtual bool isVector()const=0;
  virtual int getSize()const=0;
  virtual double operator()(int,int component=0)const=0;
  virtual void updateFieldValues()=0;
  virtual int idx(int)const=0;
};

#endif //__FIELD_H
