//-- STL includes --
#include <vector>
#include <utility>

using std::vector;
using std::pair;

// -- IO includes --
#include <iostream>

using std::cout;
using std::endl;

/*!
  constructor 

  \param comm the TML communicator used for sending the data back to the master
  \param pis a pointer to the interaction storage
  \param rdf the particle member function to access the data
*/
template <typename T>
ScalarInteractionFieldSlave<T>::ScalarInteractionFieldSlave(TML_Comm* comm,TParallelInteractionStorage<T>* pis,typename T::ScalarFieldFunction rdf):InteractionFieldSlave<T>(comm,pis)
{
  m_rdf=rdf;
} 

/*!
  send full field data and position of the interaction 
*/
template <typename T>
void ScalarInteractionFieldSlave<T>::SendDataFull()
{
  vector<pair<Vec3,double> > data;

  data=m_pis->forAllInnerInteractionsGetWithPos(m_rdf);

  // send data to master
  m_comm->send_gather(data,0);
}

/*!
  send sum only
*/
template <typename T>
void ScalarInteractionFieldSlave<T>::SendDataSum()
{
  vector<double> data_vec;

  // get data from interactions
  m_pis->forAllInnerInteractionsGet(data_vec,m_rdf);

  // sum data
  double sum=0.0;
  for(vector<double>::iterator iter=data_vec.begin();
      iter!=data_vec.end();
      iter++){
    sum+=*iter;
  }

  vector<double> sum_vec;
  sum_vec.push_back(sum);
  m_comm->send_gather(sum_vec,0);
}

/*!
  send maximum only
*/
template <typename T>
void ScalarInteractionFieldSlave<T>::SendDataMax()
{
  vector<double> data_vec;

  // get data from interactions
  m_pis->forAllInnerInteractionsGet(data_vec,m_rdf);

  // sum data
  double max=*(data_vec.begin());
  for(vector<double>::iterator iter=data_vec.begin();
      iter!=data_vec.end();
      iter++){
    max=(*iter > max) ? *iter : max;
  }

  vector<double> max_vec;
  max_vec.push_back(max);
  m_comm->send_gather(max_vec,0);
}
