#ifndef __STDSCALARFIELD_H
#define __STDSCALARFIELD_H

// -- project includes --
#include "Field.h"
#include "vec3.h"

template <class T> class ParallelParticleArray;

/*!
  \class TStdScalarField
  \brief Abstract base class for "standard" scalar fields, i.e. one scalar value per particle

  \author  Steffen Abe
  $Revision$
  $Date$
*/

template <class T>
class TStdScalarField : public AField
{
 protected:
  ParallelParticleArray<T> *m_ppa_ptr; //!< pointer to the parallel particle array
  double (T::*m_accessFn)()const; //!< particle field value access function
  vector<pair<int,double> > m_field_values;

 public:
  TStdScalarField(ParallelParticleArray<T>*,double (T::*)()const);
  virtual bool isScalar()const{return true;};
  virtual bool isVector()const{return false;};
  virtual void updateFieldValues();
  virtual int getSize()const;
  virtual int idx(int)const;
  virtual double operator()(int,int component=0)const;
};

#include "StdScalarField.hpp"

#endif //__STDSCALARFIELD_H
