
/*!
  constructor 

  \param comm the TML communicator used for sending the data back to the master
  \param pis a pointer to the interaction storage
  \param rdf the particle member function to access the data
*/
template <typename T>
VectorInteractionFieldSlave<T>::VectorInteractionFieldSlave(TML_Comm* comm,TParallelInteractionStorage<T>* pis,typename T::VectorFieldFunction rdf)
  :InteractionFieldSlave<T>(comm,pis)
{
  m_rdf=rdf;
}

/*!
  send full field data and position of the interaction 
*/
template <typename T>
void VectorInteractionFieldSlave<T>::SendDataFull()
{
  vector<pair<Vec3,Vec3> > data;

  data=m_pis->forAllInnerInteractionsGetWithPos(m_rdf);

  // send data to master
  m_comm->send_gather(data,0);
}

/*!
  send full field data and position of the interaction 
*/
template <typename T>
void VectorInteractionFieldSlave<T>::SendDataFull2()
{
  vector<pair<pair<Vec3,Vec3>,Vec3> > data;

  data=m_pis->forAllInnerInteractionsGetWithPos2(m_rdf);

  // send data to master
  m_comm->send_gather(data,0);
}

/*!
  send sum only
*/
template <typename T>
void VectorInteractionFieldSlave<T>::SendDataSum()
{
  vector<Vec3> data_vec;

  // get data from interactions
  m_pis->forAllInnerInteractionsGet(data_vec,m_rdf);

  // sum data
  Vec3 sum=Vec3(0.0,0.0,0.0);
  for(vector<Vec3>::iterator iter=data_vec.begin();
      iter!=data_vec.end();
      iter++){
    sum=sum+(*iter);
  }

  vector<Vec3> sum_vec;
  sum_vec.push_back(sum);
  m_comm->send_gather(sum_vec,0);
}

/*!
  send max of the field
*/
template <typename T>
void VectorInteractionFieldSlave<T>::SendDataMax()
{}
