/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "BoundingBox.h"

namespace esys
{

  namespace lsm
  {
    BoundingBox::BoundingBox()
      : m_minPt(Vec3::ZERO),
        m_maxPt(Vec3::ZERO)
    {
    }

    BoundingBox::BoundingBox(const Vec3 &minBBoxPt, const Vec3 &maxBBoxPt)
      : m_minPt(minBBoxPt),
        m_maxPt(maxBBoxPt)
    {
    }
  
    BoundingBox::~BoundingBox()
    {
    }
  
    double BoundingBox::getVolume() const
    {
      Vec3 diff(m_maxPt-m_minPt);
      return diff.X()*diff.Y()*diff.Z();
    }
  
    const Vec3 &BoundingBox::getMinPt() const
    {
      return m_minPt;
    }
  
    const Vec3 &BoundingBox::getMaxPt() const
    {
      return m_maxPt;
    }
    
    bool BoundingBox::operator==(const BoundingBox &bbox) const
    {
      return
        (
          (getMinPt() == bbox.getMinPt())
          &&
          (getMaxPt() == bbox.getMaxPt())
        );
    }
    
    bool BoundingBox::contains(const Vec3 &pt, double tolerence) const
    {
      return
        (
          (getMinPt().X() <= (pt.X() + tolerence))
          &&
          (getMinPt().Y() <= (pt.Y() + tolerence))
          &&
          (getMinPt().Z() <= (pt.Z() + tolerence))
          &&
          (getMaxPt().X() >= (pt.X() - tolerence))
          &&
          (getMaxPt().Y() >= (pt.Y() - tolerence))
          &&
          (getMaxPt().Z() >= (pt.Z() - tolerence))
        );
    }
    
    Vec3 BoundingBox::getSizes() const
    {
      return (getMaxPt() - getMinPt());
    }

    std::ostream &operator<<(std::ostream &oStream, const BoundingBox &bbox)
    {
      oStream << bbox.getMinPt() << " " << bbox.getMaxPt();
      return oStream;
    }
  };
};
