/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef ESYS_LSMSTRINGUTIL_H
#define ESYS_LSMSTRINGUTIL_H

#include <string>
#include <vector>
#include <sstream>

namespace esys
{
  namespace lsm
  {
    typedef std::vector<std::string> StringVector;
    /**
     * Convenience functions for string manipulation.
     */
    namespace StringUtil
    {
      template <class TmplIterator>
      class StdOStreamOp
      {
      public:
        std::string operator()(const TmplIterator &it) const
        {
          std::stringstream sStream;
          sStream << *it;

          return sStream.str();
        }
      };

      template<class TmplIterator, class TmplStringOperator>
      inline
      std::string join(
        TmplIterator begin,
        TmplIterator end,
        const std::string &delim,
        TmplStringOperator toStringOp = StdOStreamOp<TmplIterator>())
      {
        std::string str;
        if (begin != end)
        {
          TmplIterator it = begin;
          str = toStringOp(begin);
          for (it++; it != end; it++)
          {
            str += delim + toStringOp(it);
          }
        }
        return str;
      }

      template <class TmplData>
      class StdIStreamOp
      {
      public:
        TmplData operator()(const std::string &str) const
        {
          std::stringstream sStream(str);
          TmplData data;
          sStream >> data;

          return data;
        }
      };

      template<class TmplData, class TmplStdStreamOp>
      inline      
      std::vector<TmplData> split(
        const std::string &str,
        const std::string &delim,
        TmplStdStreamOp fromStringOp = StdIStreamOp<TmplData>()
      )
      {
        std::vector<TmplData> vec;
        std::string::size_type prevPos = 0;
        std::string::size_type currPos = str.find(delim);
        while (currPos != std::string::npos)
        {
          vec.push_back(
            fromStringOp(str.substr(prevPos, currPos - prevPos))
          );
          prevPos = currPos + delim.length();
          currPos = str.find(delim, prevPos);
        }
        if (prevPos < str.length())
        {
          vec.push_back(
            fromStringOp(str.substr(prevPos, str.length()))
          );
        }
        else if (prevPos == str.length())
        {
          vec.push_back("");
        }

        return vec;
      }

      inline
      StringVector splitStrings(const std::string &str, const std::string &delim)
      {
        return split<std::string, StdIStreamOp<std::string> >(str, delim, StdIStreamOp<std::string>());
      }
      
      inline
      std::string trim(const std::string &str)
      {
        int b = 0;
        for (; (b < str.size()) && std::isspace(str.at(b)); b++)
        {
          // continue
        }
        
        int e = str.size()-1;
        for (; (e > b) && std::isspace(str.at(e)); e--)
        {
          // continue
        }
        
        return ((e >= b) ? str.substr(b, e-b+1) : "");
      }
    };
  };
};

#endif
