/********************************************************************************/
/* DEBUG.H		 							*/
/* 										*/
/* Description:	  Parser evaluation functions					*/
/* Author(s):     D. Place (QUAKES)						*/
/* Creation date: March 1999							*/
/*										*/
/********************************************************************************/
#include "stdafx.h"
#include "shake_debug.h"
#include "console.h"
#include <string.h>
#include <sys/types.h>
#include <malloc.h>

ShakeDebug SDebug ;


ShakeDebug::~ShakeDebug() {
    First() ; 
    while (!IsEnd()) {
	delete Get() ;
	Next() ;
    }
}
	
ShakeDebug::ShakeDebug() {
    LastId = 0 ;
}
	
void ShakeDebug::FreeAll() {
    First() ;
    while (!IsEnd()) {
	delete (char *)(Get()->Ptr) ;
	Next() ;
    }
}

bool ShakeDebug::Assert(bool Expr, char *Where, int line) {
  if (!Expr) {
    tconsole.Critical() << "Assertion failed in " << Where << " at line " << line << "\n" ;
    exit(0) ;
    return false ;
  }
  return true ;
}

bool ShakeDebug::IsValidPtr(void *Ptr, char *Where, int line) {
  if (!Ptr) {
    tconsole.Critical() << "Null Pointer in " << Where << " at line " << line << "\n" ;
    return false;
  }

#ifdef _MEMDEBUG

  First() ;
  while (!IsEnd()) {
    size_t size = mallocblksize (Get()->Ptr) ;
    char *EndP = (char *) Get()->Ptr ;
    EndP += size ;
    if (! Get()->IsDeleted) {
      if ((Ptr >= Get()->Ptr)  && ( Ptr < EndP)) {
         return true ;
      } 
    }
    Next() ;
  }

  First() ;
  while (!IsEnd()) {
    size_t size = mallocblksize (Get()->Ptr) ;
    char *EndP = (char *) Get()->Ptr ;
    EndP += size ;
    if (Get()->IsDeleted) {
      if ((Ptr >= Get()->Ptr) && ( Ptr < EndP)) {
	 tconsole.Critical() << "Pointer is invalid in " << Where << " at line " << line << " (accessing deleted block)\n" ;
	 tconsole.Critical() << "Block allocated in "<< Get()->Where << " at line "<< Get()->line  
		               << " id size " << size << " bytes\n" ;
	 tconsole.Critical() << "Block deleted in "<< Get()->DeletedAt << " at line "<< Get()->DeletedAtLine
		               << " id size " << size << " bytes\n" ;
         return false ;
      }
    }
    Next() ;
  }
  //console.Debug() << "Pointer may be invalid in " << Where << " at line " << line << "\n" ;
#endif

  return true ;
}

void ShakeDebug::NewAlloc(void *Ptr, char *Type, char *Where, int line) {
  if (!Ptr) {
    tconsole.Critical() << "allocation has failed. Not enought memory ?\n" ;
    return ;
  }
  AllocatedBlock *B = new AllocatedBlock;
  B->Ptr = Ptr ;
  B->Type = Type ;
  B->Where = Where ;
  B->Id = LastId ;
  B->line = line ;
  B->IsDeleted = false ;
  LastId++;
  *this << B ;
}

void ShakeDebug::Delete(void *Ptr, char *Type, char *Where, int line) {
    if (!Ptr) {
      tconsole.Critical() << "Deleting a NULL pointer!\n" ;
      return ;
    }
    First() ;
    while (!IsEnd()) {
	if (Get()->Ptr == Ptr) {
	    if (! Get()->IsDeleted) {
		Get()->IsDeleted = true ;
                Get()->DeletedAtLine = line ;
		Get()->DeletedAt = Where ;
		return ;
	    }
	}
	Next() ;
    }
    First() ;
    while (!IsEnd()) {
	if (Get()->Ptr == Ptr) {
	    if (Get()->IsDeleted) {
	        tconsole.Critical() << "Trying to delete twice a block at ["<<Where<<","<<line<<"] of Type <"<<Type<<"> "  << Get()->Id << "\n";
	        tconsole.Critical() << "          deleted at ["<<Get()->DeletedAt<<","<<Get()->DeletedAtLine<<"]\n";
		return ;
	    }
	}
	Next() ;
    }
    tconsole.Critical() << "Trying to delete a non allocated block at ["<<Where<<","<<line<<"] of Type <"<<Type<<"> \n";
}

void ShakeDebug::RestartCounting() {
    First() ;
    while (!IsEnd()) {
	Clear() ;
    }
}

void ShakeDebug::PrintAllocatedBlocks() {
	int n=0,t=0 ;
    First() ;
    while (!IsEnd()) {
		t++ ;
		if (!Get()->IsDeleted) {
			n++ ;
		}
		Next() ;
    }
	tconsole.Info() << t-n << " blocks have been allocated and free\n" ;
	tconsole.Info() << n << " blocks allocated\n" ;
    First() ;
    while (!IsEnd()) {
	if (!Get()->IsDeleted) {
	    if (strcmp(Get()->Type,"char")) {
               tconsole.Debug() << "Allocated block at ["<<Get()->Where<<","<<Get()->line<<"] of Type <"<<Get()->Type<<"> " << Get()->Id << "\n";
	    } else {
               tconsole.Debug() << "Allocated block at ["<<Get()->Where<<","<<Get()->line<<"] of Type <"<<Get()->Type<<"> " << Get()->Id
		               << " - ("<< (char *)(Get()->Ptr) << ")\n" ;
	    }
	}
	Next() ;
    }
}
