/********************************************************************************/
/* DEBUG.H                                                                      */
/*                                                                              */
/* Description:                                                                 */
/* Author(s):     D. Place (QUAKES)                                             */
/* Creation date: March 1999                                                    */
/*                                                                              */
/********************************************************************************/
#ifndef _SHAKE_DEBUG_H_
#define _SHAKE_DEBUG_H_
#include "t_list.h"
//#include "config.h"

#ifdef _MEMDEBUG
#define DB_NEWALLOC(e1,e2) SDebug.NewAlloc(e1,e2,__FILE__,__LINE__)
#define DB_DELETE(e1,e2) SDebug.Delete(e1,e2,__FILE__,__LINE__)
#define DB_PRINT_ALLOCATED_BLOCK SDebug.PrintAllocatedBlocks() 
#define DB_RESTART_COUNTING SDebug.RestartCounting()
#else
#define DB_DELETE(e1,e2) /*No debug*/
#define DB_NEWALLOC(e1,e2) /*No debug*/
#define DB_PRINT_ALLOCATED_BLOCK /*No debug */
#define DB_RESTART_COUNTING /*No debug */
#endif

#ifdef _XDEBUG
#define ISVALID_PTR(e1) SDebug.IsValidPtr((void *)(e1),__FILE__,__LINE__)
#define XDEBUG(e1)  e1
#else
#define ISVALID_PTR(e1) /* No debug */
#define XDEBUG(e1) /* No debug */
#endif

#ifdef _DEBUG
#define DB_ASSERT(e1) SDebug.Assert(e1,__FILE__,__LINE__)
#else
#define DB_ASSERT(e1) /* No debug */
#endif

struct AllocatedBlock {
    void *Ptr ;
    char *Type ;
    char *Where ;
    int Id ;
    int line ;
    int IsDeleted ;
    char *DeletedAt ;
    int DeletedAtLine ;
} ;

class ShakeDebug : public List<AllocatedBlock> {
protected:
    int LastId ;
public:
    ShakeDebug() ;
    ~ShakeDebug() ;
    void NewAlloc(void *Ptr, char *Type, char *Where, int line=0) ;
    void Delete(void *Ptr, char *Type, char *Where, int line=0) ;
    void FreeAll() ;
    void RestartCounting() ;
    void PrintAllocatedBlocks() ;
    bool Assert(bool Expr, char *Where, int line) ;
    bool IsValidPtr(void *ptr, char *Where, int line) ;
};

extern ShakeDebug SDebug ;

#endif
