#ifndef _MYMATH_H_
#define _MYMATH_H_

//--- Project includes ---
#include "shake_stdefs.h"

//--- IO includes ---
#include <iostream>
#include <cstdio>

using std::ostream;

//--- system includes 
#include <cmath>
#include <cstdlib>

// --- STL includes ---
#include <utility>

using std::pair;

//class Vector2D ;

/*!
  \class SVector
  \brief simplified vector class
*/
class SVector {
 public:
  real x,y,z ;
} ;

/*!
  \class Vector
  \brief Vector (3D), handles operations, conversion etc...
*/
/*class Vector : public SVector {
 private:

 public:
  inline Vector() ;
  inline Vector(real ix,real iy, real iz) ;

  inline Vector operator-(const Vector &V) const;
  inline Vector operator-() { return Vector(-x,-y,-z); } ;
  inline Vector operator+(const Vector &V) const;
  inline Vector operator*(real s) ;
  inline Vector operator/(real s) ;
  inline real operator*(const Vector &V) ;
  inline real norm() ;
  inline real norm2() ;

  inline Vector& operator +=(const Vector &V);
  inline Vector& operator -=(const Vector &V);
  inline Vector& operator *=(real s) ;
  inline Vector& operator =(const Vector&V);
} ;


// Out of class method
//--------------------

inline Vector operator *(real s, const Vector &V) ;
inline ostream&  operator<<(ostream& Out, const Vector &P) ;

Vector comp_max(const Vector&,const Vector&); //!< per component maximum
Vector comp_min(const Vector&,const Vector&); //!< per component minimum

*/

/*!
  \class Colour
  \brief Coulour manipulation
*/
class Colour {
 public:
  real red,green,blue,alpha ;

  inline Colour() ;
  inline Colour(real ir, real ig, real ib)  ;
  inline Colour(real ir, real ig, real ib, real ia)  ;
  inline Colour operator + (const Colour & C) const;
  inline Colour & operator += (const Colour & C) ;
  inline Colour operator - (const Colour & C) const;
  inline Colour & operator -= (const Colour & C) ;
  inline Colour operator * (const float s) const;
  inline Colour operator * (const double s) const;
  inline Colour operator / (const double s) const;
  inline Colour operator *= (const float s) ;
  inline Colour operator /= (const double s)  ;
  inline Colour operator *= (const double s)  ;
} ;

inline Colour operator *(float s, const Colour &V) ;
inline Colour operator *(double s, const Colour &V)  ;
inline Colour Mix(const Colour&,const Colour&,double,double); //! mix 2 colors

//! convenience function, returns a random double value between min and max
double shake_frandom(double min,double max); 

/*!
  \class Tpair_cmp
  \brief Comparison between pairs, first components have priority. Enables the use of pairs as keys in associative containers (STL set,map,...)

*/
template <class T1, class T2> 
class Tpair_cmp 
{
 public:
  bool operator()(const pair<T1,T2> &p1, const pair<T1,T2> &p2) const;
};

#include "shake_math.hpp"

#endif

