/*
inline Vector::Vector() {
  x=0.0;
  y=0.0;
  z=0.0;
} 

inline Vector::Vector(real ix,real iy, real iz) { 
   x=ix ; y=iy; z=iz; 
} 

inline Vector Vector::operator-(const Vector &V) const { 
   return Vector(x-V.x,y-V.y,z-V.z) ; 
} 

inline Vector Vector::operator+(const Vector &V) const { 
   return Vector(x+V.x,y+V.y,z+V.z) ; 
} 

inline Vector Vector::operator*(real s) { 
   return Vector(s*x,s*y,s*z) ; 
} 

inline Vector Vector::operator/(real s) { 
   return Vector(x/s,y/s,z/s) ; 
} 

inline real Vector::operator*(const Vector &V) { 
   return (x*V.x+y*V.y+z*V.z) ; 
} 

inline real Vector::norm() { 
   return sqrt(x*x+y*y+z*z) ; 
} 

inline real Vector::norm2() { 
   return x*x+y*y+z*z ; 
} 

inline Vector& Vector::operator +=(const Vector &V)  { 
   x+=V.x ; y+=V.y; z+=V.z ; 
   return *this;
} 

inline Vector& Vector::operator -=(const Vector &V)  { 
   x-=V.x ; y-=V.y; z-=V.z ; 
   return *this;
} 

inline Vector& Vector::operator *=(real s)  { 
   x*=s ; y*=s; z*=s; 
   return *this;
} 

inline Vector& Vector::operator = ( const Vector &V)
{
  x=V.x; y=V.y; z=V.z;
  return *this;
}

// Out of class method
//--------------------

inline Vector operator *(real s, const Vector &V) { return Vector(s*V.x,s*V.y,s*V.z) ; }

inline ostream&  operator<<(ostream& Out, const Vector &P) {
   Out << "(" << P.x << "," << P.y << "," << P.z << ")" ;
   return flush(Out) ;
 }

*/
Colour::Colour() { 
	red = green = blue = 0.0 ; 
	alpha = 1.0 ; 
} 
  
Colour::Colour(real ir, real ig, real ib) { 
	red = ir ; 
	green = ig ; 
	blue = ib ; 
	alpha = 1.0 ; 
} 

Colour::Colour(real ir, real ig, real ib, real ia) { 
	red = ir ; 
	green = ig ; 
	blue = ib ; 
	alpha = ia ; 
} 

Colour Colour::operator + (const Colour & C) const
{
    real R,G,B,A ;

    R = red * alpha + C.red * C.alpha; 
    G = green * alpha + C.green * C.alpha; 
    B = blue * alpha + C.blue * C.alpha; 
	
    A = MIN(1.0,alpha + C.alpha) ;
    return Colour(MIN(R,1.0),MIN(G,1.0),MIN(B,1.0),A) ;
}

Colour & Colour::operator += (const Colour & C) 
{ 
  real R,G,B,A ;

  R = red * alpha + C.red * C.alpha; 
  G = green * alpha + C.green * C.alpha; 
  B = blue * alpha + C.blue * C.alpha;
  A = alpha + C.alpha ;	
  
  red=MIN(R,1.0);
  green=MIN(G,1.0);
  blue=MIN(B,1.0);
  alpha=MIN(A,1.0);
  
  return *this ; 
} 

Colour Colour::operator - (const Colour & C) const
{
    real R,G,B,A ;

    R = red * alpha - C.red * C.alpha; 
    G = green * alpha - C.green * C.alpha; 
    B = blue * alpha - C.blue * C.alpha; 
	
    A = MAX(1.0,alpha - C.alpha) ;
    return Colour(MAX(R,0.0),MAX(G,0.0),MAX(B,0.0),A) ;
}

Colour & Colour::operator -= (const Colour & C) 
{ 
	red -= C.red ; 
	green -= C.green ; 
	blue -= C.blue ; 
	return *this ; 
} 

Colour Colour::operator * (const float s) const 
{ 
	return Colour(red*s,green*s,blue*s) ; 
} 

Colour Colour::operator * (const double s) const
{ 
	return Colour(red*s,green*s,blue*s) ; 
} 

Colour Colour::operator / (const double s) const
{ 
	return Colour(red/s,green/s,blue/s) ; 
} 

Colour Colour::operator *= (const float s) { 
	red *= s ; 
	green *=s ; 
	blue *=s ; 
	return *this ; 
} 

Colour Colour::operator *= (const double s) { 
	red *= s ; 
	green *=s ; 
	blue *=s ; 
	return *this ; 
} 

Colour Colour::operator /= (const double s) { 
	red /= s ; 
	green /=s ; 
	blue /=s ; 
	return *this ; 
} 

Colour operator *(float s, const Colour &V) { 
	return Colour(V.red*s,V.green*s,V.blue*s); 
} 
Colour operator *(double s, const Colour &V) { 
	return Colour(V.red*s,V.green*s,V.blue*s); 
} 

Colour Mix(const Colour& C1,const Colour& C2,double w1,double w2)
{
  double r=C1.red*w1+C2.red*w2;
  double g=C1.green*w1+C2.green*w2;
  double b=C1.blue*w1+C2.blue*w2;
  double a=C1.alpha*w1+C2.alpha*w2;

  return Colour(r,g,b,a);
}

template<class T1, class T2> 
bool Tpair_cmp<T1,T2>::operator()(const pair<T1,T2> &P1,const pair<T1,T2> &P2) const
{
  return ((P1.first<P2.first) || ((P1.first==P2.first)&&(P1.second<P2.second)));
}
