#include "console.h"
#include "shake_debug.h"

void Cxdr::checksize(int n) {
    if (m_cur+n >m_len) {
       tconsole.Error() << "CXDR: Out of data\n" ;
    }
}

void Cxdr::reserve(int n) {
    if (m_cur+n >m_truelen) {
        if (!m_growable) {
	  tconsole.Critical() << "try to grow a non-growable buffer\n" ;
	  // exit(0) ;
        }
        char *newbuff= new char[MAX(m_truelen*2,m_cur+n+PAGESIZE)] ;
        if (m_buffer) {
            memcpy(newbuff,m_buffer,m_len) ;
            delete m_buffer ;
        }
        m_len = m_cur+n ;
        m_truelen = MAX(m_truelen*2,m_cur+n+PAGESIZE) ;
        m_buffer = newbuff ;
    } else {
      if (m_cur+n >m_len) {
        m_len = m_cur+n ;
      }
    }
}

void Cxdr::store(u_long &l) {
    reserve(sizeof(u_long)) ;
    *(u_long *)(&m_buffer[m_cur]) = l ;
    m_cur += sizeof(u_long) ;
}

void Cxdr::restore(u_long &l) {
    XDEBUG(checksize(sizeof(u_long))) ;
    l = *(u_long *)(&m_buffer[m_cur]) ;
    m_cur += sizeof(u_long) ;
}

void Cxdr::store(u_short &l) {
    reserve(sizeof(u_long)) ;
    *(u_short *)(&m_buffer[m_cur]) = l ;
    m_cur += sizeof(u_long) ;
}
  
void Cxdr::restore(u_short &l) {
    XDEBUG(checksize(sizeof(u_long))) ;
    l = *(u_short *)(&m_buffer[m_cur]) ;
    m_cur += sizeof(u_long) ;
}

void Cxdr::store(const char *s, int n) {
  int n2 = (n/4+1)*4 ;
    reserve(n2) ;
    memcpy(&m_buffer[m_cur],s,n) ;
    m_cur += n2 ;
}

void Cxdr::restore(char *s, int n) {
  int n2 = (n/4+1)*4 ;
    XDEBUG(checksize(n2)) ;
    memcpy(s,&m_buffer[m_cur],n) ;
    m_cur += n2 ;
}

Cxdr & Cxdr::operator << (signed long s) {
  u_long nl,*hl ;
  hl=(u_long *)(&s) ;
  nl = htonl(*hl) ;
  store(nl) ;
  return *this ;
}

Cxdr & Cxdr::operator >> (signed long &s) {
  u_long nl,*hl ;
  hl=(u_long *)(&s) ;
  restore(nl) ;
  *hl = ntohl(nl) ;
  return *this ;
}

Cxdr & Cxdr::operator << (double s) {
  XDEBUG(if (sizeof(double)!=2*sizeof(u_long)) { tconsole.Critical() << "Cannot decode doubles!\n" ;  exit(0) ; })
  u_long nl1,nl2,*hl ;
  hl=(u_long *)(&s) ;
  nl1 = htonl(*hl) ;hl++;
  nl2 = htonl(*hl) ;
  
  if (m_is_reverse) {
    store(nl1) ;
    store(nl2) ;
  } else {
    store(nl2) ;
    store(nl1) ;
  }
  return *this ;
}

Cxdr & Cxdr::operator >> (double &s) {
  XDEBUG( if (sizeof(double)!=2*sizeof(u_long)) {
    tconsole.Critical() << "Cannot encode doubles!\n" ;
    exit(0) ;
  })
  u_long nl1,nl2,*hl ;
  hl=(u_long *)(&s) ;
  restore(nl1) ;
  restore(nl2) ; 
  if(m_is_reverse){
    *hl = ntohl(nl1) ;  hl++ ;
    *hl = ntohl(nl2) ;
  } else {
    *hl = ntohl(nl2) ;  hl++ ;
    *hl = ntohl(nl1) ;
  }
  return *this ;
}


Cxdr & Cxdr::operator << (unsigned long s) {
  u_long nl ;
  nl = htonl(s) ;
  store(nl) ;
  return *this ;
}

Cxdr & Cxdr::operator >> (unsigned long &s) {
  u_long nl ;
  restore(nl) ;
  s = ntohl(nl) ;
  return *this ;
}

Cxdr & Cxdr::operator << (signed short s) {
  u_short ns,*hs ;
  hs=(u_short *)(&s) ;
  ns = htons(*hs) ;
  store(ns) ;
  return *this ;
}

Cxdr & Cxdr::operator >> (signed short &s) {
  u_short ns,*hs ;
  hs=(u_short *)(&s) ;
  restore(ns) ;
  *hs = ntohs(ns) ;
  return *this ;
}

Cxdr & Cxdr::operator << (unsigned short s) {
  u_short ns ;
  ns = htons(s) ;
  store(ns) ;
  return *this ;
}

Cxdr & Cxdr::operator >> (unsigned short &s) {
  u_short ns ;
  restore(ns) ;
  s = ntohs(ns) ;
  return *this ;
}

Cxdr & Cxdr::operator << (const char *s) {
  u_long nl,hl ;
  hl = strlen(s) ;
  nl = htonl(hl) ;
  store(nl) ;
  store(s,hl) ;
  return *this ;
}

Cxdr & Cxdr::operator >> (char *s) {
  u_long nl,hl ;
  restore(nl) ;
  hl = ntohl(nl) ;
  restore(s,hl) ;
  s[hl]='\0' ;
  return *this ;
}

char * Cxdr::data() {
  return m_buffer ;
}

unsigned long Cxdr::len() {
  return m_len ;
} 
