#include "stdafx.h"
#include <string.h>
#include "console.h"
#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "tcpsocket.h"

#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef _WINDOWS
//  #include <winsock2.h>
  #define sock_errno WSAGetLastError()
#endif

#ifndef _WINDOWS
  #include <sys/socket.h>
  #include <netdb.h>
  #include <sys/wait.h>
  #include <unistd.h>
  #include <stropts.h>
  #include <fcntl.h>
  #define closesocket(e1) ::close(e1)
  #define sock_errno errno
#endif

#if HAVE_SYS_IOCTL_H
  #include <sys/ioctl.h>
#elif HAVE_ASM_IOCTLS_H
  #include <asm/ioctls.h>
#endif


CSocket::CSocket() {
  m_isNonBlocking = false ;
}

CSocket::~CSocket() {
  //closesocket(m_sockfd) ;
}

bool CSocket::create() {
    m_sockfd = ::socket(AF_INET, SOCK_STREAM, 0);
    if (m_sockfd == -1) {
      tconsole.Error() << "Unable to create socket : " << strerror(sock_errno) << "\n" ;
    } 
    /*
      else {
      int bufsize ;
      int optlen ;
      getsockopt(m_sockfd,SOL_SOCKET,SO_SNDBUF,&bufsize,&optlen) ; 
      tconsole.XDebug() << "Socket send buffer size was " << bufsize << " int is" << sizeof(int) << " this is " << optlen << "\n" ;
      bufsize = 16384 ;
      setsockopt(m_sockfd,SOL_SOCKET,SO_SNDBUF,&bufsize,sizeof(bufsize)) ; 
      getsockopt(m_sockfd,SOL_SOCKET,SO_SNDBUF,&bufsize,&optlen) ; 
      tconsole.XDebug() << "Socket send buffer size is now " << bufsize << "\n" ;

      getsockopt(m_sockfd,SOL_SOCKET,SO_RCVBUF,&bufsize,&optlen) ; 
      tconsole.XDebug() << "Socket rcv buffer size was " << bufsize << " int is" << sizeof(int) << " this is " << optlen << "\n" ;
      bufsize = 16384 ;
      setsockopt(m_sockfd,SOL_SOCKET,SO_RCVBUF,&bufsize,sizeof(bufsize)) ; 
      getsockopt(m_sockfd,SOL_SOCKET,SO_RCVBUF,&bufsize,&optlen) ; 
      tconsole.XDebug() << "Socket rcv buffer size is now " << bufsize << "\n" ;
    }
    */
    return (m_sockfd!=-1) ;
};


bool CSocket::create(char nType) {
    if(nType == TCP) {
        return create();
    } else if(nType == UDP) {
        m_sockfd = ::socket(AF_INET, SOCK_DGRAM, 0);
	if (m_sockfd == -1) {
	  tconsole.Error() << "Unable to create socket : " << strerror(sock_errno) << "\n" ;
	}
        return (m_sockfd!=-1) ;
    } else {
        return false;
    }
}

bool CSocket::connect(int nPort, char *Hostname) {
    hostent *Host;
    int i,res ;

    Host = gethostbyname(Hostname) ;

    if (Host == NULL) {
        console.Error() << "Cannot get host name " << Hostname << "address\n" ;
        return false;
    }
        
    m_INetAddress.sin_family = AF_INET;
    m_INetAddress.sin_port = htons(nPort);
    m_INetAddress.sin_addr = *((struct in_addr *)Host->h_addr);

    //bzero(&(INetAddress.sin_zero), 8);
    for (i=0; i<8; i++) {
        ((char *)(&(m_INetAddress.sin_zero)))[i] = 0 ;
    }
    
    res = ::connect(m_sockfd, (struct sockaddr *)&m_INetAddress, sizeof(struct sockaddr)) ;
    if( res == -1) {
        tconsole.Error() << "Cannot connect to " << Hostname << ":" << nPort ;
	tconsole.Error(false) << ", " << strerror(sock_errno) << "\n" ;
        return false;
    }
    
    return true;
}

bool CSocket::print_err() {
  switch (sock_errno) {
  case EBADF:
    tconsole.Debug() << "invalid descriptor\n" ;
    break ;
  case ENOTSOCK:
    tconsole.Debug() << "not a socket\n" ;
    break ;
  case EFAULT:
    tconsole.Debug() << "invalid user space addr\n" ;
    break ;
  case EMSGSIZE:
    tconsole.Debug() << "Size require automatice send\n" ;
    break ;
  case ENOBUFS:
    tconsole.Debug() << "Buffer unavailbale, or overflow\n" ;
    break ;
  case ECONNREFUSED:
    tconsole.Debug() << "invalid port\n" ;
    break ;
  case EISCONN:
    tconsole.Debug() << "invalid socket opt\n" ;
    break ;
  case EACCES:
    tconsole.Debug() << "braldcast err\n";
    break ;
  case EHOSTDOWN:
    tconsole.Debug() << "braldcast err\n";
    break ;
  case EWOULDBLOCK:
    if (isNonBlocking()) {
      return true ;
    } else {
      tconsole.Debug() << "Would block\n" ;
    }
    break ;
  default:
    //    perror() ;
    tconsole.Debug() << "Undefined error\n" ;
    perror("err") ;
    break ;    
  }
  return false ;
}

long CSocket::MAX_TCP_LENGTH = 8192 ;

long CSocket::send(Cxdr &cxdr) {
  unsigned long length = cxdr.len(); 
  unsigned long slength = cxdr.len(); 
  long Bytes_Sent = 0,br,res ;
  char buff[5] ;

  bool blk = isNonBlocking() ;
  if (cxdr.len() > MAX_TCP_LENGTH) {
    //nonBlocking(false) ;
  }

  Cxdr xdr_length ;
  xdr_length << length ;
  send((void *)(xdr_length.data()),xdr_length.len()) ;
  while (length > MAX_TCP_LENGTH) {
    tconsole.XDebug() << "send multiple..\n" ;
    br = send((void *)(&(cxdr.data()[slength-length])),MAX_TCP_LENGTH) ;
    Bytes_Sent += br ;
    do {
      res = recv((void *)buff,1) ; // synchronize send to prevent overflow
    } while ((res==-1)&&(wouldBlock())) ;
    length -= MAX_TCP_LENGTH ;
  }
  Bytes_Sent += send((void *)(&(cxdr.data()[slength-length])),length) ;

  return Bytes_Sent ;
}

long CSocket::recv(Cxdr &cxdr) {
  unsigned long length = 0 ; 
  unsigned long slength = 0 ; 
  char buff[5] ;
  long Bytes_Rcv = 0, br ;

  Cxdr xdr_length ;
  xdr_length << length ; xdr_length.reset() ; // reserve space for one long...
  recv((void *)(xdr_length.data()),xdr_length.len()) ;
  xdr_length >> length ;
  cxdr.reset(length) ;
  slength = length ;
  while (length > MAX_TCP_LENGTH) {
    tconsole.XDebug() << "receiving multiple..\n" ;
    do {
      br = recv((void *)(&(cxdr.data()[slength-length])),MAX_TCP_LENGTH) ;
    } while (wouldBlock() && (br==-1)) ;
    Bytes_Rcv += br ;    
    send((void *)buff,1) ; // synchronize send to prevent overflow
    length -= br ; // MAX_TCP_LENGTH ;
  }

  if (cxdr.len() > MAX_TCP_LENGTH) {
    do {
      br = recv((void *)(&(cxdr.data()[slength-length])),length) ;
    } while (wouldBlock() && (br==-1)) ;
    Bytes_Rcv += br; 
    //    tconsole.XDebug() << "total expecting " << cxdr.len() << " received " << Bytes_Rcv << " bytes\n" ;    
  } else {
    Bytes_Rcv += recv((void *)(&(cxdr.data()[slength-length])),length) ;
  }
  return Bytes_Rcv ;
}

long CSocket::send(const void *Message, int nLength) {
    int Bytes_Sent;
    if (nLength==0) {
        nLength=strlen((char *)Message) ;
    }
    
    if (nLength > MAX_TCP_LENGTH) {
      tconsole.Critical() << "Buffer too big going through TCP/IP\n" ;
    }
    Bytes_Sent = ::send(m_sockfd, (char *)Message, nLength, 0);
    if ((Bytes_Sent>0) && (Bytes_Sent!=nLength)) {
      console.Critical() << "Ooops, could not send all the data...\n" ;
    }
    if ((Bytes_Sent == -1 ) || (Bytes_Sent!=nLength)) {
      print_err() ;
    }
    return Bytes_Sent;
}


long CSocket::recv(void *Buffer, int nLength) {
    int Bytes_Recv;

    if (nLength > MAX_TCP_LENGTH) {
      tconsole.Critical() << "Buffer too big going through TCP/IP\n" ;
    }
    Bytes_Recv = ::recv(m_sockfd, (char *)Buffer, nLength, 0);
    if ((Bytes_Recv>0) && (Bytes_Recv<nLength)) {
      int res ;
      while (Bytes_Recv!=nLength) {
	tconsole.XDebug() << "waiting for more data...\n" ;
	res = ::recv(m_sockfd, &((char *)Buffer)[Bytes_Recv], nLength-Bytes_Recv, 0);
	if (res>0) Bytes_Recv += res ;
      }
    }
    if (Bytes_Recv == -1 ) {
      print_err() ;
    } else if (Bytes_Recv > nLength) {
      tconsole.Critical() << "Ooops, too much data\n" ;
    }
    return Bytes_Recv;
}

bool CSocket::isNonBlocking() {
  return m_isNonBlocking ;
}

void CSocket::nonBlocking(bool flag) {

    m_isNonBlocking = flag ;

#ifdef _WINDOWS
    unsigned long uflag ;

    if (flag) {
        uflag=1 ;
    } else {
        uflag=0 ;
    }
    ioctlsocket(m_sockfd,FIONBIO,&uflag) ;
#else
    //    ioctl(m_sockfd,FIONBIO,&uflag) ;
    /* -- anotherway to set to nonblocking... seams to work as well...*/
    if (flag) {
      int flags = fcntl(m_sockfd,F_GETFL) ;
      fcntl(m_sockfd,F_SETFL,flags|O_NONBLOCK) ;
    } else {
      int flags = fcntl(m_sockfd,F_GETFL) ;
      fcntl(m_sockfd,F_SETFL,flags&(~O_NONBLOCK)) ;
    }
    
#endif
}

bool CSocket::close() {
  int res ;
  res = closesocket(m_sockfd) ;

  if( res == -1) {
    tconsole.Error() << "cannot close socket : " << strerror(sock_errno) << "\n" ;
    }
  return (res!= -1);
}

bool CSocket::shutdown() {
#ifndef _WINDOWS
  int res ;

  res = ::shutdown(m_sockfd, SHUT_RDWR) ;
  if( res == -1) {
    tconsole.Error() << "cannot shutdown socket : " << strerror(sock_errno) << "\n" ;
    }
  return (res!= -1);
#else 
  return true ;
#endif
}

bool CSocket::bind(int nPort) {
    int res ;

    m_INetAddress.sin_family = AF_INET;
    m_INetAddress.sin_port = htons(nPort);
    m_INetAddress.sin_addr.s_addr = INADDR_ANY;
    int i ;
    for (i=0; i<8; i++) {
        ((char *)(&(m_INetAddress.sin_zero)))[i] = 0 ;
    }
    //   bzero(&(INetAddress.sin_zero), 8);
    
    res = ::bind(m_sockfd, (struct sockaddr *)&m_INetAddress, sizeof(struct sockaddr)) ;
    if( res == -1) {
      tconsole.Error() << "cannot bind to port " << nPort << " : " << strerror(sock_errno) << "\n" ;
      return false;
    }
        
    return true;
}

bool CSocket::listen() {
    if(::listen(m_sockfd, 5) == -1) {
      tconsole.Error() << "cannot listen to socket : " << strerror(sock_errno) << "\n" ;
      return false;
    }
    return true;
}

bool CSocket::wouldBlock() {
#ifdef _WINDOWS
    return (WSAGetLastError() == WSAEWOULDBLOCK) ;
#else
    return sock_errno==EWOULDBLOCK ;
#endif
}

bool CSocket::accept(CSocket &s) {
#if HAVE_SYS_IOCTL_H || HAVE_ASM_IOCTLS_H
    socklen_t sin_size = sizeof(struct sockaddr_in);
#else
    int sin_size = sizeof(struct sockaddr_in);
#endif
    //sockaddr_in RemoteAddress;
    int m_newfd = ::accept(m_sockfd, (struct sockaddr *)&(s.m_RemoteAddress), &sin_size) ;
    if(m_newfd == -1) {
        if (!wouldBlock()) {
	  tconsole.Error() << "cannot accept transmission : " << strerror(sock_errno) << "\n" ;
	}
        return false;
    }
	s.m_sockfd = m_newfd ;
	strncpy((char *)&s.m_INetAddress,(char *)&m_INetAddress,sizeof(m_INetAddress)) ;
    return true;
}








