
#include "FoundationTest/Vec3Test.h"

// -- test includes --
#include "Model/RotParticle.h"
#include "Model/RotFricInteraction.h"
#include "Model/SimpleParticleData.h"

#include "UnitTest/Macros.h"
#include <boost/limits.hpp>

#include <ctime>

using namespace esys::lsm;

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( Vec3Test );

  class Vec3Test::Impl
  {
  public:
    Impl()
    {
    }
  };

  Vec3Test::Vec3Test()
    : CppUnit::TestCase(),
      m_impl(new Impl())
  {
  }

  Vec3Test::~Vec3Test()
  {
  }

  void Vec3Test::setUp()
  {
    CppUnit::TestCase::setUp();
  }

  void Vec3Test::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }

  class ArrayVec3
  {
  public:
    ArrayVec3()
    {
      m_data[0] = 0.0;
      m_data[1] = 0.0;
      m_data[2] = 0.0;
    }

    ArrayVec3(double x, double y, double z)
    {
      m_data[0] = x;
      m_data[1] = y;
      m_data[2] = z;
    }
    
    ArrayVec3(const ArrayVec3 &vec3)
    {
      m_data[0] = vec3.m_data[0];
      m_data[1] = vec3.m_data[1];
      m_data[2] = vec3.m_data[2];
    }
    
    ArrayVec3 &operator=(const ArrayVec3 &vec3)
    {
      m_data[0] = vec3.m_data[0];
      m_data[1] = vec3.m_data[1];
      m_data[2] = vec3.m_data[2];
      
      return *this;
    }

    const double &operator[](int i) const
    {
      return m_data[i];
    }

    double &operator[](int i)
    {
      return m_data[i];
    }

    ArrayVec3 operator+(const ArrayVec3 &vec3) const
    {
      return 
        ArrayVec3(
          m_data[0] + vec3.m_data[0],
          m_data[1] + vec3.m_data[1],
          m_data[2] + vec3.m_data[2]
        );
    }

    ArrayVec3 &operator+=(const ArrayVec3 &vec3)
    {
      m_data[0] += vec3.m_data[0];
      m_data[1] += vec3.m_data[1];
      m_data[2] += vec3.m_data[2];
      return *this;
    }
        
    ArrayVec3 operator-() const
    {
      return 
        ArrayVec3(
          -m_data[0],
          -m_data[1],
          -m_data[2]
        );
    }

    ArrayVec3 operator-(const ArrayVec3 &vec3) const
    {
      return 
        ArrayVec3(
          m_data[0] - vec3.m_data[0],
          m_data[1] - vec3.m_data[1],
          m_data[2] - vec3.m_data[2]        
        );
    }

    ArrayVec3 operator*(double s) const
    {
      return 
        ArrayVec3(
          m_data[0]*s,
          m_data[1]*s,
          m_data[2]*s
        );
    }
    
  private:
    double m_data[3];
  };
  
  ArrayVec3 operator*(double s, const ArrayVec3 &vec3)
  {
    return vec3*s;
  }

  class MemberVec3
  {
  public:
    MemberVec3() : m_x(0.0), m_y(0.0), m_z(0.0)
    {
    }

    MemberVec3(const MemberVec3 &vec) : m_x(vec.m_x), m_y(vec.m_y), m_z(vec.m_z)
    {
    }

    MemberVec3 &operator=(const MemberVec3 &vec)
    {
      m_x = vec.m_x;
      m_y = vec.m_y;
      m_z = vec.m_z;
      return *this;
    }

    MemberVec3(double x, double y, double z)
      : m_x(x),
        m_y(y),
        m_z(z)
    {
    }
    
    const double &operator[](int i) const
    {
      return static_cast<const double *>(&m_x)[i];
    }

    double &operator[](int i)
    {
      return static_cast<double *>(&m_x)[i];
    }

    MemberVec3 operator+(const MemberVec3 &vec3) const
    {
      return 
        MemberVec3(
          m_x + vec3.m_x,
          m_y + vec3.m_y,
          m_z + vec3.m_z
        );
    }

    MemberVec3 &operator+=(const MemberVec3 &vec3)
    {
      m_x += vec3.m_x;
      m_y += vec3.m_y;
      m_z += vec3.m_z;
      return *this;
    }
    
    MemberVec3 operator-() const
    {
      return 
        MemberVec3(
          -m_x,
          -m_y,
          -m_z
        );
    }

    MemberVec3 operator-(const MemberVec3 &vec3) const
    {
      return 
        MemberVec3(
          m_x - vec3.m_x,
          m_y - vec3.m_y,
          m_z - vec3.m_z        
        );
    }

    MemberVec3 operator*(double s) const
    {
      return 
        MemberVec3(
          m_x*s,
          m_y*s,
          m_z*s
        );
    }
    
  private:
    double m_x;
    double m_y;
    double m_z;
  };
  MemberVec3 operator*(double s, const MemberVec3 &vec3)
  {
    return vec3*s;
  }

  bool operator==(const ArrayVec3 &v1, const MemberVec3 &v2)
  {
    return
      (
        (v1[0] == v2[0])
        &&
        (v1[1] == v2[1])
        &&
        (v1[2] == v2[2])
      );
  }
  
  bool operator==(const MemberVec3 &v1, const ArrayVec3 &v2)
  {
    return (v2 == v1);
  }
  
  template <typename TmplVec3>
  void oStreamOp(std::ostream &oStream, const TmplVec3 &vec)
  {
    oStream << vec[0] << " " << vec[1] << " " << vec[2];
  }
  
  std::ostream &operator<<(std::ostream &oStream, const ArrayVec3 &vec)
  {
    oStreamOp(oStream, vec);
    return oStream;
  }

  std::ostream &operator<<(std::ostream &oStream, const MemberVec3 &vec)
  {
    oStreamOp(oStream, vec);
    return oStream;
  }

  template<typename TmplVec3>
  TmplVec3 getOperatorResult(int numIterations)
  {
    TmplVec3 prev2;
    TmplVec3 prev1;
    TmplVec3 current;
    for (int i = 1; i <= numIterations; i++)
    {
      current = -TmplVec3(10.0/i, -2.0/i, (2.0*i)/(i+10.0)) + (1.0/i)*prev1 - prev2;
      prev2   = prev1;
      prev1   = current;
    }
    return current;
  }

  class ClockT
  {
  public:
    ClockT() : m_ticks(0)
    {
    }
    
    double getTime() const
    {
      return m_ticks/static_cast<double>(CLOCKS_PER_SEC);
    }
    
    void setCurrent()
    {
      m_ticks = std::clock();
    }
    
    ClockT operator-(const ClockT &clck) const
    {
      return ClockT(m_ticks - clck.m_ticks);
    }
    
    protected:
      ClockT(const clock_t &ticks)
      {
        m_ticks = ticks;
      }
    private:
      std::clock_t m_ticks;
  };

  class Timer
  {
  public:
    Timer()
    {
    }
    
    void start()
    {
      m_start.setCurrent();
    }
    
    void stop()
    {
      m_stop.setCurrent();
    }
    
    double getInterval()
    {
      return (m_stop-m_start).getTime();
    }
    
  private:
    ClockT m_start;
    ClockT m_stop;
  };
  
  void Vec3Test::testPerformanceArrayVsMembers()
  {
    const int numIterations = 7500000;
    Timer arrayTimer;
    arrayTimer.start();
    ArrayVec3  v1 = getOperatorResult<ArrayVec3>(numIterations);
    arrayTimer.stop();
    
    Timer memberTimer;
    memberTimer.start();
    MemberVec3 v2 = getOperatorResult<MemberVec3>(numIterations);
    memberTimer.stop();
    
    LSMUNIT_ASSERT_VEC3_EQUAL2(v1, v2, DEFAULT_TOL_DBL);
    
    const double arrayTime = arrayTimer.getInterval();
    const double memberTime = memberTimer.getInterval();
    CPPUNIT_ASSERT_MESSAGE(
      (MsgStream() 
        << "memberTime = " << memberTime 
        << ", arrayTime = " << arrayTime
      ).getMsg(),
      true
    );
  }
  
  void Vec3Test::testIndex()
  {
    MemberVec3 a(1.0, 2.0, 3.0);
    CPPUNIT_ASSERT_EQUAL(1.0, a[0]);
    CPPUNIT_ASSERT_EQUAL(2.0, a[1]);
    CPPUNIT_ASSERT_EQUAL(3.0, a[2]);
  }
  
  template<typename TmplVec3>
  TmplVec3 doConstructIterations(int numIterations)
  {
    TmplVec3 result;
    for (int i = 1; i <= numIterations; i++)
    {
      const TmplVec3 vec(10.0/i, -2.0/i, (2.0*i)/(5.0*i+10.0));
      result += vec;
    }
    return result;    
  }

  template<typename TmplVec3>
  TmplVec3 doAssignConstructIterations(int numIterations)
  {
    TmplVec3 result;
    for (int i = 1; i <= numIterations; i++)
    {
      const TmplVec3 vec = TmplVec3(10.0/i, -2.0/i, (2.0*i)/(5.0*i+10.0));
      result += vec;
    }
    return result;
  }

  void Vec3Test::testPerformanceCopyVsAssignConstruct()
  {
    const int numIterations = 20000000;
    Timer constructTimer;
    constructTimer.start();
    const ArrayVec3  v1 = doConstructIterations<ArrayVec3>(numIterations);
    constructTimer.stop();

    Timer assignTimer;
    assignTimer.start();
    const ArrayVec3  v2 = doAssignConstructIterations<ArrayVec3>(numIterations);
    assignTimer.stop();
    
    LSMUNIT_ASSERT_VEC3_EQUAL2(v1, v2, 0.0);
    
    const double constructTime = constructTimer.getInterval();
    const double assignTime = assignTimer.getInterval();    
    LSMUNIT_ASSERT_DOUBLES_EQUAL2(
      constructTime,
      assignTime,
      constructTime*0.05
    );
  }
};
