#include "Foundation/vec3.h"
#include "Foundation/BoundingBox.h"
#include "Geometry/Sphere2d.h"
#include "Geometry/Line.h"
#include "Geometry/RandomBlock.h"
#include "Geometry/RandomBlock3D.h"
#include "Geometry/SplitBlock.h"
#include "Geometry/SplitBlock3D.h"
#include "Geometry/LayeredBlock.h"
#include "Geometry/PaddedBlock3D.h"
#include "Geometry/SimpleParticle.h"
#include "Geometry/GougeBlock3D.h"
#include "Geometry/GranularGougeBlock3D.h"
#include "Geometry/VtkXmlWriter.h"

// -- I/O includes --
#include <iostream>
#include <fstream>
using std::cout;
using std::endl;
using std::ofstream;

// -- System includes --
#include <cstdlib>
#include <sys/time.h>
using std::atoi;
using std::atof;
//using std::gettimeofday;

//-- STL includes --
#include <vector>
#include <string>

using std::vector;
using std::string;
using namespace esys::lsm;
int main(int argc,char** argv)
{
  double xmax = 0.0;
  double ymax = 0.0;
  double zmax = 0.0;
  double rmin = 1.0;
  int args_read=1;
  int ntry = 0;
  string filename;
  string vtkXmlFileName;
  string statfilename;
  int stat_nbins = 0;
  int split_dir = 0;
  bool calc_stat=false;
  bool seed_def=false;
  bool layered=false;
  bool tag_centre=false;
  bool tag_edge_y=false;
  bool tag_edge_z=false;
  bool split=false;
  bool tag_split=false;
  bool circular_x=false;
  bool is_3D=false;
  bool is_padded=false;
  bool isGouge = false;
  double gougeSize = 0.0;
  double gougeMinRadius = 1.0;
  double gougeMaxRadius = 1.0;
  int    gougeTag = 0;
  double faultSize = 0.0;
  double faultMinRadius = 1.0;
  double faultMaxRadius = 1.0;
  int    faultTag = 0;
  int    minDrivingTag = 0;
  int    maxDrivingTag = 0;
  double distanceFromBBoxEdge = 0.0;
  double tolerence = DBL_EPSILON*1000;
  double connTolerence = tolerence*10;
  int ctag,extag1,extag2,eztag1,eztag2,sptag;
  double extag_d,eztag_d,sptag_d;
  double ysplit;
  bool split_rough=false;
  double pad;
  unsigned int random_seed = 237;
  vector<double> layers;
  // granular stuff
  bool granular=false;
  double sdx,sdy,sdz;
  double rdx,rdy,rdz;
  int min_grain_tag;
  bool is_bonded=true;

  ARandomAssembly* CRB;

  // process args
  while(args_read<argc){
    string option=string(argv[args_read]);
    if(option=="-x"){
      if(argc>args_read){
        xmax=atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-y"){
      if(argc>args_read){
        ymax=atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-z"){
      if(argc>args_read){
        zmax=atof(argv[args_read+1]);
        is_3D=true;
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
   } else if(option=="-r"){
      if(argc>args_read){
        rmin=atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    }else if(option=="-t"){
      if(argc>args_read){
        ntry=atoi(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    }else if(option=="-o"){
      if(argc>args_read){
        filename=string(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    }else if(option=="-s"){
      if(argc>=args_read+2){
        calc_stat=true;
        stat_nbins=atoi(argv[args_read+1]);
        statfilename=string(argv[args_read+2]);
        args_read+=3;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-rs"){
      if(argc>args_read){
        seed_def=true;
        random_seed=atoi(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-tolerence"){
      if(argc>args_read){
        isGouge = true;
        tolerence = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-connTolerence"){
      if(argc>args_read){
        isGouge = true;
        connTolerence = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-gouge"){
      if(argc>args_read){
        isGouge = true;
        gougeSize = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-gougeMinRadius"){
      if(argc>args_read){
        isGouge = true;
        gougeMinRadius = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-gougeMaxRadius"){
      if(argc>args_read){
        isGouge = true;
        gougeMaxRadius = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-gougeTag"){
      if(argc>args_read){
        isGouge = true;
        gougeTag = atoi(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }      
    } else if(option=="-fault"){
      if(argc>args_read){
        isGouge = true;
        faultSize = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-faultMinRadius"){
      if(argc>args_read){
        isGouge = true;
        faultMinRadius = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }      
    } else if(option=="-faultMaxRadius"){
      if(argc>args_read){
        isGouge = true;
        faultMaxRadius = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-faultTag"){
      if(argc>args_read){
        isGouge = true;
        faultTag = atoi(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-minDrivingTag"){
      if(argc>args_read){
        isGouge = true;
        minDrivingTag = atoi(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-maxDrivingTag"){
      if(argc>args_read){
        isGouge = true;
        maxDrivingTag = atoi(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-distanceFromBBoxEdge"){
      if(argc>args_read){
        isGouge = true;
        distanceFromBBoxEdge = atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-vtkXmlFile"){
      if(argc>args_read){
	isGouge = true;
        vtkXmlFileName = argv[args_read+1];
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-tc"){
      if(argc>args_read){
        tag_centre=true;
        ctag=atoi(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      } 
    } else if(option=="-spy"){
      if(argc>args_read){
        split=true;
        split_dir=2;
        ysplit=atof(argv[args_read+1]);
	split_rough=false;
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-spz"){
      if(argc>args_read){
        split=true;
        split_dir=3;
        ysplit=atof(argv[args_read+1]);
	split_rough=false;
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-spy_rough"){
      if(argc>args_read){
        split=true;
        split_dir=2;
        ysplit=atof(argv[args_read+1]);
        args_read+=2;
	split_rough=true;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-spz_rough"){
      if(argc>args_read){
        split=true;
        split_dir=3;
        ysplit=atof(argv[args_read+1]);
	split_rough=true;
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-pad"){
      if(argc>args_read){
        is_padded=true;
        pad=atof(argv[args_read+1]);
        args_read+=2;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-ty"){
      if(argc>args_read+3){
        tag_edge_y=true;
        extag1=atoi(argv[args_read+1]);
        extag2=atoi(argv[args_read+2]);
        extag_d=atof(argv[args_read+3]);
        args_read+=4;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-tz"){
      if(argc>args_read+3){
        tag_edge_z=true;
        eztag1=atoi(argv[args_read+1]);
        eztag2=atoi(argv[args_read+2]);
        eztag_d=atof(argv[args_read+3]);
        args_read+=4;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-tsp"){
      if(argc>args_read+2){
        tag_split=true;
        sptag=atoi(argv[args_read+1]);
        sptag_d=atof(argv[args_read+2]);
        args_read+=3;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-cx"){
      circular_x=true;
      args_read++;
    } else if(option=="-nobond"){
      is_bonded=false;
      args_read++;
    } else if(option=="-granular"){
      if(argc>args_read+7){
	granular=true;
	sdx=atof(argv[args_read+1]); // seed density in x
	sdy=atof(argv[args_read+2]); // seed density in y
	sdz=atof(argv[args_read+3]); // seed density in x
	rdx=atof(argv[args_read+4]); // seed randomness in x
	rdy=atof(argv[args_read+5]); // in y
	rdz=atof(argv[args_read+6]); // in x
	min_grain_tag=atoi(argv[args_read+6]);
	args_read+=8;
      } else {
        cout << "missing argument" << endl;
      }
    } else if(option=="-l"){
      if(argc>args_read){
        layered=true;
        int nlayers=atoi(argv[args_read+1]);
        if(argc>args_read+1+nlayers){
          for(int i=0;i<nlayers;i++){
            layers.push_back(atof(argv[args_read+2+i]));
          }
          args_read+=2+nlayers;
        }else{
          cout << "missing argument" << endl;
        }
      } else {
        cout << "missing argument" << endl;
      }
    }else if(option=="-?"){
      cout << "Usage: gengeo -w WIDTH -h HIGHT -r R_MIN -t N_TRY -o OUTFILE [-s N_BINS STATFILE][-rs SEED][-l NL L1 L2 ...][-tc TAG]" << endl;
      exit(1);
    }else{ // option not recognized
      cout << "unknown option " << option << endl;
      cout << "Usage: gengeo -w WIDTH -h HIGHT -r R_MIN -t N_TRY -o OUTFILE [-s N_BINS STATFILE][-rs SEED][-l NL L1 L2 ...][-tc TAG]" << endl;
      args_read++;
      exit(1);
    }
  }

  if (isGouge) {
    try {
      srand(random_seed);
      BoolVector periodicDimensions(3, false);
      periodicDimensions[0] = circular_x;
      GougeBlockPrms prms =
        GougeBlockPrms(
          BoundingBox(Vec3(0, 0, 0), Vec3(xmax, ymax, zmax)),
          1.0,
          XZ,
          ParticleBlockPrms(faultSize, faultMinRadius, faultMaxRadius),
          ParticleBlockPrms(gougeSize, gougeMinRadius, gougeMaxRadius),
          periodicDimensions,
          ntry,
          tolerence,
          connTolerence
        );

      if(granular){ 
	// -- granular gouge --
	GranularGougeBlock3D gougeBlock(prms);
	gougeBlock.generate();
	gougeBlock.tagFaultParticles(faultTag);
	gougeBlock.generateGrains(sdx,sdy,sdz,rdx,rdy,rdz,min_grain_tag); 
	gougeBlock.createInteractionSet();
	gougeBlock.tagDrivingPlateParticles(minDrivingTag, maxDrivingTag, distanceFromBBoxEdge);
	gougeBlock.writeToFile(filename);
	if (vtkXmlFileName.size() > 0){
	  ParticleDataVisitor particleData;
	  gougeBlock.visitParticles(particleData);
	  gougeBlock.visitInteractions(particleData);
	  
	  VtkXmlWriter vtkWriter;
	  vtkWriter.setData(particleData);
	  vtkWriter.writeToFile(vtkXmlFileName);
	}
	
      } else {
	// -- single particle gouge --
	GougeBlock3D gougeBlock(prms);
	gougeBlock.generate();
	gougeBlock.tagFaultParticles(faultTag);
	gougeBlock.tagGougeParticles(gougeTag);
	gougeBlock.tagDrivingPlateParticles(minDrivingTag, maxDrivingTag, distanceFromBBoxEdge);
	gougeBlock.writeToFile(filename);
	if (vtkXmlFileName.size() > 0)
	  {
	    ParticleDataVisitor particleData;
	    gougeBlock.visitParticles(particleData);
	    gougeBlock.visitInteractions(particleData);
	    
	    VtkXmlWriter vtkWriter;
	    vtkWriter.setData(particleData);
	    vtkWriter.writeToFile(vtkXmlFileName);
	  }
      }
      return 0;
    } catch (std::runtime_error &e) {
      std::cerr << e.what() << std::endl;
    }
    return 1;
  } else if(is_3D){
    if(split){
      if(is_padded){
        CRB=new CPaddedBlock3D(0.0,xmax,0.0,ymax,0.0,zmax,rmin,1.0,ysplit,pad,split_dir,circular_x);
      } else {
        CRB=new CSplitBlock3D(0.0,xmax,0.0,ymax,0.0,zmax,rmin,1.0,ysplit,split_dir,circular_x,split_rough);
      }
    } else {
      CRB=new CRandomBlock3D(0.0,xmax,0.0,ymax,0.0,zmax,rmin,1.0,circular_x,is_bonded);
    }
  } else {
    if(layered){
      CRB=new CLayeredBlock2D(0.0,xmax,0.0,ymax,rmin,1.0);
      for(vector<double>::iterator iter=layers.begin();
          iter!=layers.end();
          iter++){
        ((CLayeredBlock2D*)(CRB))->addLayerBoundary(*iter);
      }
    } else if(split){
      CRB=new CSplitBlock2D(0.0,xmax,0.0,ymax,rmin,1.0,ysplit,circular_x);
    }else{
      CRB=new CRandomBlock2D(0.0,xmax,0.0,ymax,rmin,1.0,circular_x);
    }
  }
  if(!seed_def){
    struct timeval tv;
    gettimeofday(&tv,NULL);
    random_seed=tv.tv_usec;
  }

  cout << "random seed: " << random_seed << endl;
  CRB->generate(ntry,random_seed);

  if(tag_centre) CRB->tagParticleClosestTo(Vec3(xmax*0.5,ymax*0.5,0.0),ctag);

  if(tag_edge_y) CRB->tagEdgeY(extag1,extag2,extag_d);
  if(tag_edge_z) CRB->tagEdgeZ(eztag1,eztag2,eztag_d);

  cout << "tag_split ? " <<  tag_split << endl;

  if(tag_split) CRB->tagSplit(sptag,sptag_d);

  CRB->writeToGeoFile(filename);
  cout << "Porosity: " << CRB->calcPorosity() << endl;

  if(calc_stat){
    vector<pair<double,double> >stat=CRB->getSizeDistribution(stat_nbins);
    ofstream statfile(statfilename.c_str());
    for(vector<pair<double,double> >::iterator iter=stat.begin();
      iter!=stat.end();
      iter++){
      statfile << iter->first << " " << iter->second << endl;
    }
    statfile.close();
  }

  delete CRB;
  return 0;
}

