#ifndef __BASICINTERACTION_H
#define __BASICINTERACTION_H

//-- IO includes --
#include <iostream>

using std::ostream;
using std::endl;

/*!
  \class BasicInteraction
  \brief Class to represent the common part of a pair 
  interaction, i.e. the IDs of the particles and the 
  interaction tag

  \author Steffen Abe
*/
class BasicInteraction
{
 private:
  int m_p1,m_p2,m_tag;

 public:
  BasicInteraction(int,int,int tag=0);

  int first()const {return m_p1;};
  int second()const {return m_p2;};
  
  int getTag() const
  {
    return m_tag;
  }
  
  template <typename TmplVisitor>
  void visit(TmplVisitor &visitor) const
  {
    visitor.visitBasicInteraction(*this);
  }

  friend ostream& operator<<(ostream&,const BasicInteraction&);
  friend class BILess;
};

/*!
  \class BILess
  \brief function object for the ordering of BasicInteraction


  \author Steffen Abe
*/
class BILess
{
 public:
  bool operator()(const BasicInteraction&,const BasicInteraction&); 
};

#endif //__BASICINTERACTION_H
