/*=============================================================================

******************************************************************************
*                                               *
* COPYRIGHT ??????? - All Rights Reserved *
*  *
* This software is the property of ??????????????. No part of this code   *
* may be copied in any form or by any means without the expressed written   *
* consent of ???????????. Copying, use or modification of this software     *
* by any unauthorised person is illegal unless that    *
* person has a software license agreement with ?????????????.            *
*                                                          *
******************************************************************************

*********************************************************************************/

#ifndef ESYS_LSMBLOCKGENERATOR_H
#define ESYS_LSMBLOCKGENERATOR_H

#include <Geometry/ParticleGenerator.h>
#include <Geometry/SimpleParticle.h>
#include <Foundation/BoundingBox.h>

#include <vector>
#include <set>

namespace esys
{
  namespace lsm
  {
    typedef std::vector<bool> BoolVector;
    /**
     *
     */
    class BlockGenerator : public ParticleGenerator
    {
    public:
      BlockGenerator(
        NTable            &nTable,
        ParticlePool      &particlePool,
        const BoundingBox &bBox,
        const BoolVector  &periodicDimensions,
        double            tolerence
      );

      virtual ~BlockGenerator();

      virtual void generate() = 0;

      virtual void generateSeedParticles();

      virtual SimpleParticle generateParticle(const Vec3 &point);

      virtual double getRadius() const = 0;
      
      virtual double getGridRadius() const = 0;

      size_t getNumParticles() const;

      int getNextId();

      virtual bool particleFits(const SimpleParticle &particle) const;

      bool is2d() const;

      bool particleFitsInBBox(const SimpleParticle &particle) const;

      bool particleFitsWithNeighbours(const SimpleParticle &particle) const;

      void insertParticle(const SimpleParticle &particle);

      double getTolerence() const;

      const BoundingBox &getBBox() const;

      bool contains(const SimpleParticle &particle) const;

      typedef NTable::ParticleVector ParticleVector;
      typedef NTable::ParticleIterator ParticleIterator;
      
      ParticleIterator getParticleIterator()
      {
        return ParticleIterator(m_particleVector);
      }

      typedef std::set<int> IdSet;
    private:
      BoundingBox    m_bBox;
      BoolVector     m_periodicDimensions;
      ParticleVector m_particleVector;
      double         m_tolerence;
      IdSet          m_idSet;
    };
  };
};

#endif
