#ifndef __CORNER_H
#define __CORNER_H

//-- Project includes --
#include "Foundation/vec3.h"
#include "Geometry/Triangle.h"
#include "Geometry/Edge.h"

//-- STL includes --
#include <vector>

using std::vector;

/*!
  \class Corner
  \brief Class representing the corner of a polygon

  \author Steffen Abe
  $Revision$
  $Date$
*/
class Corner
{
 private:
  Vec3 m_p;
  vector<Edge*> m_edges;
  vector<Triangle*> m_triangles;

 public:
  Corner(const Vec3&);

  void addEdge(Edge*);
  void addTriangle(Triangle*);
  double sep(const Vec3&) const;
  pair<bool,double> dist(const Vec3&) const ; // signed separation according to direction of the normal
  bool isValidContact(const Vec3&) const;
  Vec3 getDirectionFromPoint(const Vec3&) const;
  void move(const Vec3&);
  Vec3 getPos()const {return m_p;};
};

#endif // __CORNER_H
