#ifndef __EDGE_H
#define __EDGE_H

//-- Project includes --
#include "Foundation/vec3.h"
#include "Geometry/Triangle.h"

//-- STL includes --
#include <utility>

using std::pair;
using std::make_pair;

/*!
  \class Edge
  \brief Class representing the edge of a polygon

  \author Steffen Abe
  $Revision$
  $Date$
*/
class Edge
{
 private:
  Vec3 m_p1,m_p2;
  Triangle *m_t1,*m_t2;
  int m_id1,m_id2;

 public:
  Edge(int,int,const Vec3&,const Vec3&);
  Edge(int,int,const Vec3&,const Vec3&,Triangle*);
  Edge(int,int,const Vec3&,const Vec3&,Triangle*,Triangle*);

  double sep(const Vec3&) const;
  pair<bool,double> dist(const Vec3&) const ; // signed separation according to direction of the normal
  bool isValidContact(const Vec3&) const;
  Vec3 getBoundingBoxMin() const; 
  Vec3 getBoundingBoxMax() const; 
  Vec3 getDirectionFromPoint(const Vec3&) const;
  void moveNode(int,const Vec3&);

  pair<int,int> getIDs() const {return make_pair(m_id1,m_id2);};

  friend ostream& operator << (ostream&,const Edge&);
};
#endif // __EDGE_H
