/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef ESYS_LSMGEOMETRYINFO_H
#define ESYS_LSMGEOMETRYINFO_H

#include "Foundation/vec3.h"

#include <vector>
#include <iostream>

namespace esys
{
  namespace lsm
  {
    typedef std::vector<bool> BoolVector;
    typedef std::vector<Vec3> Vec3Vector;
    typedef std::vector<int>  IntVector;

    /**
     * Container class for geometry meta-info.
     */
    class GeometryInfo
    {
    public:
      /**
       *
       */
      GeometryInfo();

      /**
       *
       */
      GeometryInfo(
        float            version,
        const Vec3       &bBoxMin,
        const Vec3       &bBoxMax,
        const BoolVector &periodicDimensions,
        bool             is2d = false
      );

      GeometryInfo(const GeometryInfo &geoInfo);

      GeometryInfo &operator=(const GeometryInfo &geoInfo);

      ~GeometryInfo();

      bool operator==(const GeometryInfo &geoInfo) const;

      /**
       * Sets the bounding box for geometry data.
       */
      void setBBox(const Vec3 &min, const Vec3 &max);

      /**
       * Returns true if any of the x, y or z dimensions
       * have been specified as periodic.
       */
      bool hasAnyPeriodicDimensions() const;

      /**
       * Returns true info indicates two-dimensional particle data.
       */
      bool is2d() const;
      
      /**
       * Returns two corner points of bounding box.
       */
      Vec3Vector getBBoxCorners() const;

      /**
       * Returns the periodic dimensions.
       */
      IntVector getPeriodicDimensions() const;
      
      /**
       * Parses specified istream and assigns to this object.
       */
      void read(std::istream &iStream);

      /**
       * Writes to specified istream in form parsable by the
       * GeometryInfo::read method.
       */
      void write(std::ostream &oStream) const;
      
    private:
      class Impl;
      
      Impl *m_pImpl;
    };
    
    std::ostream &operator<<(std::ostream &oStream, const GeometryInfo &geoInfo);
    std::istream &operator<<(std::istream &iStream, GeometryInfo &geoInfo);
  };
};

#endif
