#ifndef __PLANE_H
#define __PLANE_H

//-- Project includes --
#include "Foundation/vec3.h"

/*!
  \class Plane
  \brief Class representing a Plane

  \author Steffen Abe
  $Revision$
  $Date$

*/
class Plane
{
 protected:
  Vec3 U,V ;
  bool force3D ;
  Vec3 Dir ;  
  Vec3 Pos ;
  
  void Create() ;
 public:
  Plane();
  Plane(const Vec3& iDir,const Vec3& iPos);
  Plane(const Vec3& iU,const Vec3& iV,const Vec3& iPos);
    
  virtual double sep(const Vec3&) const;
  virtual double dist(const Vec3&) ; // signed separation according to Direction of the normal
  virtual Vec3 ToClosest(const Vec3& M) {return dist(M)*Dir; } ; // return the vector PM where P is the closest point to M on the surface.

  inline Vec3 GetU() const { return U; } ; // return U (for planes in a 2D space (ie. line)  U is in the space)
  inline Vec3 GetV() const { return V; } ; // V is null if this is a 2D space.
  inline const Vec3 &GetW() const { return Dir; } ; // The normal
  inline const Vec3 &GetO() const { return Pos; } ;
} ;


#endif // __PLANE_H
