#ifndef __RANDOMASSEMBLY3D_H
#define __RANDOMASSEMBLY3D_H

//-- project includes --
#include "Geometry/ARandomAssembly.h"
#include "Geometry/SimpleParticle.h"
#include "Geometry/BasicInteraction.h"
#include "Geometry/Plane.h"

//-- STL includes --
#include <vector>
#include <set>

using std::set;
using std::vector; 

/*!
  \class ARandomAssembly3D
  \brief Abstract base class for random assemblies, to be used for initialization of random lattices. 
    
  \author Steffen Abe
  $Revision$
  $Date$
*/
class ARandomAssembly3D : public ARandomAssembly
{
 protected:
  vector<Plane> Borders;
  double m_rmin,m_rmax;                             //!< min/max particle radius
  double m_xmin,m_xmax,m_ymin,m_ymax,m_zmin,m_zmax; //!< x,y,z borders of the lattice
  set<BasicInteraction,BILess> m_iset;
  bool m_circ_x;

  virtual Vec3 getAPoint()=0;

  bool findAFit(SimpleParticle&, const vector<SimpleParticle>&);
  bool findAFit(SimpleParticle&, const vector<SimpleParticle>&, const Plane&);
  bool checkAFit(const SimpleParticle&);
  Plane getClosestPlane(const SimpleParticle&);
  void fillSpace(int);
  virtual int getNParts() const=0;

 public:
  virtual void generate(int,unsigned int)=0;
  virtual void insertParticle(const SimpleParticle)=0;
  virtual void tagParticleClosestTo(const Vec3&,int)=0; 
  virtual void tagEdgeY(int,int,double)=0; 
  virtual void tagEdgeZ(int,int,double)=0; 
};

#endif // __RANDOMASSEMBLY3D_H
