/*=============================================================================

******************************************************************************
*                                               *
* COPYRIGHT ??????? - All Rights Reserved *
*  *
* This software is the property of ??????????????. No part of this code   *
* may be copied in any form or by any means without the expressed written   *
* consent of ???????????. Copying, use or modification of this software     *
* by any unauthorised person is illegal unless that    *
* person has a software license agreement with ?????????????.            *
*                                                          *
******************************************************************************

*********************************************************************************/

#ifndef SIMPLEPARTICLE_H
#define SIMPLEPARTICLE_H

#include <Foundation/vec3.h>

/**
 *
 */
class SimpleParticle
{
public:
  static const SimpleParticle INVALID;
  
  SimpleParticle(const Vec3&,double,int,int tag=0);
  SimpleParticle(const SimpleParticle&);

  inline const Vec3 &getPos() const {return m_pos;};
  inline double getRad() const {return m_rad;};
  inline int getID() const {return m_id;};
  inline int getTag() const {return m_tag;}
  
  inline void moveTo(Vec3 v){m_pos=v;}; //!< move absolute
  inline void setRad(double r){m_rad=r;};
  inline void setTag(int t){m_tag=t;};
  
  inline bool isValid() const {return (m_id >= 0);}
  
  template <typename TmplVisitor>
  void visit(const TmplVisitor &visitor) const
  {
    visitor.visitSimpleParticle(*this);
  }

  template <typename TmplVisitor>
  void visit(TmplVisitor &visitor)
  {
    visitor.visitSimpleParticle(*this);
  }

  friend ostream& operator<<(std::ostream &oStream, const SimpleParticle &particle);

protected:
  Vec3 m_pos;
  double m_rad;
  int m_id;
  int m_tag;
};

#endif
