/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef ESYS_LSMVTKXMLWRITER_H
#define ESYS_LSMVTKXMLWRITER_H

#include <Geometry/SimpleParticle.h>
#include <Geometry/BasicInteraction.h>

#include <iostream>
#include <boost/shared_ptr.hpp>

namespace esys
{
  namespace lsm
  {
    class ParticleDataVisitor
    {
      typedef SimpleParticle    Particle;
      typedef BasicInteraction  Interaction;
    public:
      ParticleDataVisitor();

      void visitSimpleParticle(const SimpleParticle &particle);

      void visitBasicInteraction(const BasicInteraction &interaction);

      size_t getNumParticles() const;

      size_t getNumInteractions() const;

      int getIndex(int particleId) const;

      void writeCentrePoints(std::ostream &oStream) const;

      void writeRadii(std::ostream &oStream) const;

      void writeTags(std::ostream &oStream) const;

      void writeIds(std::ostream &oStream) const;

      void writeParticleIndexInteractions(std::ostream &oStream) const;
      
      void writeInteractionTags(std::ostream &oStream) const;

    private:
      class Impl;
      typedef boost::shared_ptr<Impl> ImplPtr;
      ImplPtr m_implPtr;
    };

    /**
     *
     */
    class VtkXmlWriter
    {
    public:
      VtkXmlWriter();

      ~VtkXmlWriter();

      void setData(const ParticleDataVisitor &particleData);

      size_t getNumParticles() const;

      size_t getNumInteractions() const;
      
      virtual void writePoints(std::ostream &oStream);
      virtual void writePointData(std::ostream &oStream);
      virtual void writeCells(std::ostream &oStream);
      virtual void writeCellData(std::ostream &oStream);

      virtual void write(std::ostream &oStream);
      
      virtual void writeToFile(const std::string &fileName);

      private:

      class Impl;
      typedef boost::shared_ptr<Impl> ImplPtr;
      ImplPtr m_implPtr;
    };
  };
};

#endif
