/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "GeometryTest/CircularNeighbourTableTest.h"
#include "GeometryTest/TestParticle.h"

#include "Geometry/CircularNeighbourTable.h"

using namespace esys::lsm;

namespace LsmTest
{


  CPPUNIT_TEST_SUITE_REGISTRATION( CircularNeighbourTableTest );

  CircularNeighbourTableTest::CircularNeighbourTableTest()
      : CppUnit::TestCase()
  {
  }


  CircularNeighbourTableTest::~CircularNeighbourTableTest()
  {
  }


  void CircularNeighbourTableTest::setUp()
  {
    CppUnit::TestCase::setUp();
  }

  void CircularNeighbourTableTest::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }

  typedef CircularNeighbourTable<TestParticle> TestTable;
  
  void CircularNeighbourTableTest::testSingleCircularDimension()
  {
    TestTable::ParticlePool particlePool(1024);
    TestTable::BoolVector   circularDims(3, false);
    circularDims[0] = true;
    TestTable table = 
      TestTable(
        BoundingBox(Vec3(0, 0, 0), Vec3(30, 20, 10)),
        2.0,
        circularDims,
        particlePool
      );

    TestParticle p1 = TestParticle(Vec3(0, 1, 1), 0.5);
    table.insert(p1);
    
    TestTable::ParticleVector neighbours = table.getNeighbourVector(Vec3(1, 1, 1));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());

    neighbours = table.getNeighbourVector(Vec3(29, 1, 1));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());
    CPPUNIT_ASSERT_EQUAL(p1.getId(), neighbours[0]->getId());
    CPPUNIT_ASSERT_EQUAL(Vec3(30, 1, 1), neighbours[0]->getPos());
    
    TestParticle p2 = TestParticle(Vec3(29.5, 3, 1), 0.75);
    table.insert(&p2);
    
    neighbours = table.getNeighbourVector(Vec3(29, 3, 1));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());
    CPPUNIT_ASSERT_EQUAL(&p2, neighbours[0]);

    neighbours = table.getNeighbourVector(Vec3(0, 3, 1));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());
    CPPUNIT_ASSERT_EQUAL(p2.getId(), neighbours[0]->getId());
    CPPUNIT_ASSERT_EQUAL(Vec3(-0.5, 3, 1), neighbours[0]->getPos());
  }

  void CircularNeighbourTableTest::testGetNeighbours()
  {
    TestTable::ParticlePool particlePool(1024);
    TestTable::BoolVector   circularDims(3, false);
    circularDims[0] = true;
    TestTable table =
      TestTable(
        BoundingBox(Vec3(0, 0, 0), Vec3(20, 20, 20)),
        1.0,
        circularDims,
        particlePool,
        5.0
      );

      TestParticle p1 = TestParticle(Vec3(4.5, 4.5, 4.5), 0.25);
      table.insert(p1);

      TestTable::ParticleVector neighbours = table.getNeighbourVector(Vec3(24.5, 4.5, 4.5));
      CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());


      TestParticle p2 = TestParticle(Vec3(5.5, 6.5, 6.5), 0.20);
      table.insert(p2);
            
      neighbours = table.getNeighbourVector(Vec3(25.5, 6.5, 6.5));
      CPPUNIT_ASSERT_EQUAL((size_t)0, neighbours.size());      
  }
  
  void CircularNeighbourTableTest::testMultipleCircularDimension()
  {
    TestTable::ParticlePool particlePool(1024);
    TestTable::BoolVector   circularDims(3, false);
    circularDims[0] = true;
    circularDims[2] = true;
    TestTable table =
      TestTable(
        BoundingBox(Vec3(0, 0, 0), Vec3(30, 20, 10)),
        2.0,
        circularDims,
        particlePool
      );

    TestParticle p1 = TestParticle(Vec3(0, 1, 1), 0.5);
    table.insert(p1);
    
    TestTable::ParticleVector neighbours = table.getNeighbourVector(Vec3(1, 1, 1));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());
    
    neighbours = table.getNeighbourVector(Vec3(29, 1, 1));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());
    CPPUNIT_ASSERT_EQUAL(p1.getId(), neighbours[0]->getId());
    CPPUNIT_ASSERT_EQUAL(Vec3(30, 1, 1), neighbours[0]->getPos());
    
    neighbours = table.getNeighbourVector(Vec3(1, 1, 10));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());
    CPPUNIT_ASSERT_EQUAL(p1.getId(), neighbours[0]->getId());
    CPPUNIT_ASSERT_EQUAL(Vec3(0, 1, 11), neighbours[0]->getPos());
    
    neighbours = table.getNeighbourVector(Vec3(29, 1, 10));
    CPPUNIT_ASSERT_EQUAL((size_t)1, neighbours.size());
    CPPUNIT_ASSERT_EQUAL(p1.getId(), neighbours[0]->getId());
    CPPUNIT_ASSERT_EQUAL(Vec3(29, 1, 11), neighbours[0]->getPos());
  }
};
