/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef LSMTESTTESTPARTICLE_H
#define LSMTESTTESTPARTICLE_H

#include "Foundation/vec3.h"

namespace LsmTest
{

  /**
   *
   */
  class TestParticle
  {
  public:
    typedef int Id;
    TestParticle()
      : m_radius(0.0),
        m_pos(Vec3(0, 0, 0)),
        m_force(Vec3(0, 0, 0)),
        m_id(getNextId())
    {
    }

    TestParticle(const Vec3 &pos, double radius)
      : m_radius(radius),
        m_pos(pos),
        m_force(Vec3(0, 0, 0)),
        m_id(getNextId())
    {
    }

    bool operator==(const TestParticle &particle) const
    {
      return (getId() == particle.getId());
    }

    double getRad() const
    {
      return m_radius;
    }
    
    const Vec3 &getPos() const
    {
      return m_pos;
    }

    double getId() const
    {
      return m_id;
    }
    
    Vec3 getForce() const
    {
      return m_force;
    }
    
    void moveTo(const Vec3 &newPosition)
    {
      m_pos = newPosition;
    }

    void applyForce(const Vec3& f,const Vec3& p){
      m_force+=f;
    }

  protected:
    static Id currentId;
    static Id getNextId()
    {
      return ++currentId;
    }

  private:
    double m_radius;
    Vec3   m_pos;
    Id     m_id;
    Vec3   m_force;
  };
};

#endif
