/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

// --- test includes ---
#include "GeometryTest/TriangleTest.h"
#include "GeometryTest/TestParticle.h"

// --- project includes ---
#include "Geometry/Triangle.h"

// --- system includes ---
#include <cmath>
using std::sqrt;

//using namespace esys::lsm;

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( TriangleTest );
  
  /*! 
    constructor - do nothing
  */
  TriangleTest::TriangleTest() : CppUnit::TestCase()
  {}
  
  /*!
    destructor - do nothing
  */
  TriangleTest::~TriangleTest(){}

  /*!
   */
  void TriangleTest::setUp()
  {
    CppUnit::TestCase::setUp();
  }
  
  /*!
   */
  void TriangleTest::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }

  /*!
    test Triangle::dist
  */
  void TriangleTest::testDist()
  {
    Vec3 p1=Vec3(0.0,0.0,0.0);
    Vec3 p2=Vec3(1.0,0.0,1.0);
    Vec3 p3=Vec3(0.0,1.0,1.0);
    Vec3 pm=0.3*p1+0.4*p2+0.3*p3;
    Vec3 pm2=Vec3(1.0,1.0,0.0);
    Vec3 pm3=Vec3(0.0,0.0,1.0);
    Vec3 pm4=Vec3(1.0,1.0,-2.0);
    Triangle Tr1=Triangle(1,2,3,p1,p2,p3,0,0);
    pair<bool,double> res;
    res=Tr1.dist(pm);
    CPPUNIT_ASSERT_EQUAL(true,res.first);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,res.second,1e-10);
    res=Tr1.dist(pm2);
    CPPUNIT_ASSERT_EQUAL(true,res.first);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0*sqrt(1.0/3.0),res.second,1e-10);
    res=Tr1.dist(pm3);
    CPPUNIT_ASSERT_EQUAL(true,res.first);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.0*sqrt(1.0/3.0),res.second,1e-10);
    res=Tr1.dist(pm4);
    CPPUNIT_ASSERT_EQUAL(false,res.first);
  }
  
  /*!
    test Triangle::sep
  */
  void TriangleTest::testSep()
  {
    Vec3 p1=Vec3(0.0,0.0,0.0);
    Vec3 p2=Vec3(1.0,0.0,1.0);
    Vec3 p3=Vec3(0.0,1.0,1.0);
    Vec3 pm2=Vec3(1.0,1.0,0.0);
    Vec3 pm3=Vec3(0.0,0.0,1.0);
    Vec3 pm4=Vec3(1.0,3.0,-3.0);
    Triangle Tr1=Triangle(1,2,3,p1,p2,p3,0,0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0*sqrt(1.0/3.0),Tr1.sep(pm2),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0*sqrt(1.0/3.0),Tr1.sep(pm3),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(sqrt(19.0),Tr1.sep(pm4),1e-10);
  }
  
  /*!
    test Triangle::getBoundingBoxMin
  */
  void TriangleTest::testBBXMin()
  {
    Vec3 p1=Vec3(0.0,0.0,0.0);
    Vec3 p2=Vec3(1.0,0.0,1.0);
    Vec3 p3=Vec3(0.0,1.0,1.0);
    Triangle Tr1=Triangle(1,2,3,p1,p2,p3,0,0);
    Vec3 bbm=Tr1.getBoundingBoxMin();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,bbm.X(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,bbm.Y(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,bbm.Z(),1e-10);
  }

  /*!
    test Triangle::getBoundingBoxMax
  */
  void TriangleTest::testBBXMax()
  {
    Vec3 p1=Vec3(0.0,0.0,0.0);
    Vec3 p2=Vec3(1.0,0.0,1.0);
    Vec3 p3=Vec3(0.0,1.0,1.0);
    Triangle Tr1=Triangle(1,2,3,p1,p2,p3,0,0);
    Vec3 bbm=Tr1.getBoundingBoxMax();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,bbm.X(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,bbm.Y(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,bbm.Z(),1e-10);
  }

  /*!
    test Triangle::containsEdge
  */
  void TriangleTest::testContainsEdge()
  {
    Vec3 p1=Vec3(0.0,0.0,0.0);
    Vec3 p2=Vec3(1.0,0.0,1.0);
    Vec3 p3=Vec3(0.0,1.0,1.0);
    Triangle Tr1=Triangle(1,2,3,p1,p2,p3,0,0);
    CPPUNIT_ASSERT(Tr1.containsEdge(p1,p2));
    CPPUNIT_ASSERT(Tr1.containsEdge(p2,p1));
    CPPUNIT_ASSERT(!Tr1.containsEdge(p1,p1));
    CPPUNIT_ASSERT(!Tr1.containsEdge(p1,Vec3(2.0,0.0,0.0)));
  }

  /*!
    test Triangle::toGlobal
   */
  void TriangleTest::testToGlobal()
  {
    Vec3 p1=Vec3(0.0,0.0,2.0);
    Vec3 p2=Vec3(0.0,1.0,2.0);
    Vec3 p3=Vec3(1.0,0.0,2.0);
    Vec3 pt1=Vec3(0.5,0.5,1.0);
    Triangle Tr1=Triangle(1,2,3,p1,p2,p3,0,0);
    Vec3 p_tr=Tr1.toGlobal(pt1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5,p_tr.X(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5,p_tr.Y(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,p_tr.Z(),1e-10);
    Tr1.moveNode(2,Vec3(1.0,0.0,0.0));
    p_tr=Tr1.toGlobal(pt1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,p_tr.X(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5,p_tr.Y(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,p_tr.Z(),1e-10);  
  }
};
