#include "stdafx.h"
#include <stdlib.h>
#include "BackendCom.h"
#include "CPartRender.h"
#include "remoteConsole.h"
#include "RemoteData.h"
#include "RDFields.h"
#include <sys/time.h>
#include "vec3.h"

/***************************************************************************************/
/* Class CBackEndCom                                                                   */
/*                                                                                     */
/*                                                                                     */
/***************************************************************************************/

CBackEndCom::CBackEndCom() {
}

CBackEndCom::~CBackEndCom() {
}

bool CBackEndCom::ping() { // ping the back-end return true if OK.
  if (sock->client() && sock->client()->connected()) {
    console.Debug() << "Sending packet, waiting for answer...\n" ;
    sock->sendTag("PNG") ;
    if (sock->sendTag("PNG") != -1) {
      sock->waitFor("ACK") ;
      return true ;
    } else {
      return false ;
    }
  } else {
    return false ;
  }
}

bool CBackEndCom::startSim() { 
  if (sock->client() && sock->client()->connected()) {
     if (sock->sendTag("RUN") != -1) {
      sock->waitFor("ACK") ;
      return true ;
    } else {
      return false ;
    }
  } else {
    return false ;
  }
}

bool CBackEndCom::pauseSim() { 
  if (sock->client() && sock->client()->connected()) {
     if (sock->sendTag("PSE") != -1) {
      sock->waitFor("ACK") ;
      return true ;
    } else {
      return false ;
    }
  } else {
    return false ;
  }
}

bool CBackEndCom::shutdown() { // shutdown the back-end return true if OK.
  if (sock->client() && sock->client()->connected()) {
    sock->sendTag("XIT") ;
    sock->waitFor("ECK") ;
    sock->client()->close() ;
    return true ;
  } else {
    return false ;
  }
}

bool CBackEndCom::disconnect() { // disconect from the back-end return true if OK.
  if (sock->client() && sock->client()->connected()) {
    sock->sendTag("END") ;
    sock->waitFor("ECK") ;
    sock->client()->close() ;
    return true ;
  } else {
    return false ;
  }
} 

bool CBackEndCom::getAvailableFields(CRemoteData *data) {
  CShakePacket *pack; 
  unsigned long n ;
  int i ;
  char buffer[1024] ;
  unsigned long type,shape;

  if (sock->client() && sock->client()->connected()) {
    sock->sendTag("QFT") ;
    pack = sock->waitFor("HDT") ;
    Cxdr xdata((char *)(pack->getData()),pack->getSize()) ;
    xdata >> n ;
    if (n==0) {
      console.Warning() << "No data available, is the model initialized?\n" ;
    } else {
      pack = sock->waitFor("DDT") ;
      Cxdr fdata((char *)(pack->getData()),pack->getSize()) ;
      for (i=0; i<n; i++) {
	CFieldDescription *f = new CFieldDescription ;
	fdata >> &buffer[0] ;
	f->setName(string(buffer)) ;
	fdata >> type ;
	f->setType((ADataField::ADataType)type) ;
	// field shapes
	unsigned long nshapes;
	fdata >> nshapes;
	for(int j=0;j<int(nshapes);j++){
	  fdata >> shape;
	  f->addShape(ADataField::AFieldShape(shape));
	}
	data->addFieldDescription(*f) ;
      }
      sock->sendTag("DCK") ;
    }
    return true ;
  } else {
    console.Error() << "Not connected!\n" ;
    return false ;
  }  
}

#define S_TIME(e1,e2)  (double)(e1.tv_sec-e2.tv_sec)+1e-6*(double)(e1.tv_usec-e2.tv_usec)

bool CBackEndCom::getParticlesData(CRemoteData *data) 
{
  CShakePacket *pack; 
  unsigned long n ;
  double f ;
  int j,in ;
  struct timeval time1,time2,time3,time4;
  char buffer[1024];

  gettimeofday(&time1,NULL) ;
  console.XDebug() << "get Particle Data..\n" ;
  if (sock->client() && sock->client()->connected()) {
    sock->sendTag("GDT") ;
    pack = sock->waitFor("DTA") ;
    Cxdr xdata((char *)(pack->getData()),pack->getSize()) ;
    tconsole.XDebug() << "got DTA size : " << pack->getSize() << "\n" ;

    xdata >> n ;

    if (n==0) {
      console.Error() << "No data available, is the model initialized?\n" ;
      return true ;
    }

    gettimeofday(&time2,NULL) ;
    tconsole.XDebug() << "Time for setup " << S_TIME(time2,time1) << "s. \n" ;
    
    in=0 ; 
    tconsole.XDebug() << "wait DDA #" << in << "\n" ;
    pack = sock->waitFor("DDA") ;
    Cxdr pdata((char *)(pack->getData()),pack->getSize()) ;
    tconsole.XDebug() << "got DDA #" << in << "\n" ;
    long nbpart ;
	
    gettimeofday(&time3,NULL) ;
    tconsole.XDebug() << "Time for receiving " << S_TIME(time3,time2) << "s. \n" ;
	
    pdata >> &buffer[0]; // particle type;
    string ptype=string(buffer);
    tconsole.XDebug()<< "particle type is : " << ptype.c_str() << "\n";

    pdata >> nbpart ;
 
    if(ptype=="Basic"){
      for (j=0; j<nbpart; j++) {
	Vec3 Pos;
	double rad;
	signed long  id;
	
	pdata >> f ;
	Pos.X()=f;
	pdata >> f ;
	Pos.Y()=f;
	pdata >> f ;
	Pos.Z() = f ;
	pdata >> rad ;
	pdata >> id ;
	data->addParticle(Pos,rad,int(id));
	in++ ;
      }

    } else if(ptype=="SPC"){
      for (j=0; j<nbpart; j++) {
	unsigned long is2D;

	SPRenderParticle crp;
	crp.extractFromPacket(pdata);
	data->addParticle(crp,j);
      }
    } else {
      tconsole.Critical() << "got message containing particles of unknown type\n";
    } 
    gettimeofday(&time4,NULL) ;
    tconsole.XDebug() << "Time for unpacking " << S_TIME(time4,time3) << "s. \n" ;
   

    return true ;
  } else {
    console.Error() << "Not connected!\n" ;
    return false ;
  }  
}

/*!
  get the data of a field from  the BE 

  \param field the field
*/
bool CBackEndCom::getFieldData(ADataField *field)
{
  console.XDebug()<<"CBackEndCom::getFieldData\n";

  CShakePacket *pack;
  bool ret;
  unsigned long n;

  if (sock->client() && sock->client()->connected()) {
    Cxdr xdata ;
    xdata << field->getName().c_str();
    xdata << (unsigned long)(field->getType());
    CShakePacket GFD(xdata,"GFD");  
    sock->socket()->send(GFD);


    pack = sock->waitFor("DTA") ;
    Cxdr ndata((char *)(pack->getData()),pack->getSize()) ;

    ndata >> n ;

    if (n==0) {
      console.Error() << "No data available, is the model initialized?\n" ;
      return true ;
    }

    int j;
    unsigned long idx; 
    double f;

    pack = sock->waitFor("DDA") ;
    Cxdr fdata((char *)(pack->getData()),pack->getSize()) ;
    long nbpart ;
    fdata >> nbpart ;
    for (j=0; j<nbpart; j++) {
      switch(field->getType()){
      case ADataField::FLD_SCALAR:
	fdata >> f ;
	fdata >> idx ;
	field->addScalar(idx,f);
	break;
      case ADataField::FLD_VECTOR:
	Vec3 V;
	fdata >> f ;
	V.X()=f;
	fdata >> f ;
	V.Y()=f;
	fdata >> f ;
	V.Z() = f ;
	fdata >> idx;
	field->addVector(idx,V);
	break;
      }
    }


    ret=true;
  } else {
    console.Error() << "Not connected!\n" ;
    ret=false ;
  }
  return ret;
}
