#ifndef __CPARTRENDER_H
#define __CPARTRENDER_H

// -- project includes --
#include "shake_math.h"
#include "shake_xdr.h"

// -- OpenGL includes --
#include <GL/gl.h>
#include <GL/glu.h>

// -- STL includes --
#include <vector>

struct rspatch
{
  Vec3 center;
  Vec3 normal;
  double radius;
  double angle;
};

using std::vector;

/*!
  \class SPRenderParticle 
  \brief class for rendering a spherical patch particle

  \author Steffen Abe
  $Revision$
  $Date$
*/
class SPRenderParticle 
{
 private:
  vector<rspatch> m_patches;
  Colour m_col;
  bool m_is2D;
  Vec3 m_center;

  void drawPatch(const Vec3&,double,const Vec3&,double,const Colour&);
  void drawPatch2D(const Vec3&,double,const Vec3&,double,const Colour&);
  
 public:
  SPRenderParticle();
  void addPatch(const Vec3&,double,const Vec3&,double);
  void addColor(const Colour&);
  void draw();  
  void scale(double);
  Vec3 minPos() const;
  Vec3 maxPos() const;

  void extractFromPacket(Cxdr&);
};

#endif //__CPARTRENDER_H
