#ifndef __COLORMAP_H
#define __COLORMAP_H

// --- project includes ---
#include "shake_math.h" // for Colour

// --- STL includes ---
#include <string>

using std::string;

/*!
  \class AColorMap 
  \brief Abstract base class for a colormap 

  \author Steffen Abe
  $Revision$
  $Date$
*/
class AColorMap
{
 public:
  //! boundary conditions: periodic, constant, transparent
  enum Bcon {BPeriodic, BConst, BTrans}; 

 private:
  string m_name;

 protected:
  Bcon m_boundary;

 public:
  AColorMap();
  AColorMap(const string&);
  AColorMap(const AColorMap&);
  virtual ~AColorMap()=0;

  AColorMap& operator=(const AColorMap&);

  virtual Colour getColor(double) const=0; // get color for a scalar value
  virtual void SaveAs(const string&) const;
  void SetBcon(Bcon bc){m_boundary=bc;}; //! set boundary condition
  virtual double getLow()const=0;
  virtual double getHigh()const=0;
};
#endif //__COLORMAP_H
