/*--------------------------------------------------------------------------
 * GTK display for shake++
 * D. Place
 *-------------------------------------------------------------------------*/ 

#ifndef _GTKDISPLAY_H
#define _GTKDISPLAY_H

#include "ShakeDisplay.h"
#include "BackendCom.h"
#include "remoteConsole.h"
#include <gtk/gtk.h>
#include "GtkPackage.h"
#include "GtkCWindow.h"
#include "GtkDisplayCon.h"
#include "GtkScriptEditor.h"
#include "t_list.h"

/***************************************************************************************/
/* Class CDisplayGTK                                                                   */
/*                                                                                     */
/*                                                                                     */
/***************************************************************************************/

class CFileDialogGTK  ;
class CConnectGTK  ;
class CRendererGTK  ;
class CAboutGTK  ;
class CMessageGTK  ;
class CPropertiesGTK  ;

class CDisplayGTK : public CWindowGTK, public CShakeDisplay, public TPackageGTK<CDisplayGTK>
{
 private:

  // Data for menu...
  enum SENSITIVITY {DISK_SYNCHRONIZED,CONNECTED,DISCONNECTED,PARSED,EMPTYED,MODIFIED} ;
  struct {
    bool m_connected,m_parsed,m_diskSynchonized,m_modified,m_emptyed ;
  } state ;
  static GtkItemFactoryEntry m_menu_items[] ;
  GtkItemFactory *m_item_factory;

  void getMainMenu( GtkWidget  *window, GtkWidget **menubar) ;
  void setSensitivity(SENSITIVITY s) ;
  void updateMenuSensitivity() ;

 protected:
  guint m_tag ;

  // The interfaces for console, editor and connection
  CDisplayConsoleGTK m_ConDsp ;
  CBackEndCom m_backend ;
  CDisplayEditorGTK m_editor ;

  // The dialogs...
  CFileDialogGTK *m_FileDialogWindow ;
  CConnectGTK *m_connectDLG ;
  CAboutGTK *m_aboutDLG ;
  CMessageGTK *m_messageDLG ;
  CPropertiesGTK *m_propertiesDLG ;

  // callbacks from the menus
  void newCB() ;
  void openCB() ;
  void saveCB() ;
  void saveasCB() ;
  void parseCB() ;
  void pingCB() ;
  void simStartCB() ;
  void simPauseCB() ;
  void propertiesCB() ;
  void aboutCB() ;
  void shutdownCB() ;
  void openRendererCB() ;
  void closeRendererCB() ;
  void stereoCB() ;
  void evalmainCB() ;
  void remoteEvalCB() ;
  void connectCB() ;
  void disconnectCB() ;
  void quitCB() ;

  // callbacks from dialogs
  void quitOK2() ;
  void quitOK() ;
  void newOK() ;
  void openOK() ;
  void openOK2() ;
  void saveAsOK() ;
  void connectOK() ;
  void shutdownOK() ;

  // misc.
  void disconnected() ;

  void timeoutCB() ;
  void start_timer() ;
  void stop_timer() ;
 public:

  CDisplayGTK(char *filename, int argc, char **argv) ;
  ~CDisplayGTK() ;

  CBackEndCom & backEnd() { return m_backend ; } ;
  virtual int exec() ;    // Main event loop
  virtual void create() ; // Create window & widgets...

  void scriptModified() ; // called by Editor class (CDisplayEditorGTK) when script is modified.
};

#endif 

