#include "GtkRenderer.h"
#include "gtkglarea.h"
#include "RemoteData.h"
#include <GL/gl.h>
#include <gtk/gtkvscale.h>
#include <sys/time.h>

GtkItemFactoryEntry CRendererGTK::m_menu_items[] = {
  { "/_Renderer",         NULL,         NULL, 0, "<Branch>" },
  { "/Renderer/_Update",    "<control>U", (void (*) ())(CRendererGTK::menuCB_vv), 0, NULL },
  { "/Renderer/_Auto Update",    "<control>A", (void (*) ())(CRendererGTK::menuCB_vv), 0, NULL },
  { "/Renderer/_Close",     "<control>Q", (void (*) ())(CRendererGTK::menuCB_vv), 0, NULL },
  { "/_Fields",         NULL,         NULL, 0, "<Branch>" },
  { "/Fields/_Select",     NULL,         (void (*) ())(CRendererGTK::menuCB_vv), 0, NULL },
  { "/Fields/_Transform",     NULL,         (void (*) ())(CRendererGTK::menuCB_vv), 0, NULL },
  { "/_Print",         NULL,         NULL, 0, "<Branch>" },
  { "/Print/_PostScript",     NULL,         (void (*) ())(CRendererGTK::menuCB_vv), 0, NULL }
} ;

CRendererGTK::~CRendererGTK()  {
  if (m_fieldWindow!=0)m_fieldWindow->close(); 
  if (m_renderer!=0) delete m_renderer;
  destroy() ;
}

CRendererGTK::CRendererGTK(CDisplayGTK *parent, int id) : CShakeView(parent,id), 
							  CWindowGTK(), 
							  TPackageGTK<CRendererGTK>()  
{
  m_renderer=new CShakeRenderer();
  m_parent = parent ;
  m_backend = &(parent->backEnd()) ;
  m_remoteData.setBackend(m_backend);
  m_fieldWindow=NULL;

  int i ;
  int nmenu_items ;
  nmenu_items = sizeof (m_menu_items) / sizeof (m_menu_items[0]);

  for (i=0; i<nmenu_items; i++) {
    if (!strcmp(m_menu_items[i].path,"/Renderer/_Update")){  
      m_menu_items[i].callback_action = call_int(&CRendererGTK::updateCB) ;
    } else if (!strcmp(m_menu_items[i].path,"/Renderer/_Auto Update")){
      m_menu_items[i].callback_action = call_int(&CRendererGTK::autoupdateCB) ;
    } else if (!strcmp(m_menu_items[i].path,"/Renderer/_Close")){
      m_menu_items[i].callback_action = call_int(&CRendererGTK::closeCB) ;
    } else if (!strcmp(m_menu_items[i].path,"/Fields/_Select")){
      m_menu_items[i].callback_action = call_int(&CRendererGTK::field_selectCB) ;
    } else if (!strcmp(m_menu_items[i].path,"/Print/_PostScript")){
      m_menu_items[i].callback_action = call_int(&CRendererGTK::printPSCB) ;
    }
  }
  m_tag = 0 ;
  m_autoUpdate = false ;
}

void CRendererGTK::updateCB() 
{
  update_particles();
  update_fields();
  update_view();
}

void CRendererGTK::printPSCB() 
{
  printPS("output.ps");
}

void CRendererGTK::autoupdateCB() 
{
  m_autoUpdate = ! m_autoUpdate ;
  if (m_autoUpdate) {
    console.Info() << "Auto update is on.\n" ;
    m_tag = gtk_timeout_add(1000,&CRendererGTK::signalCB_iv,call(&CRendererGTK::timeoutCB)) ;
  } else {
    console.Info() << "Auto update is off.\n" ;
    if (m_tag) { gtk_timeout_remove(m_tag); m_tag=0 ; }
  }
}

#define S_TIME(e1,e2)  (double)(e1.tv_sec-e2.tv_sec)+1e-6*(double)(e1.tv_usec-e2.tv_usec)

void CRendererGTK::timeoutCB() {
  console.Info() << "Updating...\n" ;
  
  if (m_tag) { gtk_timeout_remove(m_tag); m_tag=0 ; }
  struct timeval time1,time2,time3 ;
  gettimeofday(&time1,NULL) ;
  update_particles();
  update_fields();
  gettimeofday(&time2,NULL) ;
  update_view();  
  gettimeofday(&time3,NULL) ;
  console.Debug() << "Time for update :" << S_TIME(time2,time1) << "s.\n" ;
  console.Debug() << "Time for view :" << S_TIME(time3,time2) << "s.\n" ;
  if (m_autoUpdate) m_tag = gtk_timeout_add(1000,&CRendererGTK::signalCB_iv,call(&CRendererGTK::timeoutCB)) ;
}

void CRendererGTK::field_selectCB()
{
  console.Debug()<<"CRendererGTK::field_selectCB()\n";

  if(m_fieldWindow==NULL){
    m_fieldWindow=new CFieldWindowGTK(this);
    m_fieldWindow->create();
    m_fieldWindow->show();
  }
}

void CRendererGTK::close() {
  if (m_fieldWindow!=0)m_fieldWindow->close(); 
  destroy() ;
}

void CRendererGTK::create() {
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *handlebox1;
  GtkWidget *menubar1;
  GtkWidget *hbox1;
  GtkWidget *scrolledwindow1;
  GtkWidget *viewport1;
  GtkWidget *drawingarea1;
  GtkWidget *vscale1;
  GtkWidget *statusbar1;

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  m_window = window1 ;
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), "Renderer");
  gtk_window_set_default_size (GTK_WINDOW (window1), 450, 450);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);

  handlebox1 = gtk_handle_box_new ();
  gtk_widget_ref (handlebox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "handlebox1", handlebox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (handlebox1);
  gtk_box_pack_start (GTK_BOX (vbox1), handlebox1, FALSE, TRUE, 0);

  get_main_menu (window1, &menubar1);
  //menubar1 = gtk_menu_bar_new ();
  //gtk_widget_ref (menubar1);
  //gtk_object_set_data_full (GTK_OBJECT (window1), "menubar1", menubar1,
  //                          (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menubar1);
  gtk_container_add (GTK_CONTAINER (handlebox1), menubar1);
  //gnome_app_fill_menu (GTK_MENU_SHELL (menubar1), menubar1_uiinfo,
  //                     NULL, FALSE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (hbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_NEVER);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_ref (viewport1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "viewport1", viewport1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);
  gtk_container_set_border_width (GTK_CONTAINER (viewport1), 3);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport1), GTK_SHADOW_ETCHED_OUT);

  int attrlist[] = {
    GDK_GL_RGBA,
    GDK_GL_RED_SIZE,4,
    GDK_GL_GREEN_SIZE,4,
    GDK_GL_BLUE_SIZE,4,
    GDK_GL_DEPTH_SIZE,4,
    GDK_GL_DOUBLEBUFFER,
    GDK_GL_STEREO,
    GDK_GL_NONE
  };
  if (!m_renderer->stereo) {
    attrlist[10] = GDK_GL_NONE ;
  }

  //GTK_WIDGET(gtk_gl_area_new(attrlist));
  drawingarea1 = gtk_gl_area_new(attrlist) ; //gtk_drawing_area_new ();
  if (drawingarea1 == NULL) {
    console.Error() << "Cannot open display\n" ;
    m_theDisplay->rendererHasClosed(m_id) ;
    close();
    return ;
  }
  gtk_widget_ref (drawingarea1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "drawingarea1", drawingarea1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (drawingarea1);
  gtk_container_add (GTK_CONTAINER (viewport1), drawingarea1);

  GtkObject *adj ;
  adj = gtk_adjustment_new (0, -500, 1010, 1, 100, 100) ;
  vscale1 = gtk_vscale_new (GTK_ADJUSTMENT (adj));
  gtk_widget_ref (vscale1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vscale1", vscale1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vscale1);
  gtk_box_pack_start (GTK_BOX (hbox1), vscale1, FALSE, TRUE, 2);
  gtk_scale_set_draw_value (GTK_SCALE (vscale1), FALSE);
  gtk_scale_set_digits (GTK_SCALE (vscale1), 0);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_ref (statusbar1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "statusbar1", statusbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (statusbar1), 2);

  m_glarea = drawingarea1 ;

  gtk_widget_set_events(GTK_WIDGET(m_glarea),
                        GDK_EXPOSURE_MASK|
                        GDK_BUTTON_PRESS_MASK|
			GDK_BUTTON_RELEASE_MASK|
			GDK_POINTER_MOTION_MASK|
                        GDK_POINTER_MOTION_HINT_MASK);

  gtk_signal_connect (GTK_OBJECT (m_window), "destroy",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vv), call(&CRendererGTK::quitOK)) ;
  gtk_signal_connect (GTK_OBJECT (m_glarea), "expose_event",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vvv),call(&CRendererGTK::drawCB)) ;
  gtk_signal_connect (GTK_OBJECT (m_glarea), "configure_event",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vvv),call(&CRendererGTK::reshapeCB)) ;
  gtk_signal_connect (GTK_OBJECT (m_glarea), "realize",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vv), call(&CRendererGTK::initCB)) ;
  gtk_signal_connect (GTK_OBJECT (m_glarea), "button_release_event",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vvv),call(&CRendererGTK::inputCB,(void *)BTN_RELEASE)) ;
  gtk_signal_connect (GTK_OBJECT (m_glarea), "button_press_event",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vvv),call(&CRendererGTK::inputCB,(void *)BTN_PRESS)) ;
  gtk_signal_connect (GTK_OBJECT (m_glarea), "motion_notify_event",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vvv),call(&CRendererGTK::inputCB,(void *)MOUSE_MOVE)) ;
  gtk_signal_connect (GTK_OBJECT (adj),"value_changed",
		      GTK_SIGNAL_FUNC (&CRendererGTK::signalCB_vv), call(&CRendererGTK::vscaleCB)) ;
  m_button = 0 ;
}

void CRendererGTK::repaint()
{
  if (gtk_gl_area_make_current(GTK_GL_AREA(m_glarea))){
    m_renderer->paintGLArea() ;
    gtk_gl_area_swap_buffers(GTK_GL_AREA(m_glarea));
  }
}

void CRendererGTK::vscaleCB()
{
  console.Debug() << "Value changed to " << GTK_ADJUSTMENT(getCBargv(0))->value << "\n" ;
  m_renderer->setTransZ(-(float)(GTK_ADJUSTMENT(getCBargv(0))->value) / 100.0 );
  repaint() ;
}

void CRendererGTK::drawCB()
{
  GdkEventExpose *event = (GdkEventExpose *)getCBargv(1) ;
  /* Draw only last expose. */
  if (event->count > 0)
    return ;

  repaint() ;
}

void CRendererGTK::initCB()
{
  /* OpenGL functions can be called only if make_current returns true */
  if (gtk_gl_area_make_current(GTK_GL_AREA(m_glarea))){
    m_renderer->initializeGLArea() ;
  }
}

void CRendererGTK::reshapeCB()
{
  if (gtk_gl_area_make_current(GTK_GL_AREA(m_glarea))){
    m_renderer->resizeGLArea(m_glarea->allocation.width, m_glarea->allocation.height) ; 
  }
}

void CRendererGTK::inputCB() {
  int eventType; 
  GdkModifierType state ;
  GdkEventMotion* event ;
  GdkEventButton* event_bp ;

  eventType = (int)(getCBargv(2)) ;
  switch (eventType) {
  case BTN_PRESS:
    event_bp = (GdkEventButton*)(getCBargv(1)) ;
    m_button = event_bp->button ;
    m_lastx = event_bp->x ; 
    m_lasty = event_bp->y ;
    break ;
  case BTN_RELEASE:
    m_button = 0 ;
    break ;
  case MOUSE_MOVE:
    event = (GdkEventMotion*)(getCBargv(1)) ;
    if (event->is_hint) {
      gdk_window_get_pointer(event->window, &m_mx, &m_my, &state);
    } else {
      m_mx = event->x;
      m_my = event->y;
      state = (GdkModifierType)event->state ;
    }
    switch (m_button) {
    case 1:
      switch (m_rotationType) {
      default:
	m_renderer->addRotY((m_mx - m_lastx)/2) ;
	m_renderer->addRotX((m_my - m_lasty)/2) ;
	break ;
      case ROTZX:
	m_renderer->addRotZ((m_mx - m_lastx)/2) ;
	m_renderer->addRotX((m_my - m_lasty)/2) ;
	break ;
      case ROTZY:
	m_renderer->addRotZ((m_mx - m_lastx)/2) ;
	m_renderer->addRotY((m_my - m_lasty)/2) ;
	break ;
      }
      m_lastx = m_mx ;
      m_lasty = m_my ;
      repaint() ;
      break ;
    case 2:
      break ;
    case 3:
      m_renderer->addTransX((float)((m_mx - m_lastx)/100.0)) ;
      m_renderer->addTransY((float)((m_lasty - m_my)/100.0)) ;
      m_lastx = m_mx ;
      m_lasty = m_my ;
      repaint() ;
      break ;
    default:
      break ;
    }
    break ;
  default:
    break;
  }
}



/*!
  Close the renderer and notify the display that the renderer has been closed.
*/void CRendererGTK::closeCB()
{
  console.Debug() << "renderer #"<< m_id << " closing\n" ;
  m_theDisplay->rendererHasClosed(m_id) ;
  close();
}

void CRendererGTK::quitOK()
{
  console.Debug() << "Closing renderer #"<< m_id << "\n" ;
  m_theDisplay->rendererHasClosed(m_id) ;
}

void CRendererGTK::get_main_menu( GtkWidget  *window, GtkWidget **menubar )
{
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (m_menu_items) / sizeof (m_menu_items[0]);

  accel_group = gtk_accel_group_new ();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
     or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
     the accelerator table while generating menus.
  */

  m_item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<renderer>",
				       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (m_item_factory, nmenu_items, m_menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  if (menubar)
    /* Finally, return the actual menu bar created by the item factory. */
    *menubar = gtk_item_factory_get_widget (m_item_factory, "<renderer>");
}
