#ifndef __LINEARCOLORMAP_H
#define __LINEARCOLORMAP_H

// --- project includes ---
#include "ColorMap.h"

// --- STL includes ---
#include <map>

using std::map;

/*!
  \class CLinearColorMap 
  \brief Class for a colormap with piecewise linear mapping. An arbitrary number of node points can be specified at which a color is defined. Colors between node points are determined by linear intepolation between the two nearest node points. Colors outside the defined node points are either set to the nearest node point, is set the colormap is periodically repeated or are set transparent, depending on the value of m_boundary \sa AColorMap.  

  \author Steffen Abe
  $Revision$
  $Date$
*/
class CLinearColorMap : public AColorMap
{
 protected:
  map<double,Colour> m_data;

 public:
  CLinearColorMap();
  CLinearColorMap(const string&);
  CLinearColorMap(const CLinearColorMap&);
  virtual ~CLinearColorMap();

  CLinearColorMap& operator=(const CLinearColorMap&);

  virtual Colour getColor(double) const;// get color for a scalar value
  virtual void SaveAs(const string&) const;

  virtual void addNode(double,const Colour&);
  virtual void deleteNode(int);
  virtual void setNodePos(int,double);
  virtual void setNodeColor(int,const Colour&);
  virtual double getLow()const;
  virtual double getHigh()const;
};

#endif //__LINEARCOLORMAP_H
