#ifndef _RDFIELD_H
#define _RDFIELD_H

// -- project includes --
#include "shake_math.h"
#include "console.h"
#include "ShakeRenderer.h"
#include "ColorMap.h"

// -- STL includes  --
#include <map>
#include <string>

using std::map;
using std::string;


/*!
  \class ADataField
  \brief Abstract class for a data field

  \author David Place, Steffen Abe
  $Revision$
  $Date$
*/
class ADataField  
{
 public:
  enum ADataType { FLD_OTHER, FLD_SCALAR, FLD_VECTOR} ;
  enum AFieldShape { FLD_PARTICLE, FLD_CONN, FLD_FREE }; 

 private:
  bool m_isInitialized ;     //!< flag indicating if initialized

  string m_fieldName;        //!< name of field
  ADataType m_dataType;      //!< Type of field (ie. scalar or vector)
  AFieldShape m_fieldShape;  //!< Shape of the field 

 protected:
  void setShape(AFieldShape afs) {m_fieldShape=afs;};     //!< Set shape of field 

  AColorMap *m_colormap;
  bool m_autoscale;

 public:
  int m_Id ;                             //!< a unique Id number
  virtual int getSize() const =0;        //!< Return size of field (m_tsize)

  ADataField(ADataType,const string&);   //!< constructor 
  virtual ~ADataField() ;                //!< destructor

  virtual void clear()=0; 
  void setName(const string&) ;          //!< Set the name of field
  string getName() const;                //!< Return name of field
  
  inline ADataType getType(){return m_dataType;} ;      //!< Return type of field (ie. scalar or vector)
  inline AFieldShape getShape(){return m_fieldShape;} ; //!< Return shape of field (ie. scalar or vector)

  virtual void addScalar(int,double){};
  virtual void addVector(int,const Vec3&){};

  //! draw field into ShakeRenderer
  virtual void draw(CShakeRenderer *)=0;
} ;

/*!
  \class AScalarField
  \brief Abstract class for a scalar field

  \author Steffen Abe
  $Revision$
  $Date$
*/
class AScalarField : public ADataField
{
 protected:
  map<int,double> m_data;

 public:
  AScalarField(const string&);
  
  virtual void clear();
  virtual int getSize() const {return  m_data.size();};
  virtual void addScalar(int i,double d){m_data[i]=d;};

  inline double operator[](int i){return m_data[i];};
}; 

/*!
  \class AVectorField
  \brief Abstract class for a vector field

  \author Steffen Abe
  $Revision$
  $Date$
*/
class AVectorField : public ADataField
{
 protected:
  map<int,Vec3> m_data;

 public:
  AVectorField(const string&);
  
  virtual void clear();
  virtual int getSize() const {return  m_data.size();};
  virtual void addVector(int,const Vec3&);

  inline Vec3 operator[](int i) {return m_data[i];};
}; 

#endif //_RDFIELD_H
