#include "RDParticleField.h"
#include "LinearColorMap.h"

CScalarParticleField::CScalarParticleField(const string& name):AParticleField(),
							       AScalarField(name)
{
   setShape(ADataField::FLD_PARTICLE); 
   m_colormap=new CLinearColorMap("testcolormap"); // testing
}

CScalarParticleField::~CScalarParticleField()
{
  if(m_colormap!=NULL) delete m_colormap;
}

void CScalarParticleField::draw(CShakeRenderer * rend) 
{
  console.XDebug() << "CScalarParticleField::draw\n";

  if(m_autoscale){
    double min_data,max_data,min_cm,max_cm; 
    // get min/max
    min_data=m_data[0];
    max_data=m_data[0];
    for(int i=1;i<m_data.size();i++){
      max_data=(m_data[i]>max_data) ? m_data[i] : max_data;
      min_data=(m_data[i]<min_data) ? m_data[i] : min_data;
    }
    min_cm=m_colormap->getLow();
    max_cm=m_colormap->getHigh();
    if(min_data==max_data){
      for(int i=0;i<m_data.size();i++){
	Colour c=m_colormap->getColor(min_cm);
	rend->addColorToParticle(i,c);
      }
    }else {
      // draw scaled
      for(int i=0;i<m_data.size();i++){
	double d=m_data[i]; // raw data
	double d_scaled=(d-min_data)/(max_data-min_data); // scaled to [0,1]
	double c_scaled=min_cm+d_scaled*(max_cm-min_cm);  // scaled to map

	Colour c=m_colormap->getColor(c_scaled);
	rend->addColorToParticle(i,c);
      }
    }
  } else { // draw unscaled
    for(int i=0;i<m_data.size();i++){
      Colour c=m_colormap->getColor(m_data[i]);
      rend->addColorToParticle(i,c);
    }
  }
}
