#ifndef _REMOTEDATA_H_
#define _REMOTEDATA_H_

// -- Project includes --
#include "BackendCom.h"
#include "RDFields.h"
#include "CPartRender.h"

// -- STL includes --
#include <vector>
#include <list>
#include <string>
#include <map>
#include <utility>

using std::vector;
using std::list;
using std::string;
using std::map;
using std::pair;

class CShakeRenderer;

/*!
  \class CFieldDescription
  \brief class containing description of a field

  \author David Place, Steffen Abe
  $Revision$
  $Date$
*/
class CFieldDescription {
protected:
  ADataField::ADataType m_type ;
  ADataField::AFieldShape m_shape ;
  string m_name ;
  vector<ADataField::AFieldShape> m_available_shapes;

public:
  CFieldDescription() ;
  ~CFieldDescription() ;

  void setName(string name) ;
  string getName() ;

  void addShape(const ADataField::AFieldShape&);
  vector<ADataField::AFieldShape> getAvailableShapes();
  ADataField::AFieldShape getShape(){return m_shape;};

  ADataField::ADataType getType() ;
  void setType(ADataField::ADataType type) ;
} ;


/*!
  \class CDataParticle
  \brief class containing remote data for a particle (ie. Position, radius)

  \author David Place, Steffen Abe
  $Revision$
  $Date$
*/
class CDataParticle 
{
 private:
  Vec3 m_Pos;             //!< Position of the particle
  double m_rad ;            //!< Radius of the particle
  bool m_valid;

 public:
  CDataParticle();
  CDataParticle(const Vec3&,double);

  inline bool isValid(){return m_valid;};
  inline Vec3 Pos(){return m_Pos;};
  inline double Rad(){return m_rad;};
} ;

/*!
  \class CRemoteData
  \brief class containing necessary information to display particle data

  \author David Place, Steffen Abe
  $Revision$
  $Date$
*/
class CRemoteData 
{
private:
  CBackEndCom *m_backend; 

  map<int,CDataParticle> m_particles; //!< all particles...
  map<int,SPRenderParticle> m_sp_particles; //!< all particles...
  list<ADataField *> m_activefields ;   //!< list of active fields
  vector<CFieldDescription> m_fieldsAvailable; //!< list of field available
  bool scale_calculated ;
  int cur_Id; //!< current cummulative Id
  double m_scale ; //!< scale factor for display to fit in a (-1,-1,-1)-(1,1,1) cube
  Vec3 m_centre ; //!< Centre of the display area

  bool isActive(const string&);
  int addField(ADataField *) ; 

public:
  CRemoteData() ;
  virtual ~CRemoteData() ;
  void setBackend(CBackEndCom *BE){m_backend=BE;};

  double scale() ;
  Vec3 & centre() ;
  void calcScale() ;

  void addFieldDescription(CFieldDescription) ; 
  void activateField(const string&);
  void reloadActiveFields();

  void deleteField(const string&) ; 

  // particle related functions
  int getNbParticle() ;
  int getNbCParticle(){return m_sp_particles.size();} ;
  void addParticle(const Vec3&,double,int);
  void addParticle(const SPRenderParticle&,int);
  CDataParticle & getParticle(int i) ; 
  SPRenderParticle & getCParticle(int i) ; 

  // available fields
  int getNbFields(){return m_fieldsAvailable.size();};
  CFieldDescription getFieldDecription(int i){return m_fieldsAvailable[i];};

  // active fields
  int getNbActiveFields(){return m_activefields.size();};
  vector<string> getActiveFieldNames(); // ?
  ADataField* getActiveField(int) const;
} ;

#endif
