#include "ShakeRenderer.h"
#include "shake_debug.h"

CShakeRenderer::CShakeRenderer() 
{
  m_xRot = 0.0;  m_yRot = 0.0;  m_zRot = 0.0;
  m_xTrans = 0.0;  m_yTrans = 0.0;  m_zTrans = 0.0;
  eyex=0.0;  eyey=0.0;  eyez=4.0;
  SphereInit=false ; 
  CylInit=false ;

  initColors() ;
}

CShakeRenderer::~CShakeRenderer() 
{}


void CShakeRenderer::initColors() {
  mat_specular[0] = 0.1f; mat_specular[1]= 0.1f; mat_specular[2]= 0.1f; mat_specular[3]= 1.0f ;
  mat_shin[0] = 50.0 ;
  col_zero[0] = 0.f ; col_zero[1] = 0.f ; col_zero[2] = 0.f ; col_zero[3] = 1.f;

  l0_ambient[0] = (float)(0.2); 
  l0_ambient[1] = (float)(0.2); 
  l0_ambient[2] = (float)(0.2); 
  l0_ambient[3] =  1.0f ;
  l0_diffuse[0] = (float)(0.8); 
  l0_diffuse[1] = (float)(0.8); 
  l0_diffuse[2] = (float)(0.8); 
  l0_diffuse[3] =  1.0f ;
  l0_position[0] = (float)(4.0) ;
  l0_position[1] = (float)(4.0) ;
  l0_position[2] = (float)(4.0) ;
  l0_position[3] = 0.0f ;

  lmodel_ambient[0] = 0.4f; lmodel_ambient[1] = 0.4f; lmodel_ambient[2] = 0.4f; lmodel_ambient[3] = 1.0f ;
}

void CShakeRenderer::createSphere(int res)
{
  XDEBUG(console.XDebug()<<"ShakeRendererST::createSphere ( " << res << " ) \n");
  SphereListNr=glGenLists(1);
  XDEBUG(console.XDebug()<< "Nr. " << SphereListNr << "\n");
  glNewList(SphereListNr,GL_COMPILE);
  GLUquadricObj * quadObj = gluNewQuadric ();
  gluQuadricDrawStyle (quadObj, (GLenum) GLU_FILL);
  gluQuadricNormals(quadObj, (GLenum) GLU_SMOOTH);
  gluSphere (quadObj, 1.0 , res, res/2);
  glEndList();
}

void CShakeRenderer::createCyl(int res)
{
  sin_t=new real[res+1];
  DB_NEWALLOC(sin_t,"real") ;
  cos_t=new real[res+1];
  DB_NEWALLOC(cos_t,"real") ;
  real step=2*M_PI/(real)(res);
  for(int i=0;i<res+1;i++){
    cos_t[i]=cos(real(i)*step);
    sin_t[i]=sin(real(i)*step);
  }
} 

void CShakeRenderer::drawTriangle(Vec3 P1,Vec3 P2,Vec3 P3, Vec3 N, Colour C, Colour C2, Colour C3) {
  GLfloat Cols[4],Cols2[4],Cols3[4] ;
  
  Cols[0] = (float)C.red ;
  Cols[1] = (float)C.green ;
  Cols[2] = (float)C.blue ;
  Cols[3] = (float)C.alpha ;
  Cols2[0] = (float)C2.red ;
  Cols2[1] = (float)C2.green ;
  Cols2[2] = (float)C2.blue ;
  Cols2[3] = (float)C2.alpha ;
  Cols3[0] = (float)C3.red ;
  Cols3[1] = (float)C3.green ;
  Cols3[2] = (float)C3.blue ;
  Cols3[3] = (float)C3.alpha ;
  
  glBegin(GL_TRIANGLES);
  glNormal3d(N.X(),N.Y(),N.Z()) ;
  glColor4fv(Cols);
  glVertex3d(P1.X(),P1.Y(),P1.Z()) ;
  glColor4fv(Cols2);
  glVertex3d(P2.X(),P2.Y(),P2.Z()) ;
  glColor4fv(Cols3);
  glVertex3d(P3.X(),P3.Y(),P3.Z()) ;
  glEnd();
}

void CShakeRenderer::drawSphere(Vec3 P ,real Size, Colour C)
{
  GLfloat Cols[4] ;

  if(!SphereInit){
    createSphere(8);
    SphereInit=true;
  }
  
  glEnable(GL_NORMALIZE);

  Cols[0] = (float)C.red ;
  Cols[1] = (float)C.green ;
  Cols[2] = (float)C.blue ;
  Cols[3] = (float)C.alpha ;
 
  glEnable(GL_LIGHTING) ; 
  glPushMatrix() ; 
  glEnable(GL_CULL_FACE)  ; // cull backfaces only if not transparent
  glTranslated (P.X(), P.Y(), P.Z()); 
  glColor4fv(Cols) ;
    
  glScaled(Size,Size,Size);
  glCallList(SphereListNr);

  glPopMatrix() ;
  glDisable(GL_CULL_FACE);
  glDisable(GL_NORMALIZE);
}

void CShakeRenderer::drawVector(Vec3 P,Vec3 V,Colour C)
{
  GLfloat Cols[4] ;
  Vec3 H,E(eyex,eyey,eyez);
  
  Cols[0] = (float)C.red ;
  Cols[1] = (float)C.green ;
  Cols[2] = (float)C.blue ;
  Cols[3] = (float)C.alpha ;

  glDisable(GL_LIGHTING);
  glDepthMask (GL_FALSE);
  glDisable(GL_DEPTH_TEST);
  glColor4fv(Cols);
  glLineWidth(1) ;
  glBegin(GL_LINES);
  glVertex3d(P.X(),P.Y(),P.Z());
  glVertex3d(P.X()+V.X(),P.Y()+V.Y(),P.Z()+V.Z());
  glEnd();

  // direction of H is perpendicular to line of view and vector V
  H=cross(V,E); 
  if(H.norm()!=0)
    {
      H=(H/H.norm())*V.norm()*0.2;

      glBegin(GL_LINES);
      glVertex3d(P.X()+V.X(),P.Y()+V.Y(),P.Z()+V.Z());
      glVertex3d(P.X()+V.X()*0.7+H.X(),P.Y()+V.Y()*0.7+H.Y(),P.Z()+V.Z()*0.7+H.Z());
      glVertex3d(P.X()+V.X(),P.Y()+V.Y(),P.Z()+V.Z());
      glVertex3d(P.X()+V.X()*0.7-H.X(),P.Y()+V.Y()*0.7-H.Y(),P.Z()+V.Z()*0.7-H.Z());
      glEnd();
    }
  glBegin(GL_POINTS);
  glVertex3d(P.X(),P.Y(),P.Z());
  glEnd();
  glEnable(GL_LIGHTING);
  glDepthMask (GL_TRUE);
  glEnable(GL_DEPTH_TEST);
}

void CShakeRenderer::drawLine(Vec3 P1,Vec3 P2,Colour C)
{
  GLfloat Cols[4] ;
  
  Cols[0] = (float)C.red ;
  Cols[1] = (float)C.green ;
  Cols[2] = (float)C.blue ;
  Cols[3] = (float)C.alpha ;

  glDisable(GL_LIGHTING);
  glLineWidth(2) ;
  glColor4fv(Cols);
  glBegin(GL_LINES);
  glVertex3d(P1.X(),P1.Y(),P1.Z());
  glVertex3d(P2.X(),P2.Y(),P2.Z()) ;
  glEnd();
  glEnable(GL_LIGHTING);

}

void CShakeRenderer::drawCylinder(Vec3 P1,Vec3 P2,real rad, int Sub, Colour C)
{
  GLfloat Cols[4] ;
  Vec3 Dir=P2-P1 ;


  if(!CylInit){
    createCyl(24);
    CylInit=true;
  }

  Vec3 z = Dir/Dir.norm() ;
  Vec3 x = Vec3(z.Y(),-z.X(),0) ;
  if (x.norm()==0) {
    x = Vec3(0,z.Z(),-z.Y()) ;
  }
  if (x.norm()==0) {
    x = Vec3(z.Z(),0,-z.X()) ;
  }
  x *= 1/x.norm() ;
  Vec3 y = cross(z,x) ;
      
  Cols[0] = (float)C.red ;
  Cols[1] = (float)C.green ;
  Cols[2] = (float)C.blue ;
  Cols[3] = (float)C.alpha ;
      

  glEnable(GL_LIGHTING) ;
  glEnable(GL_CULL_FACE); // cull backfaces if not transparent
    
  glBegin(GL_QUAD_STRIP);
  glColor4fv(Cols) ;
  for (int i=0;i<Sub;i++){
    Vec3 M1=P1+(x*cos_t[i]+y*sin_t[i])*rad*0.5;
    Vec3 M4=P2+(x*cos_t[i]+y*sin_t[i])*rad*0.5;
    Vec3 N=x*cos_t[i]+y*sin_t[i];
    Vec3 N2=x*cos_t[i+1]+y*sin_t[i+1];
    glNormal3d(N.X(),N.Y(),N.Z()) ;
    glVertex3d(M4.X(),M4.Y(),M4.Z()) ;
    glNormal3d(N.X(),N.Y(),N.Z()) ;
    glVertex3d(M1.X(),M1.Y(),M1.Z()) ;
  }
  glEnd();
  glDisable(GL_CULL_FACE);
}	


void CShakeRenderer::drawCylinder(Vec3 P1,Vec3 P2,real rad1, real rad2, int Sub, Colour C1, Colour C2)
{
  GLfloat Cols1[4] ;
  GLfloat Cols2[4] ;
  Vec3 Dir=P2-P1 ;

  Vec3 z = Dir/Dir.norm() ;
  Vec3 x = Vec3(z.Y(),-z.X(),0) ;
  if (x.norm()==0) {
    x = Vec3(0,z.Z(),-z.Y()) ;
  }
  if (x.norm()==0) {
    x = Vec3(z.Z(),0,-z.X()) ;
  }
  x *= 1/x.norm() ;
  Vec3 y = cross(z,x) ;
      
  real step,teta ;
      
  Cols1[0] = (float)C1.red ;
  Cols1[1] = (float)C1.green ;
  Cols1[2] = (float)C1.blue ;
  Cols1[3] = (float)C1.alpha ;
  Cols2[0] = (float)C2.red ;
  Cols2[1] = (float)C2.green ;
  Cols2[2] = (float)C2.blue ;
  Cols2[3] = (float)C2.alpha ;
      

  glEnable(GL_LIGHTING) ;
  glEnable(GL_CULL_FACE); // cull backfaces if not transparent

    
  glBegin(GL_QUAD_STRIP);
  step = 2*M_PI/(real)(Sub) ;
  float eps=0.1f;
  for (teta=0; teta<2*M_PI+eps; teta += step) {
    Vec3 M1=P1+(x*cos(teta)+y*sin(teta))*(rad1/2) ; 
    Vec3 M4=P2+(x*cos(teta)+y*sin(teta))*(rad2/2) ; 
    Vec3 N = x*cos(teta)+y*sin(teta) ;
    Vec3 N2 = x*cos(teta+step)+y*sin(teta+step) ;
    glNormal3d(N.X(),N.Y(),N.Z()) ;
    glColor4fv(Cols2) ;
    glVertex3d(M4.X(),M4.Y(),M4.Z()) ;
    glNormal3d(N.X(),N.Y(),N.Z()) ;
    glColor4fv(Cols1) ;
    glVertex3d(M1.X(),M1.Y(),M1.Z()) ;
  }
  glEnd();
  glDisable(GL_CULL_FACE);

}

void CShakeRenderer::drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4,Colour C)
{
  GLfloat Cols[4] ;
  
  Cols[0] = (float)C.red ;
  Cols[1] = (float)C.green ;
  Cols[2] = (float)C.blue ;
  Cols[3] = (float)C.alpha ;

  glColor4fv(Cols);
  glBegin(GL_QUADS);
  glVertex3d(V1.X(),V1.Y(),V1.Z());
  glVertex3d(V2.X(),V2.Y(),V2.Z());
  glVertex3d(V3.X(),V3.Y(),V3.Z());
  glVertex3d(V4.X(),V4.Y(),V4.Z());
  glEnd();
  glFlush();
}

void CShakeRenderer::drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4, Vec3 N, Colour C)
{
  GLfloat Cols[4] ;
  
  Cols[0] = (float)C.red ;
  Cols[1] = (float)C.green ;
  Cols[2] = (float)C.blue ;
  Cols[3] = (float)C.alpha ;

  glColor4fv(Cols);
  glBegin(GL_QUADS);
  glNormal3d(N.X(),N.Y(),N.Z()) ;
  glVertex3d(V1.X(),V1.Y(),V1.Z());
  glVertex3d(V2.X(),V2.Y(),V2.Z());
  glVertex3d(V3.X(),V3.Y(),V3.Z());
  glVertex3d(V4.X(),V4.Y(),V4.Z());
  glEnd();
  glFlush();
}

void CShakeRenderer::drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4, Vec3 N, Colour C1,Colour C2,Colour C3,Colour C4)
{
  GLfloat Cols[4] ;
  
  glNormal3d(N.X(),N.Y(),N.Z()) ;
  glBegin(GL_QUADS);
  Cols[0] = (float)C1.red ;
  Cols[1] = (float)C1.green ;
  Cols[2] = (float)C1.blue ;
  Cols[3] = (float)C1.alpha ;
  glNormal3d(N.X(),N.Y(),N.Z()) ;
  glColor4fv(Cols);
  glVertex3d(V1.X(),V1.Y(),V1.Z());
  Cols[0] = (float)C2.red ;
  Cols[1] = (float)C2.green ;
  Cols[2] = (float)C2.blue ;
  Cols[3] = (float)C2.alpha ;
  glNormal3d(N.X(),N.Y(),N.Z()) ;
  glColor4fv(Cols);
  glVertex3d(V2.X(),V2.Y(),V2.Z());
  Cols[0] = (float)C3.red ;
  Cols[1] = (float)C3.green ;
  Cols[2] = (float)C3.blue ;
  Cols[3] = (float)C3.alpha ;
  glNormal3d(N.X(),N.Y(),N.Z()) ;
  glColor4fv(Cols);
  glVertex3d(V3.X(),V3.Y(),V3.Z());
  Cols[0] = (float)C4.red ;
  Cols[1] = (float)C4.green ;
  Cols[2] = (float)C4.blue ;
  Cols[3] = (float)C4.alpha ;
  glNormal3d(N.X(),N.Y(),N.Z()) ;
  glColor4fv(Cols);
  glVertex3d(V4.X(),V4.Y(),V4.Z());
  glEnd();
  glFlush();
}

void CShakeRenderer::drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4,Colour C1,Colour C2,Colour C3,Colour C4)
{
  GLfloat Cols[4] ;
  
  glBegin(GL_QUADS);
  Cols[0] = (float)C1.red ;
  Cols[1] = (float)C1.green ;
  Cols[2] = (float)C1.blue ;
  Cols[3] = (float)C1.alpha ;
  glColor4fv(Cols);
  glVertex3d(V1.X(),V1.Y(),V1.Z());
  Cols[0] = (float)C2.red ;
  Cols[1] = (float)C2.green ;
  Cols[2] = (float)C2.blue ;
  Cols[3] = (float)C2.alpha ;
  glColor4fv(Cols);
  glVertex3d(V2.X(),V2.Y(),V2.Z());
  Cols[0] = (float)C3.red ;
  Cols[1] = (float)C3.green ;
  Cols[2] = (float)C3.blue ;
  Cols[3] = (float)C3.alpha ;
  glColor4fv(Cols);
  glVertex3d(V3.X(),V3.Y(),V3.Z());
  Cols[0] = (float)C4.red ;
  Cols[1] = (float)C4.green ;
  Cols[2] = (float)C4.blue ;
  Cols[3] = (float)C4.alpha ;
  glColor4fv(Cols);
  glVertex3d(V4.X(),V4.Y(),V4.Z());
  glEnd();
  glFlush();
}

void  CShakeRenderer::initializeGLArea()
{
  glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

  GLfloat local_view[] = { 0.0f };
  GLfloat two_side[] = { 1.0f };
  
  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH) ;
  glLightfv(GL_LIGHT0, GL_AMBIENT, l0_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, l0_diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, l0_position);
  
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, two_side);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);
  
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  
  glEnable(GL_LIGHTING) ;     
  glEnable(GL_LIGHT0) ;       

}

void  CShakeRenderer::resizeGLArea( int width, int height )
{
  m_winWidth = width ;
  m_winHeight = height ;
  glViewport( 0, 0, width, height );
  //glViewport( 0, 0, width, height );
  //glMatrixMode(GL_PROJECTION);
  //glLoadIdentity();
  //gluPerspective(30.0,GLfloat(width)/GLfloat(height),0.1,200.0);
  //gluLookAt (eyex, eyey, eyez, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
} 

bool CShakeRenderer::stereo = false ;

void CShakeRenderer::paintGLArea()
{
  glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS,mat_shin) ;
  glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,mat_specular) ;

  glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE) ;
  glEnable(GL_COLOR_MATERIAL) ;
  if (stereo) {
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    GLfloat focallength = 2.0 ;
    GLfloat ratio = GLfloat(m_winWidth)/GLfloat(m_winHeight) ;
    //GLfloat apertureX = 53.0 ;
    //GLfloat apertureY = 44.26 ;
    GLfloat apertureX = 50.0 ;
    GLfloat apertureY = 50.0 ;
    GLfloat camnear = 0.2 ;
    GLfloat eyesep = 0.07 ;
    GLfloat camfar = 200.0 ;
    GLfloat wd2 = camnear * tan(apertureX/2.0*3.14159/180.0) ;
    GLfloat wd3 = camnear * tan(apertureY/2.0*3.14159/180.0) ;
    GLfloat ndfl = camnear / focallength ;
    GLfloat left = -ratio * wd2 - 0.5 * eyesep * ndfl ;
    GLfloat right = ratio * wd2 - 0.5 * eyesep * ndfl ;
    glFrustum(left,right,-wd3,wd3,camnear,camfar) ;
    glMatrixMode(GL_MODELVIEW);
    glDrawBuffer(GL_BACK_RIGHT) ;
    glLoadIdentity();
    Vec3 P(eyex, eyey, eyez) ;
    Vec3 D = -P ;
    Vec3 U(0,1.0,0.0) ;
    Vec3 R = cross(D,U) ;
    R = (1/R.norm()*eyesep/2.0)*R ;
    gluLookAt (eyex+R.X(), eyey+R.Y(), eyez+R.Z(), R.X(), R.Y(), R.Z(), U.X(), U.Y(), U.Z());
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix() ;
    internal_paintGLArea() ;
    glPopMatrix() ;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    left = -ratio * wd2 + 0.5 * eyesep * ndfl ;
    right = ratio * wd2 + 0.5 * eyesep * ndfl ;
    glFrustum(left,right,-wd3,wd3,camnear,camfar) ;
    glMatrixMode(GL_MODELVIEW);
    glDrawBuffer(GL_BACK_LEFT) ;
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    gluLookAt (eyex-R.X(), eyey-R.Y(), eyez-R.Z(), -R.X(), -R.Y(), -R.Z(), U.X(), U.Y(), U.Z());

    glPushMatrix() ;
    internal_paintGLArea() ;
    glPopMatrix() ;
  } else {
    glDrawBuffer(GL_BACK) ;
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(40.0,GLfloat(m_winWidth)/GLfloat(m_winHeight),0.1,200.0);
    gluLookAt (eyex, eyey, eyez, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    internal_paintGLArea() ;
  }
}

void CShakeRenderer::internal_paintGLArea()
{
  //XDEBUG(console.XDebug() << "OpenGL Vendor string: " << (char *)(glGetString(GL_VENDOR)) << "\n") ;

  //glMatrixMode(GL_MODELVIEW);
  //glLoadIdentity();

  glTranslatef( m_xTrans, m_yTrans, m_zTrans);

  
  glRotatef( m_xRot, 1.0, 0.0, 0.0 ); 
  glRotatef( m_yRot, 0.0, 1.0, 0.0 ); 
  glRotatef( m_zRot, 0.0, 0.0, 1.0 ); 

  render() ;

  glFlush();
}

/*!
  render the current scene

  \todo remove hardcoded resolution for cylinders
*/
void CShakeRenderer::render() 
{
  tconsole.XDebug() << "CShakeRenderer::render() \n";
  for(vector<sph>::iterator iter=m_spheres.begin();iter!=m_spheres.end();iter++){
     drawSphere(iter->pos,iter->r,iter->c);
  }
  for(map<pair<int,int>,con,Tpair_cmp<int,int> >::iterator iter2=m_conn.begin();
      iter2!=m_conn.end();iter2++){
    int idx1=iter2->first.first;
    int idx2=iter2->first.second;
    Vec3 pos1=m_spheres[idx1].pos;
    Vec3 pos2=m_spheres[idx2].pos;
    drawCylinder(pos1,pos2,iter2->second.r,8,iter2->second.c);
  }
  for(vector<SPRenderParticle>::iterator iter3=m_sp_parts.begin();iter3!=m_sp_parts.end();iter3++){
    iter3->draw();
  }
  tconsole.XDebug() << "end CShakeRenderer::render() \n";
}

/*!
  Clear scene
*/
void CShakeRenderer::clear()
{
  m_spheres.erase(m_spheres.begin(),m_spheres.end());
  m_sp_parts.erase(m_sp_parts.begin(),m_sp_parts.end());
}

/*!
  Add a sphere to the scene. Returns the Id of the sphere

  \param pos the position
  \param r the radius
*/
int CShakeRenderer::addParticle(Vec3 pos, float r)
{
  sph new_sph;
  new_sph.pos=pos;
  new_sph.r=r;
  new_sph.c=Colour(0.0,0.0,0.0,0.0);
  m_spheres.push_back(new_sph);

  return m_spheres.size()-1;
}

/*!
  Add a complex particle to the scene. Returns the Id of the particle

  \param cp the complex particle 
*/
int CShakeRenderer::addComplexParticle(SPRenderParticle cp)
{
  m_sp_parts.push_back(cp);

  return m_sp_parts.size()-1;
}

/*!
  Add a color to a sphere

  \param id the Id of the sphere
  \param c the color
*/
void CShakeRenderer::addColorToParticle(int id, Colour c)
{
  int ns=m_spheres.size();
  console.XDebug()<< "id,ns :  " << id << " , " << ns <<  "\n";
  if(id<ns){
    m_spheres[id].c+=c;
    console.XDebug()<< "color to Sphere\n"; 
  } else {
    m_sp_parts[id-ns].addColor(c);
    console.XDebug()<< "color to complex particle\n"; 
  }
}

/*!
  Add a color to a complex particle

  \param id the Id of the sphere
  \param c the color
*/
void CShakeRenderer::addColorToComplexParticle(int id, const Colour& C)
{
  m_sp_parts[id].addColor(C);
}

/*!
  Add a connection to the scene. Returns the Id of the connection

  \param i id of one endparticle
  \param j id of the other endparticle
  \param r the radius
*/
int CShakeRenderer::addConnection(int i,int j,float r)
{
  con new_conn;
  
  new_conn.r=r;
  new_conn.c=Colour(0.0,0.0,0.0,0.0);
  m_conn[pair<int,int>(i,j)]=new_conn;

  return m_conn.size()-1;
}

void CShakeRenderer::addColorToConnection(int i, int j,const Colour& c)
{
  m_conn[pair<int,int>(i,j)].c+=c;
  //console.XDebug() << "color [ " << m_spheres[id].c.red << ", "<< m_spheres[id].c.green << ", "<< m_spheres[id].c.blue << ", "<< m_spheres[id].c.alpha << "]\n";
}
