#ifndef __SHAKERENDERER_H
#define __SHAKERENDERER_H

// -- project includes --
#include "shake_math.h"
#include "CPartRender.h"

// -- OpenGL includes --
#include <GL/gl.h>
#include <GL/glu.h>

// -- STL includes --
#include <vector>
#include <map>
#include <utility>

using std::vector;
using std::map;
using std::pair;

class CShakeDisplay ;

/*!
  \class CShakeRenderer
  \brief  Windowsystem independent part of the shake++ 
  OpenGL rendering class 

  \author David Place,Steffen Abe
  $Revision$
  $Date$
*/
class CShakeRenderer
{
private:
  GLuint SphereListNr;
  real* cos_t;
  real* sin_t;
  bool SphereInit,CylInit;
  void createSphere(int res) ;
  void createCyl(int res) ;
  float m_xTrans,m_yTrans,m_zTrans,m_xRot,m_yRot,m_zRot ;
  int m_winWidth, m_winHeight ;

  struct sph{
    Vec3 pos;
    Colour c;
    float r;
  };

  struct con{
    Colour c;
    float r;
  };

  vector<sph> m_spheres;
  vector<SPRenderParticle> m_sp_parts; 
  map<pair<int,int>,con,Tpair_cmp<int,int> > m_conn;

protected:
    
  GLfloat eyex,eyey,eyez;    
  GLfloat mat_specular[4],mat_shin[1],col_zero[4];
  GLfloat l0_ambient[4],l0_diffuse[4],l0_position[4];
  GLfloat lmodel_ambient[4] ;

  //! drawing functions 
  void initColors() ;
  void drawTriangle(Vec3 P1,Vec3 P2,Vec3 P3, Vec3 N, Colour C, Colour C2, Colour C3) ;
  void drawSphere(Vec3 P ,real Size, Colour C) ;
  void drawVector(Vec3 P,Vec3 V,Colour C) ;
  void drawLine(Vec3 P1,Vec3 P2,Colour C) ;
  void drawCylinder(Vec3 P1,Vec3 P2,real rad, int Sub, Colour C) ;
  void drawCylinder(Vec3 P1,Vec3 P2,real rad1, real rad2, int Sub, Colour C1, Colour C2) ;
  void drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4,Colour C); 
  void drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4, Vec3 N, Colour C) ;
  void drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4, Vec3 N, Colour C1,Colour C2,Colour C3,Colour C4) ;
  void drawQuad(Vec3 V1,Vec3 V2,Vec3 V3,Vec3 V4,Colour C1,Colour C2,Colour C3,Colour C4) ;

  void render() ;
  void internal_paintGLArea() ;

public:
    
  static bool stereo ;
  CShakeRenderer();
  virtual ~CShakeRenderer() ;


  void initializeGLArea() ;
  void resizeGLArea( int width, int height ) ;
  void paintGLArea() ;

  //! Functions to control the point of view 
  //! translations
  void setTransX(float f){m_xTrans=f;};
  void setTransY(float f){m_yTrans=f;};
  void setTransZ(float f){m_zTrans=f;};
  
  void addTransX(float f){m_xTrans+=f;};
  void addTransY(float f){m_yTrans+=f;};
  void addTransZ(float f){m_zTrans+=f;};

  //! rotations
  void setRotX(float f){m_xRot=f;};
  void setRotY(float f){m_yRot=f;};
  void setRotZ(float f){m_zRot=f;};

  void addRotX(float f){m_xRot+=f;};
  void addRotY(float f){m_yRot+=f;};
  void addRotZ(float f){m_zRot+=f;};

  //! drawing interface
  void clear();
  int addParticle(Vec3,float);
  int addComplexParticle(SPRenderParticle);
  void addColorToParticle(int,Colour);
  void addColorToComplexParticle(int,const Colour&);
  int addConnection(int,int,float);
  void addColorToConnection(int,int,const Colour&);
};


#endif
