#include "ShakeView.h"
#include "ShakeDisplay.h"

//--- GL2PS library ---
#include "gl2ps.h"

CShakeView::CShakeView(CShakeDisplay *disp, int id)
{
  m_id=id;
  m_theDisplay=disp;
}

CShakeView::~CShakeView()
{
  
}

/*!
  Get current particle data from BE
*/
void CShakeView::update_particles()
{
  if (m_backend) {
    m_backend->getParticlesData(&m_remoteData) ;
  } else {
    console.Warning() << "Unable to connect, backend ptr not initialized...\n" ;
  }
}

/*!
  Get current field data from BE
*/
void CShakeView::update_fields()
{
  if (m_backend) {
    m_backend->getAvailableFields(&m_remoteData);
    m_remoteData.calcScale() ;
  } else {
    console.Warning() << "Unable to connect, backend ptr not initialized...\n" ;
  }
}

/*!
  Put data into the renderer
*/
void CShakeView::update_view()
{
  tconsole.XDebug() << "CShakeView::update_view()\n";
  int i;

  m_renderer->clear();
  m_remoteData.reloadActiveFields();
  double scl=1.0/m_remoteData.scale();
  tconsole.XDebug() << "scale : " << scl << "\n";
  int npart=m_remoteData.getNbParticle();
  tconsole.XDebug() << npart <<  " particles\n";
  for(i=0;i<npart;i++){
    CDataParticle cdp=m_remoteData.getParticle(i);
    m_renderer->addParticle((cdp.Pos()-m_remoteData.centre())*scl,cdp.Rad()*scl);
  }
  int ncpart=m_remoteData.getNbCParticle();
  tconsole.XDebug() << ncpart <<  " complex particles\n";
  for(i=0;i<ncpart;i++){
    SPRenderParticle srp=m_remoteData.getCParticle(i);
    srp.scale(scl);
    m_renderer->addComplexParticle(srp);
  }
  int nafield=m_remoteData.getNbActiveFields();
  for(i=0;i<nafield;i++){
    ADataField *adf=m_remoteData.getActiveField(i);
    (*adf).draw(m_renderer);
  }
  repaint();
}

void CShakeView::printPS(const char* filename)
{
  FILE *fp = fopen(filename,"w");
  int bs=0;
  int state=GL2PS_OVERFLOW;

  while(state==GL2PS_OVERFLOW){
    bs+=1024*1024; // add 1MB
    gl2psBeginPage("test","mview",
		   GL2PS_EPS,GL2PS_SIMPLE_SORT,
		   GL2PS_NONE,
		   GL_RGBA,0,NULL,bs,fp,NULL);
    repaint();
    state=gl2psEndPage();
  }
}
