#include "stdafx.h"

//--- Project includes ---
#include "LSMSocket.h"
#include "remoteConsole.h"
#include "SocketLoop.h"
#include "RemoteData.h"

//--- IO includes ---
#include <iostream>
using std::cout;

#ifdef _WINDOWS
#include "Windows/LSMearth.h"
#ifdef _GTKWIN
#include "GTK/GtkDisplay.h"
#endif
#else
#define MPI_NO_CPPBIND
#include "GTK/GtkDisplay.h"
#ifndef _WINDOWS
#include <unistd.h>
#endif
#endif

void DisplayHelp() {
cout << "\n" ;
cout << " LSMearth [-v] [-V [option]] [-help] [-f script] [-Id name] [-P port]\n" ;
cout << "\n" ;
cout << "  -v \n" ;
cout << "      Turn on console (stdout) message output on console.\n" ;
cout << "  -V [options]\n" ;
cout << "      Verbose level for message output on console. Options\n" ;
cout << "      are: all, debug, info, message, warning, error, critical, silent\n" ;
cout << "      (eg. -v info set the verbose level to info).\n" ;
cout << "  -help  \n" ;
cout << "      Display this message\n" ;
cout << "  -f script\n" ;
cout << "      Load the script <script> in the internal editor.\n" ;
cout << "  -Id \n" ;
cout << "      Set the Id for the client.\n" ;
cout << "  -H <host>\n" ;
cout << "      Host where the back-end is running.\n" ;
cout << "  -S <name>\n" ;
cout << "      Name of the back-end.\n" ;
cout << "  -P \n" ;
cout << "      Set the Port for sockets.\n" ;
cout << "\n" ;
}

void startIG(char *filename,int argc, char **argv) ;

int LSMmain(int argc, char **argv) {
    
  char * filename=NULL ;
  int i ;
  char *myId; //,*myServer ; //*myHost
  char defaultId[] = "MyClient" ;
  //char defaultHost[] = "localhost" ;
  //char defaultServer[] = "MyServer" ;
  int myPort ;
  int defaultPort = sock->minPort() ;

  console.SetQuiet(true) ;
  if (argc >=2) {
    for (i=1; i<argc; i++) {
      if (!strcmp(argv[i],"-help")) {
        DisplayHelp() ;
        return true ;
      }
      if (!strcmp(argv[i],"-v")) {
        console.SetQuiet(false) ;
      }
      if (!strcmp(argv[i],"-V")) {
        console.SetVerbose(BasicCon::dbg) ;
	i++ ;
	if ( i<argc) {
	  if (!strcmp(argv[i],"all")) {
	    console.SetVerbose(Con::all) ;
	  } else if (!strcmp(argv[i],"debug")) {
	    console.SetVerbose(Con::dbg) ;
	  } else if (!strcmp(argv[i],"info")) {
	    console.SetVerbose(Con::info) ;
	  } else if (!strcmp(argv[i],"message")) {
	    console.SetVerbose(Con::msg) ;
	  } else if (!strcmp(argv[i],"warning")) {
	    console.SetVerbose(Con::warning) ;
	  } else if (!strcmp(argv[i],"error")) {
	    console.SetVerbose(Con::err) ;
	  } else if (!strcmp(argv[i],"critical")) {
	    console.SetVerbose(Con::critical) ;
	  } else if (!strcmp(argv[i],"silent")) {
	    console.SetVerbose(Con::silent) ;
	  } else {
	    i-- ;
	  }
	} else {
	  i-- ;
	}
      }
    }
  }

  console.Message(false) << "\n" ;
  console.Message(false) << "LSMearth -- Version 4.0a (c) 1993-2001" << "\n" ;
  console.Message(false) << "\n" ;

  myId = &defaultId[0] ;
  //myHost = &defaultHost[0] ;
  //myServer = &defaultServer[0] ;
  myPort = defaultPort ;

  if (argc >=2) {
    for (i=1; i<argc; i++) {
      if (!strcmp(argv[i],"-f")) {
	i++ ;
	if (i<argc) {
	  filename = argv[i] ;
	} else {
  	  console.Critical() << "Command line syntax error\n" ;
	  DisplayHelp() ;
	  return true ;
	}
      } else if (!strcmp(argv[i],"-Id")) {
	i++ ;
	if (i<argc) {
	  myId = argv[i] ;
	} else {
  	  console.Critical() << "Command line syntax error\n" ;
	  DisplayHelp() ;
	  return true ;
	}
      } else if (!strcmp(argv[i],"-H")) {
	i++ ;
	if (i<argc) {
	  //myHost = argv[i] ;
	} else {
  	  console.Critical() << "Command line syntax error\n" ;
	  DisplayHelp() ;
	  return true ;
	}
      } else if (!strcmp(argv[i],"-S")) {
	i++ ;
	if (i<argc) {
	  //myServer = argv[i] ;
	} else {
  	  console.Critical() << "Command line syntax error\n" ;
	  DisplayHelp() ;
	  return true ;
	}
      } else if (!strcmp(argv[i],"-P")) {
	i++ ;
	if (i<argc) {
	  myPort = atol(argv[i]) ;
	} else {
  	  console.Critical() << "Command line syntax error\n" ;
	  DisplayHelp() ;
	  return true ;
	}
      }
    }
  }

  sock->maxPort() = (sock->maxPort()-sock->minPort()) + myPort ;
  sock->minPort() = myPort ;

  CShakeSocketClient CS(myId) ;
  CSocketLoop MainLoop(CS) ;

  // initialise the socket for this application
  sock->initAsClient(&MainLoop,&CS) ;

  ARemoteConsoleServer::setSocketLoop(&MainLoop) ;
        
  startIG(filename,argc,argv) ;

  tconsole.Debug() << "Exiting...\n" ;

  if (sock->client() && sock->client()->connected()) {
    sock->sendTag("END") ; // tell server we end
    sock->waitFor("ECK") ; // wait until we can quit (ie. server received the end msg.)
  }
  
  return 0 ;
}

#ifndef _WINDOWS

int main(int argc, char **argv) {	
    return LSMmain(argc,argv) ;
}

void startIG(char *filename,int argc, char **argv) {
    CDisplayGTK *App = new CDisplayGTK(filename,argc,argv) ;
    App->create() ; // construct window
    App->exec() ;   // execute window...
}

#else

INT WINAPI WinMain(HINSTANCE   hInstance,    // handle to current instance     
                   HINSTANCE   hPrevInstance,// handle to previous instance     
                   LPSTR       lpCmdLine,    // pointer to command line     
                   INT         nShowCmd)     // show state of window 
{ 
    char *args[]={"LSMearth",NULL} ;
    char **argv ;
    argv = args ;
    
    return LSMmain(1,argv) ;
}

#ifdef _GTKWIN

void startIG(char *filename,int argc, char **argv) {
    CDisplayGTK *App = new CDisplayGTK(filename,argc,argv) ;
    App->create() ; // construct window
    App->exec() ;   // execute window...
}

#else

void startIG(char *filename, int argc, char **argv) {
    CLSMearthApp theApp;
    if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), SW_SHOWNORMAL))
    {
        tconsole.Error()  << _T("Fatal Error: MFC initialization failed") << endl;
        return ;
    }
    theApp.InitApplication() ;
    theApp.InitInstance() ;
    theApp.Run() ;
}

#endif

#endif
