//--project includes--
#include "stdafx.h"
#include "LSMSocket.h"
#include "interface.h"
#include "ModelFnc.h"
#include "remoteConsole.h"
#include "SocketLoop.h"
#include "SocketServer.h"
#include "master_main.h"
#include "Parallel/ParserLmInitialiser.h"

using namespace esys::lsm;

//--- IO includes ---
#include <iostream>
using std::cout;

#ifdef _WINDOWS
#include "Windows/LSMearth.h"
#else
#define MPI_NO_CPPBIND
#include <mpi.h>
#ifndef _WINDOWS
#include <unistd.h>
#include "BackEnd/GtkDisplayServer.h"
#endif
#endif

extern int m_main();
extern int s_main();

void DisplayHelp() 
{
  cout << "\n" ;
  cout << " LSMearthBE [-V [option]] [-help] [-Id name] [-P port] [-nodisplay] [-nf Filename] [-time]\n" ;
  cout << "\n" ;
  cout << "  -v \n" ;
  cout << "      Turn on console (stdout) message output on console.\n" ;
  cout << "  -V [options]\n" ;
  cout << "      Verbose level for message output on console. Options\n" ;
  cout << "      are: all, debug, info, message, warning, error, critical, silent\n" ;
  cout << "      (eg. -v info set the verbose level to info).\n" ;
  cout << "  -help  \n" ;
  cout << "      Display this message\n" ;
  cout << "  -Id \n" ;
  cout << "      Set the Id for the client.\n" ;
  cout << "  -P \n" ;
  cout << "      Set the Port for sockets.\n" ;
  cout << "  -nodisplay \n  " ;
  cout << "      run without a graphical display \n";
  cout << "  -nf Filename \n " ;
  cout << "      run without frontend, immediately executing the given script \n"; 
  cout << "  -time \n" ;
  cout << "      enable display of timing information \n";
  cout << "  -timingfile filename\n" ;
  cout << "      timing information written to specified filename \n";
  cout << "\n" ;
}

void startIG(int argc, char **argv) ;

int LSMmainBE(int argc, char **argv, bool isMaster) {
    
  int i ;
  char *myId ; 
  char *scriptfile;
  int myPort ;
  char defaultId[] = "MyServer" ;
  int defaultPort ;
  bool nodisplay ;
  bool nofe=false ;
  std::string timingFileName;

  // Default values

  defaultPort = sock->minPort() ;
  nodisplay = false ;
  console.SetQuiet(true) ;
  console.SetTiming(false) ;
  myId = &defaultId[0] ;
  myPort = defaultPort ;

  // Read the argument list...


  if (!isMaster) { 
    nodisplay = true ;
    console.SetQuiet(false) ;
  }

  if (argc >=2) {
    for (i=1; i<argc; i++) {
      if (!strcmp(argv[i],"-help")) {
        DisplayHelp() ;
        return true ;
      }
      if (!strcmp(argv[i],"-nodisplay")) {
        nodisplay = true ;
      }
      if (!strcmp(argv[i],"-nf")) {
        nofe = true ;
        i++;
        scriptfile=argv[i];
      }
      if (!strcmp(argv[i],"-v")) {
        console.SetQuiet(false) ;
      }
      if (!strcmp(argv[i],"-time")) {
        console.SetTiming(true) ;
      }
      if (!strcmp(argv[i],"-timingfile")) {
        console.SetTiming(true);
        if ((i+1) < argc) {
          timingFileName = argv[++i];
        }
      }
      if (!strcmp(argv[i],"-V")) {
        console.SetVerbose(BasicCon::dbg) ;
        i++ ;
        if ( i<argc) {
          if (!strcmp(argv[i],"all")) {
            console.SetVerbose(Con::all) ;
          } else if (!strcmp(argv[i],"debug")) {
            console.SetVerbose(Con::dbg) ;
          } else if (!strcmp(argv[i],"info")) {
            console.SetVerbose(Con::info) ;
          } else if (!strcmp(argv[i],"message")) {
            console.SetVerbose(Con::msg) ;
          } else if (!strcmp(argv[i],"warning")) {
            console.SetVerbose(Con::warning) ;
          } else if (!strcmp(argv[i],"error")) {
            console.SetVerbose(Con::err) ;
          } else if (!strcmp(argv[i],"critical")) {
            console.SetVerbose(Con::critical) ;
          } else if (!strcmp(argv[i],"silent")) {
            console.SetVerbose(Con::silent) ;
          } else {
            i-- ;
          }
        } else {
          i-- ;
        }
      }
      if (!strcmp(argv[i],"-Id")) {
        i++ ;
        if (i<argc) {
          myId = argv[i] ;
        } else {
            console.Critical() << "Command line syntax error\n" ;
          DisplayHelp() ;
          return true ;
        }
      } else if (!strcmp(argv[i],"-P")) {
        i++ ;
        if (i<argc) {
          myPort = atol(argv[i]) ;
        } else {
            console.Critical() << "Command line syntax error\n" ;
          DisplayHelp() ;
          return true ;
        }
      }
    }
  }


  if (isMaster) { 
    console.Message(false) << "\n" ;
    console.Message(false) << "LSMearth -- Version 4.0a (c) 1993-2001" << "\n" ;
    console.Message(false) << "\n" ;

    if(nofe){ // run without Front End -> parse and exec srcipt directly
      m_main(); // create and initialize LatticeMaster
      
      ParserLmInitialiser parserInitialiser(*TheLatticeMaster, parserBridge);
      parserBridge.addLocalVariables() ;
      
      TheLatticeMaster->setTimingFileName(timingFileName);
      console.Info() << "reading script " << scriptfile << "\n"; 
      int res = SParserAccess::Parser()->Parse(scriptfile) ;
      if (res) {
        console.Info() << "executing script \n"; 
        res = SParserAccess::Parser()->Eval("main") ;
      }
      TheLatticeMaster->run() ;
    } else {
      sock->maxPort() = (sock->maxPort()-sock->minPort()) + myPort ;
      sock->minPort() = myPort ;

      console.Debug() << "Creating socket for " << myId << "\n" ;
      CShakeSocketServer CS(myId,sock->minPort(),sock->maxPort()) ;
      CSocketLoop MainLoop(CS) ;

      // initialise the socket for this application
      sock = new CBackEndSocketSvc ;
      sock->initAsServer(&MainLoop,&CS) ;

      //sock->server()->create() ;
      m_main() ; 
      ParserLmInitialiser parserInitialiser(*TheLatticeMaster, parserBridge);      
      if (nodisplay) {
        CRemoteConsoleClient newconsole ;
        //int n=0 ;
        while(!sock->mustExit()) {
          usleep(1000) ; // will flood the socket if no sleep
          sock->msgloop() ;
          if ((sock->server()->connected())&& (newconsole.getSocket()==NULL)) {
            newconsole.connectTo(sock->server()) ;
            newconsole.setAsConsole() ;
            // sock->server()->inboundClose() ; // do not listen anymore...
          }
          if (sock->server()->connected()) {
            //usleep(10) ;
            //sock->server()->nonBlocking(true) ;
            //console.Message() << "Hello (" << n++ << ") from server...\n" ;
          } else if (newconsole.getSocket()) {
            newconsole.connectTo(NULL) ;
          }
        }
      } else {
        startIG(argc,argv) ;
      }
    }
  } else {
    s_main() ;   
  }

  return 0 ;
}

#ifndef _WINDOWS

int main(int argc, char **argv) {	
    int myid, numprocs;

    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);
    LSMmainBE(argc,argv,myid==0) ; 
}

void startIG(int argc, char **argv) {
    CDisplaySvcGTK *App = new CDisplaySvcGTK(argc,argv) ;
    App->create() ; // construct window
    App->exec() ;   // execute window...
}

#else

INT WINAPI WinMain(HINSTANCE   hInstance,    // handle to current instance     
                   HINSTANCE   hPrevInstance,// handle to previous instance     
                   LPSTR       lpCmdLine,    // pointer to command line     
                   INT         nShowCmd)     // show state of window 
{ 
    char *args[]={"LSMearth",NULL} ;
    char **argv ;
    argv = args ;
    
    return LSMmainBE(1,argv) ;
}

#endif
