#ifndef __ADHESIVEFRICTION_H
#define __ADHESIVEFRICTION_H

// -- project includes --
#include "FrictionInteraction.h"

/*!
  \struct CAdhesiveFrictionIGP
  \brief  CAdhesivFrictionIGP
  \brief Interaction parameters for adhesive frictional interaction
  \author Steffen Abe
  $Revision$
  $Date$
*/
struct CAdhesiveFrictionIGP
{
  double k;
  double mu;
  double k_s;
  double dt;
  double r_cut;
};

/*!
   \class CAdhesiveFriction
   \brief Frictional+adhesive Elastic interaction between particles
   \author Steffen Abe
   $Revision$
   $Date$
*/
class CAdhesiveFriction : public CFrictionInteraction
{
public: // types
  typedef CAdhesiveFrictionIGP ParameterType;

  typedef double (CAdhesiveFriction::* ScalarFieldFunction)() const;
  typedef pair<bool,double> (CAdhesiveFriction::* CheckedScalarFieldFunction)() const;
  typedef Vec3 (CAdhesiveFriction::* VectorFieldFunction)() const;

 protected:
  double m_r_cut;
  double m_r_cut_h;
 
 public:
  CAdhesiveFriction();
  CAdhesiveFriction(CParticle*,CParticle*,const CAdhesiveFrictionIGP&);
  virtual ~CAdhesiveFriction();

  virtual void calcForces();
 
  friend class TML_PackedMessageInterface;
};

#endif // __ADHESIVEFRICTION_H
