#ifndef __BWALLINTERACTIONGROUP_H
#define __BWALLINTERACTIONGROUP_H

//--- project includes ---
#include "Model/BWallInteraction.h"
#include "Model/EWallInteraction.h"
#include "Model/WallIG.h"
#include "Model/EWallInteractionGroup.h"
#include "Fields/Field.h"

//--- STL includes ---
#include <map>

using std::map;

template <class T> class ParallelParticleArray;

/*!
  \class CBWallIGP
  \brief Interaction group parameters for CBWallInteractionGroups

  \author Steffen Abe
  $Revision$
  $Date$
*/
class CBWallIGP : public CEWallIGP
{
 protected:
  int m_tag;
 public:
  virtual void  packInto(CVarMPIBuffer*) const;
  void setTag(int tag){m_tag=tag;};
  int getTag()const{return m_tag;};

  friend ostream& operator<<(ostream&,const CBWallIGP&);
};

CBWallIGP* extractBWallIGP(AMPIBuffer*);


/*!
  \class CBWallInteractionGroup
  \brief Class for a group of bonded,elastic interactions between particles and a wall

  \author Steffen Abe
  $Revision$.
  $Date$
*/
template<class T>
class CBWallInteractionGroup : public AWallInteractionGroup<T>
{
 protected:
  vector<CBondedWallInteraction<T> > m_bonded_interactions; //!< bonded interactions for tagged particles
  vector<CElasticWallInteraction<T> > m_elastic_interactions; //!< elastic interactions for the rest
  double m_k; //!< spring constant
  int m_tag;
  
 public:
  CBWallInteractionGroup(TML_Comm*);
  CBWallInteractionGroup(TML_Comm*,const CBWallIGP*);
  virtual ~CBWallInteractionGroup(){if (m_wall!=NULL) delete m_wall;};

  virtual void calcForces();
  virtual void applyForce(const Vec3&);
  virtual void Update(ParallelParticleArray<T>*);

  void addToFields(map<string,AField*>*);

  friend ostream& operator<< <>(ostream&,const CBWallInteractionGroup&);
};

#include  "BWallInteractionGroup.hpp"

#endif //__BWALLINTERACTIONGROUP_H
