/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef ESYS_LSMBODYFORCEGROUP_H
#define ESYS_LSMBODYFORCEGROUP_H

#include "Foundation/vec3.h"
#include "Model/InteractionGroup.h"

template <class TmplParticle> class ParallelParticleArray;

namespace esys
{
  namespace lsm
  {
    class BodyForceIGP : public AIGParam
    {
    public:

      BodyForceIGP();

      BodyForceIGP(const std::string &name, const Vec3 &acceleration);

      virtual ~BodyForceIGP();

      const Vec3 &getAcceleration() const;

      const std::string &getName() const;

      void packInto(CVarMPIBuffer *pBuffer) const;

      static BodyForceIGP extract(CVarMPIBuffer *pBuffer);

    private:
      Vec3 m_acceleration;
    };

    /**
     * Objects of this class apply a gravitational body-acceleration
     * to individual particles.
     */
    template <class TmplParticle>
    class BodyForceGroup : public AInteractionGroup<TmplParticle>
    {
    public:
      typedef ParallelParticleArray<TmplParticle> ParticleArray;
      typedef typename ParticleArray::ParticleListIterator ParticleIterator;

      BodyForceGroup(const BodyForceIGP &prms, ParticleArray &particleArray);

      ~BodyForceGroup();

      /**
       * Returns the force which would be applied to a
       * particle of the specified mass.
       *
       * @param mass A mass ("units" assumed to be same as
       *             the acceleration units).
       */
      Vec3 getForce(double mass) const;

      /**
       * Applies body force to the specified particle.
       *
       * @param particle Force applied to this particle using
       *                 a call to particle.applyForce(...).
       */
      void applyForce(TmplParticle &particle) const;

      virtual void Update(ParallelParticleArray<TmplParticle> *particleArray);

      virtual void calcForces();

    private:
      Vec3          m_acceleration;
      ParticleArray *m_pParticleArray;
    };
  };
};

#include "Model/BodyForceGroup.hpp"

#endif
