
#ifndef MODEL_EWALLINTERACTION_HPP
#define MODEL_EWALLINTERACTION_HPP

/*!
  constructor for elastic interaction between particle & wall

  \param p pointer to the particle
  \param w pointer to the wall
  \param k spring constant
  \param iflag flag if the particle is in the inner part of the local NTable
*/
template <class T>
CElasticWallInteraction<T>::CElasticWallInteraction(T* p,CWall* w,double k,bool iflag):
  AWallInteraction<T>(p,w,iflag)
{
  m_k=k;
}

/*!
  calculate free elastic forces.
*/
template <class T>
void CElasticWallInteraction<T>::calcForces()
{


  double dist=(m_p->getPos()-m_wall->getOrigin())*m_wall->getNormal(); 

  if(dist<m_p->getRad()){
    Vec3 force=m_k*(m_p->getRad()-dist)*m_wall->getNormal();
    Vec3 pos=m_p->getPos()-dist*m_wall->getNormal();

 /*   Vec3 disp = m_p->getPos()- m_p->getOldPos() ;
    Vec3 x_d = Vec3(1.0,0.0,0.0);
    Vec3 y_d = Vec3(0.0,1.0,0.0);
    Vec3 fy_vert = -m_k*disp.Y()*y_d;
    Vec3 fx_vert = -m_k*disp.X()*x_d;
    force = force+ fx_vert + fy_vert ;*/


// friction wall 
/*
     Vec3 x_d = Vec3(1.0,0.0,0.0);
     Vec3 y_d = Vec3(0.0,1.0,0.0);
     Vec3 fx_vert, fy_vert ;
     double miu = 1.0; 

    if ( m_p->getVel().X() >0.0)  fx_vert = - miu*force.norm()*x_d;
    else fx_vert =  miu*force.norm()*x_d;

    if ( m_p->getVel().Y() >0.0)  fy_vert = - miu*force.norm()*y_d;
    else fy_vert =  miu*force.norm()*y_d;
     
    force = force+ fx_vert + fy_vert ;
 */
    m_p->applyForce(force,pos);
    if(m_inner_flag) m_wall->addForce(-1.0*force);
  }
}

/*!
  calculate & return free elastic forces, don't apply them
*/
template <class T>
Vec3 CElasticWallInteraction<T>::getForce()
{
  Vec3 force=Vec3(0.0,0.0,0.0);
  double dist=(m_p->getPos()-m_wall->getOrigin())*m_wall->getNormal();
  if(dist<m_p->getRad()){
    force=m_k*(m_p->getRad()-dist)*m_wall->getNormal();
  }

  return -1.0*force;
}

#endif
