#ifndef __EWALLINTERACTIONGROUP_H
#define __EWALLINTERACTIONGROUP_H

//--- project includes ---
#include "Model/EWallInteraction.cpp"
#include "Model/WallIG.h"
#include "Model/ElasticInteractionGroup.h"
#include "Model/IGParam.h"
#include "Fields/Field.h"

template <class T> class ParallelParticleArray;

//--- STL includes ---
#include <map>

using std::map;

/*!
  \class CEWallIGP
  \brief Interaction group parameters for CEWallInteractionGroups

  \author Steffen Abe
  $Revision$
  $Date$
*/
class CEWallIGP : public CElasticIGP
{
 protected:
  Vec3 m_ipos,m_inorm; //<! initial wall position/normal
 public:
  virtual void  packInto(CVarMPIBuffer*) const;
  void setWall(const Vec3& p,const Vec3& n){m_ipos=p;m_inorm=n;};
  Vec3 getWallPos() const{return m_ipos;};
  Vec3 getWallNormal() const{return m_inorm;};

  friend ostream& operator<<(ostream&,const CEWallIGP&);
};

CEWallIGP* extractEWallIGP(AMPIBuffer*);
CEWallIGP* extractEWallIGP_p(AMPIBuffer*);

/*!
  \class CEWallInteractionGroup
  \brief Class for a group of unbonded,elastic interactions between particles and a wall

  \author Steffen Abe
  $Revision$.
  $Date$
*/
template<class T>
class CEWallInteractionGroup : public AWallInteractionGroup<T>
{
 protected:
  vector<CElasticWallInteraction<T> > m_interactions;
  double m_k; //!< spring constant

 public:
  CEWallInteractionGroup(TML_Comm*);
  CEWallInteractionGroup(TML_Comm*,const CEWallIGP*);
  virtual ~CEWallInteractionGroup(){if (m_wall!=NULL) delete m_wall;};

  virtual void calcForces();
  virtual void applyForce(const Vec3&);
  virtual void Update(ParallelParticleArray<T>*);

  void addToFields(map<string,AField*>*);

  friend ostream& operator<< <>(ostream&,const CEWallInteractionGroup&);
};

#include  "EWallInteractionGroup.hpp"

#endif //__EWALLINTERACTIONGROUP_H
