#include "ElasticInteraction.h"
#include "console.h"


CElasticInteraction::CElasticInteraction(CParticle* p1,CParticle* p2,const CElasticIGP& param):APairInteraction(p1,p2)
{
  m_k=param.m_k;
  m_force=Vec3(0.0,0.0,0.0);
}

/*!
  Calculate free elastic forces. 23 Flops if in contact, 10 Flops if not
 */
void CElasticInteraction::calcForces()
{
  //  console.XDebug() << "elastic interaction: [" << m_p1->getID() << " - " << m_p2->getID() << "]" << m_p1->getPos() << m_p2->getPos() << "\n"; 
  Vec3 D=m_p1->getPos()-m_p2->getPos();
  double dist=D*D;
  double eq_dist=m_p1->getRad()+m_p2->getRad();
  if(dist<(eq_dist*eq_dist)){
    dist=sqrt(dist);
    m_force=D*(m_k*(dist-eq_dist)/dist);
    Vec3 pos=m_p2->getPos()+(m_p2->getRad()/eq_dist)*D;
    m_p2->applyForce(m_force,pos);
    m_p1->applyForce(-1.0*m_force,pos); 
  }
}

/*!
  get the potential energy stored in the interaction
*/
double CElasticInteraction::getPotentialEnergy() const
{
  double e_pot_norm=0.5*m_force*m_force/m_k;

  return e_pot_norm;
}

ostream& operator<<(ostream& ost,const CElasticInteraction& BI)
{
  ost << "[" << BI.m_p1->getID() << " - " << BI.m_p2->getID() << "]";
  return ost;
}

/*!
  Get the particle member function which returns a scalar field of a given name.
 
  \param name the name of the field
*/
CElasticInteraction::ScalarFieldFunction CElasticInteraction::getScalarFieldFunction(const string& name)
{
  CElasticInteraction::ScalarFieldFunction sf;

  if (name=="potential_energy")
    {
      sf=&CElasticInteraction::getPotentialEnergy;
    }
  else if (name=="count")
    {
      sf=&CElasticInteraction::Count;
    }
  else
    {
      sf=NULL;
      cerr << "ERROR - invalid name for interaction scalar  access function" << endl;
    }
  
  return sf;
}

/*!
  Get the particle member function which returns a checked scalar field of a given name.

  \param name the name of the field
*/
CElasticInteraction::CheckedScalarFieldFunction CElasticInteraction::getCheckedScalarFieldFunction(const string& name)
{
  CElasticInteraction::CheckedScalarFieldFunction sf;

	
  sf=NULL;
  cerr << "ERROR - invalid name for interaction scalar  access function" << endl;
  
  return sf;
}


/*!
  Get the particle member function which returns a vector field of a given name.
 
  \param name the name of the field
*/
CElasticInteraction::VectorFieldFunction CElasticInteraction::getVectorFieldFunction(const string& name)
{
  CElasticInteraction::VectorFieldFunction vf;

  vf=NULL;
  cerr << "ERROR - invalid name for interaction vector access function" << endl;
  
  return vf;
}
