#ifndef __ELASTICINTERACTION_H
#define __ELASTICINTERACTION_H

#include "Interaction.h"
#include "Particle.h"
#include "IGParam.h"


/*!
  \class CElasticIGP
  \brief Interaction group parameters for CElasticInteractionGroups

  \author Steffen Abe
  $Revision$
  $Date$
*/
class CElasticIGP : public AIGParam
{
 protected:
 public:
  double m_k;

  virtual void  packInto(CVarMPIBuffer*) const;
  void setSpringConst(double k){m_k=k;};
  double getSpringConst() const{return m_k;};

  friend ostream& operator<<(ostream&,const CElasticIGP&);
};

CElasticIGP* extractElasticIGP(AMPIBuffer*);
CElasticIGP* extractElasticIGP_p(AMPIBuffer*);

/*!
  \class CElasticInteraction
  \brief Elastic Inteaction between free particles
  \author Steffen Abe
  $Revision$
  $Date$
*/
class CElasticInteraction : public APairInteraction
{
public:

  typedef double (CElasticInteraction::* ScalarFieldFunction)() const;
  typedef pair<bool,double> (CElasticInteraction::* CheckedScalarFieldFunction)() const;
  typedef Vec3 (CElasticInteraction::* VectorFieldFunction)() const;

  static ScalarFieldFunction getScalarFieldFunction(const string&);
  static CheckedScalarFieldFunction getCheckedScalarFieldFunction(const string&);
  static VectorFieldFunction getVectorFieldFunction(const string&);

private:
  double m_k;//!< spring constant
  Vec3 m_force; // caching force for E_pot
  Vec3 m_cpos;

public:
  typedef CElasticIGP ParameterType;

  CElasticInteraction(CParticle*,CParticle*,const CElasticIGP&);
  virtual ~CElasticInteraction(){};

  virtual Vec3 getPos(){return m_cpos;};
  double getPotentialEnergy() const;

  virtual void calcForces();
  friend ostream& operator<<(ostream&,const CElasticInteraction&);
};
#endif //__ELASTICINTERACTION_H
