#ifndef __FRACTALFRICTION_H
#define __FRACTALFRICTION_H

// -- project includes --
#include "FrictionInteraction.h"

/*!
  \struct FractalFrictionIGP
  \brief Interaction parameters for frictional interaction with a fractal 
  distribution of the coefficient of friction
  \author Steffen Abe

  $Revision$
  $Date$
*/
struct FractalFrictionIGP
{
  double k;
  double mu_0;
  double k_s;
  double dt;
  std::auto_ptr<double> mu;          //!< pointer to the array of friction coeff.
  double x0,y0,dx,dy;  //!< origin and grid spacing of the array
  int nx,ny;           //!< array size

  FractalFrictionIGP(){};
  FractalFrictionIGP(const FractalFrictionIGP &);
  ~FractalFrictionIGP();
  
  FractalFrictionIGP &operator=(const FractalFrictionIGP &);
};

/*!
  \class CFractalFriction
  \brief Frictional+Elastic interaction between particles with fractal distribution of the 
  coefficient of friction

  \author Steffen Abe
   $Revision$
   $Date$
*/
class CFractalFriction : public CFrictionInteraction
{
 public: // types
  typedef FractalFrictionIGP ParameterType;

  typedef double (CFractalFriction::* ScalarFieldFunction)() const; 
  typedef Vec3 (CFractalFriction::* VectorFieldFunction)() const; 
	typedef pair<bool,double> (CFractalFriction::* CheckedScalarFieldFunction)() const;
 private:

 public:
  CFractalFriction();
  CFractalFriction(CParticle*,CParticle*,const FractalFrictionIGP&);
  virtual ~CFractalFriction();

  static ScalarFieldFunction getScalarFieldFunction(const string&);
  static VectorFieldFunction getVectorFieldFunction(const string&);
	static CheckedScalarFieldFunction getCheckedScalarFieldFunction(const string&);
  friend ostream& operator<<(ostream&,const CFractalFriction&);
  friend class TML_PackedMessageInterface;
};

#endif //__FRACTALFRICTION_H
