#ifndef __INTERACTION_H
#define __INTERACTION_H

// -- project includes --
#include "Particle.h"

// -- STL includes --
#include <utility>
#include <vector>
using std::vector;
using std::pair;
using std::make_pair;


/*!
  \class AInteraction
  \brief Abstract base class for interactions
  \author Steffen Abe

  $Revision$
  $Date$
*/
class AInteraction
{
 protected:
  vector<int> m_id; //!< id's of the particles involved
  int m_iid;  //!< interaction id
  bool m_init;

 public:
  AInteraction();
  virtual ~AInteraction();

  bool initialized() const;
  virtual void calcForces()=0;
  vector<int> getAllID() const;
  int getID(){return m_iid;};
  virtual bool hasTag(int,int) const=0;
  virtual Vec3 getPosFirst() const=0;
  virtual Vec3 getPosSecond() const{return Vec3(0.0,0.0,0.0);};
  inline double Count() const {return 1.0;};
};


/*!
  \class APairInteraction
  \brief Abstract base class for 2-particle interactions

  \author Steffen Abe
  $Revision$
  $Date$
*/
class APairInteraction : public AInteraction
{
 protected:
  CParticle *m_p1,*m_p2;

 public:
  // functions 
  APairInteraction();
  APairInteraction(CParticle*,CParticle*);
  virtual ~APairInteraction();

  inline const CParticle* first()const {return m_p1;}
  inline const CParticle* second()const {return m_p2;}
  inline CParticle* first() {return m_p1;}
  inline CParticle* second() {return m_p2;}
  
  inline pair<int,int> getPairID() const {return make_pair(m_p1->getID(),m_p2->getID());};
  virtual Vec3 getPos()=0;
  virtual void calcForces()=0;
  void checkIDs();
  virtual bool hasTag(int,int) const;
  virtual Vec3 getPosFirst() const {return m_p1->getPos();};
  virtual Vec3 getPosSecond() const{return m_p2->getPos();};

  template <class TmplParticle> void setPP(const vector<TmplParticle *> &pp)
  {
    m_p1=pp[0];
    m_p2=pp[1];
    m_id.clear();
    m_id.push_back(m_p1->getID());
    m_id.push_back(m_p2->getID());
  }
};
#endif
