#include "MeshData.h"

// --- TML includes ---
#include "tml/message/packed_message_interface.h"

// === NODE DATA ===========

/*!
  read node data from istream

  \param instream the stream to read from
*/
void MeshNodeData::read(istream& instream)
{
  int dummy;
  instream >> id >> dummy >> tag >> x >> y >> z;
}


/*!
  Pack MeshNodeData into a TML packed message

  \param d the data
*/
template<>
void TML_PackedMessageInterface::pack<MeshNodeData>(const MeshNodeData& d)
{
  append(d.id);
  append(d.tag);
  append(d.x);
  append(d.y);
  append(d.z);
}

/*!
  Unpack MeshNodeData from a TML packed message
*/
template<>
void TML_PackedMessageInterface::unpack<MeshNodeData>(MeshNodeData& d)
{
  d.id=pop_int();
  d.tag=pop_int();
  d.x=pop_double();
  d.y=pop_double();
  d.z=pop_double();
}


// === TRI DATA ===========

/*!
  read triangle data from istream

  \param instream the stream to read from
*/
void MeshTriData::read(istream& instream)
{
  instream >> id >> tag >> p1 >> p2 >> p3;
}

/*!
  Pack MeshTriData into a TML packed message

  \param d the data
*/
template<>
void TML_PackedMessageInterface::pack<MeshTriData>(const MeshTriData& d)
{
  append(d.id);
  append(d.tag);
  append(d.p1);
  append(d.p2);
  append(d.p3);
}

/*!
  Unpack MeshTriData from a TML packed message
*/
template<>
void TML_PackedMessageInterface::unpack<MeshTriData>(MeshTriData& d)
{
  d.id=pop_int();
  d.tag=pop_int();
  d.p1=pop_int();
  d.p2=pop_int();
  d.p3=pop_int();
}
