/********************************************************************************/
/* ModelFNC.cpp                                                                 */
/*                                                                              */
/* Description:   Parser functions                                              */
/* Author(s):     D. Place (QUAKES)                                             */
/* Creation date: November 2001                                                 */
/*                                                                              */
/********************************************************************************/

#include "stdafx.h"
#include <stdio.h>
#include "interface.h"
#include "ModelFnc.h"
#include "console.h"

CParserBridge parserBridge ;

class PF_AGlobalFunction : public PredefinedFunction {
public:
    PF_AGlobalFunction() : PredefinedFunction(0,OpValue::NONE) { } ;
    OpValue fn(ExpressionNode *) {
        SParserAccess::Interface()->GetAllValues() ;
	console.Debug() << "Calling THE Global function\n" ;
        SParserAccess::Interface()->SetAllValues() ;
        return OpValue(1L) ;

    } ;
} ;

class PF_AModelFunction : public PredefinedFunction {
public:
    PF_AModelFunction() : PredefinedFunction(0,OpValue::NONE) { } ;
    OpValue fn(ExpressionNode *) {
        SParserAccess::Interface()->GetAllValues() ;
	console.Debug() << "Calling THE Model function\n" ;
        SParserAccess::Interface()->SetAllValues() ;
        return OpValue(1L) ;
    } ;
} ;

CParserBridge::CParserBridge() {
}

void CParserBridge::addLocalInterface(AModelInterface *AMI) {
  m_Locals.Append(AMI) ;
}

void CParserBridge::addGlobalInterface(AModelInterface *AMI) {
  m_Globals.Append(AMI) ;
}

void CParserBridge::addLocalVariables() {
  m_Locals.First() ; 
  while (!m_Locals.IsEnd()) {
    ISVALID_PTR(SParserAccess::Interface()->GetVarTable()) ;
    ISVALID_PTR(m_Locals.Get());
    //cout << "add variables..." << endl << flush ;
    m_Locals.Get()->AddVariablesTo(*SParserAccess::Interface()->GetVarTable()) ;
    m_Locals.Next() ;
  }
}

void CParserBridge::addGlobalVariables() {
  m_Globals.First() ; 
  while (!m_Globals.IsEnd()) {
    ISVALID_PTR(SParserAccess::Interface()->GetVarTable()) ;
    ISVALID_PTR(m_Globals.Get());
    m_Globals.Get()->AddVariablesTo(*SParserAccess::Interface()->GetVarTable()) ;
    m_Globals.Next() ;
  }
}

void CParserBridge::removeLocalVariables() {
  m_Locals.First() ; 
  while (!m_Locals.IsEnd()) {
    ISVALID_PTR(SParserAccess::Interface()->GetVarTable()) ;
    ISVALID_PTR(m_Locals.Get());
    m_Locals.Get()->DelVariablesFrom(*SParserAccess::Interface()->GetVarTable()) ;
    m_Locals.Next() ;
  }
}

void CParserBridge::removeGlobalVariables() {
  m_Globals.First() ; 
  while (!m_Globals.IsEnd()) {
    ISVALID_PTR(SParserAccess::Interface()->GetVarTable()) ;
    ISVALID_PTR(m_Globals.Get());
    m_Globals.Get()->DelVariablesFrom(*SParserAccess::Interface()->GetVarTable()) ;
    m_Globals.Next() ;
  }
}


void CGlobalFunctions::AddVariablesTo(InternVFTable &P) {
    ADDFN(PF_AGlobalFunction,"AGlobalFunction") ;
}

void CGlobalFunctions::DelVariablesFrom(InternVFTable &P) {
    P.DelFnc("AGlobalFunction") ;
}

void CModelFunctions::AddVariablesTo(InternVFTable &P) {
    ADDFN(PF_AModelFunction,"AModelFunction") ;
}

void CModelFunctions::DelVariablesFrom(InternVFTable &P) {
    P.DelFnc("AModelFunction") ;
}

void CModelVariables::AddVariablesTo(InternVFTable &) {
  //    P.AddVar("Lattice.Origin.x",&Origin.x) ;
}

void CModelVariables::DelVariablesFrom(InternVFTable &) {
  //    P.DelVar("Lattice.Integration.dt") ;
} 

