#ifndef __ROTBONDEDINTERACTION_H
#define __ROTBONDEDINTERACTION_H

// -- project includes --
#include "Model/RotPairInteraction.h"
#include "Model/RotParticle.h"
#include "Model/InteractionParam.h"
#include "Foundation/vec3.h"

// -- I/O includes --
#include <iostream>
using std::ostream;

/*!
  \struct CRotBondedIGP
  \brief Interaction parameters for bonded interaction between rotational particles
  \author Shane Latham, Steffen Abe
  $Revision$
  $Date$
*/

 double calc_angle( double , double ) ;

 struct CRotBondedIGP
{
  double kr,ks,kt,kb ;
  double max_nForce, max_shFroce,max_tMoment, max_bMoment   ;
};


/*!
   \class CRotBondedInteraction
   \brief Elastic interaction between bonded particles between rotational particles
   \author Shane Latham, Steffen Abe
   $Revision$
   $Date$
*/
class CRotBondedInteraction : public ARotPairInteraction
{
 public: // types
  typedef CRotBondedIGP ParameterType;

  typedef double (CRotBondedInteraction::* ScalarFieldFunction)() const; 
  typedef pair<bool,double> (CRotBondedInteraction::* CheckedScalarFieldFunction)() const;
  typedef Vec3 (CRotBondedInteraction::* VectorFieldFunction)() const; 

 private:

  //   protected:
  double m_dist;  //!< current distance, cached from last calcForces()

  double m_kr ;     //!< spring constant
  double m_ks ;
  double m_kb ;
  double m_kt ;

  double m_max_nForce;   // always >0
  double m_max_shForce ;
  double m_max_tMoment ;
  double m_max_bMoment ;

  double m_nForce;  //  >0, pulling; <0 , compressing
  double m_shForce ; // always >0
  double m_tMoment ;
  double m_bMoment ;

  Vec3 m_force;   //!< current force, cached for E_pot calculation
  Vec3 m_moment ;

  Vec3 m_cpos; // ?

 public:

  CRotBondedInteraction();
  CRotBondedInteraction(CRotParticle*,CRotParticle*,const CRotBondedIGP&);
  virtual ~CRotBondedInteraction();
                                                                                
  static ScalarFieldFunction getScalarFieldFunction(const string&);
  static CheckedScalarFieldFunction getCheckedScalarFieldFunction(const string&);
  static VectorFieldFunction getVectorFieldFunction(const string&);

  void calcForces();
  //void setBreak(double);
  bool broken();

  double getPotentialEnergy() const;

  virtual Vec3 getPos(){return m_cpos;};
  
  friend ostream& operator<<(ostream&,const CRotBondedInteraction&);
  friend class TML_PackedMessageInterface;
};


#endif //__BONDEDINTERACTION_H
