#include "RotDamping.h"
#include "console.h"

double CRotDamping::s_limit2=1e-12; // default error limit : 1e-6 
int CRotDamping::s_flops;

/*!
  Construct a damping "interaction" for a particle

  \param P the particle
  \param V the reference velocity
  \param visc the artificial viscosity
  \param dt the time step 
  \param mi the maximum number of iterations
 */
CRotDamping::CRotDamping(CRotParticle* P,const Vec3& V,double visc,double dt,int mi)
{
  m_p=P;
  m_vref=V;
  m_visc=visc;
  m_dt=dt;
  m_maxiter=mi;
}

CRotDamping::~CRotDamping()
{}

/*!
  Calculate the damping force.

  25*count+8 flops
*/
void CRotDamping::calcForces()
{
  Vec3 v=m_p->getVel();
  Vec3 v_rel=Vec3(0.0,0.0,0.0);
  Vec3 frc=m_p->getForce();
  double s=1.0/m_p->getMass();                       // 1 flop
  double error=1.0;
  int count=0;
  while((error*error>s_limit2) & (count<m_maxiter)){ // 1 flop
    count++;
    Vec3 v_old=v_rel;
    v_rel=v-m_vref+s*m_dt*(frc-v_rel*m_visc);        // 16 flops
    error=(v_rel-v_old).norm2();                     // 8 flops  
  }
  if(count>=m_maxiter){
    //console.Warning() << "damping diverges for particle  " << m_p->getID() << "error= " << error << "\n";
    v_rel=Vec3(0.0,0.0,0.0);
  }
  m_p->applyForce(-1.0*m_visc*v_rel,m_p->getPos());  // 3 flops
  CRotDamping::s_flops+=25*count+8;
}
