#ifndef __ROTDAMPING_H
#define __ROTDAMPING_H

// -- project includes --
#include "vec3.h"
#include "Interaction.h"
#include "RotParticle.h"

/*!
  \class CDamping
  \brief Damping of the particle motion by an artificial viscosity

  \author Steffen Abe
  $Revision:&
  $Date$
*/

class CRotDamping
{
protected:
  CRotParticle *m_p; //!< the particle
  Vec3 m_vref; //!< reference velocity
  double m_visc; //!< artificial viscosity
  double m_dt;   //!< time step
  int m_maxiter;   //!< iteration limit
 
  static double s_limit2; //!< square error limit for iteration 
  static int s_flops;       

public:
  CRotDamping(CRotParticle*,const Vec3&,double,double,int);
  virtual ~CRotDamping();

  inline void setLimit(double limit){s_limit2=limit*limit;};
  virtual void calcForces();

  static void zeroFlops(){s_flops=0;};
  static int Flops(){return s_flops;};
};

#endif //__DAMPING_H
