#ifndef __ROTDAMPINGGROUP_H
#define __ROTDAMPINGGROUP_H

//-- project includes --
#include "Model/InteractionGroup.h"
#include "Model/RotDamping.h"
#include "Foundation/vec3.h"
#include "ppa/src/pp_array.h"

#include "mpibuf.h"

//-- STL includes --
#include <vector>

using std::vector;

/*!
  \class CDampingIGP
  \brief Interaction group parameters for CDampingGroup

  \author Steffen Abe
  $Revision$
  $Date$
*/ 
class CRotDampingIGP : public AIGParam
{
protected:
  Vec3 m_vref; //!< reference velocity
  double m_visc; //!< artificial viscosity
  double m_dt;   //!< time step
  int m_max_iter; //!< max nr. of iterations

public:
  virtual void  packInto(CVarMPIBuffer*) const;
  void setVRef(const Vec3 V){m_vref=V;};
  Vec3 getVRef()const{return m_vref;};
  void setVisc(double v){m_visc=v;};
  double getVisc()const{return m_visc;};
  void setTimeStep(double t){m_dt=t;};
  double getTimeStep()const{return m_dt;};
  void setMaxIter(int mi){m_max_iter=mi;};
  int getMaxIter()const {return m_max_iter;};
};

CRotDampingIGP* extractRotDampingIGP(AMPIBuffer*);

/*!
  \class CDampingGroup
  \brief Class for group of single particle interactions implementing the damping of particle motion by artificial viscosity

  \author Steffen Abe
  $Revision$
  $Date$
*/ 
template <class T>
class CRotDampingGroup : public AInteractionGroup<T>
{
private:
  vector<CRotDamping> m_interactions;
  Vec3 m_vref;
  double m_visc;
  double m_dt;
  int m_max_iter;

public:
  CRotDampingGroup();
  virtual ~CRotDampingGroup();

  void setParam(const CRotDampingIGP*);
  virtual void addInteraction(T*);
  virtual void calcForces();
  virtual void Update(ParallelParticleArray<T>*);
};

#include "Model/RotDampingGroup.hpp"

#endif //__ROTDAMPINGGROUP_H
