//----------------------------------------------
//       CDampingGroup functions
//----------------------------------------------

template <class T>
CRotDampingGroup<T>::CRotDampingGroup()
{}

template <class T>
CRotDampingGroup<T>::~CRotDampingGroup()
{}

/*!
  set the parameters for the damping group

  \param Param the parameters
*/
template <class T>
void CRotDampingGroup<T>::setParam(const CRotDampingIGP* Param)
{
  m_vref=Param->getVRef();
  m_visc=Param->getVisc();
  m_dt=Param->getTimeStep();
  m_max_iter=Param->getMaxIter();
  console.XDebug()<< "damping group params: vref,visc,dt,max_iter: " <<  m_vref << " | " << m_visc << " | " << m_dt << " | " << m_max_iter << "\n";
}

/*!
  Update damping group from PPA, i.e. add all local particles to it
*/
template <class T>
void CRotDampingGroup<T>::Update(ParallelParticleArray<T>* PPA)
{
  console.XDebug() << "CDampingGroup<T>::Update()\n" ;
  // empty particle list first
  m_interactions.erase(m_interactions.begin(),m_interactions.end());
  // build new particle list
  typename ParallelParticleArray<T>::ParticleListHandle plh=PPA->getAllParticles();
  for(typename ParallelParticleArray<T>::ParticleListIterator iter=plh->begin();
      iter!=plh->end();
      iter++){
    addInteraction(*iter);
  }
  console.XDebug() << "end CDampingGroup<T>::Update()\n" ;
}

/*!
  Add a particle to the damping group

  \param P the particle
*/
template <class T>
void CRotDampingGroup<T>::addInteraction(T* P)
{
  m_interactions.push_back(CRotDamping(P,m_vref,m_visc,m_dt,m_max_iter));
}

/*!
  calculate the forces
*/
template <class T>
void CRotDampingGroup<T>::calcForces()
{
  CRotDamping::zeroFlops();
  for(vector<CRotDamping>::iterator it=m_interactions.begin();it!=m_interactions.end();it++){
    it->calcForces();
  } 
  console.XDebug()<< "damping forces: " << CRotDamping::Flops() << " Flops\n";
}
