#ifndef __ROTELASTICINTERACTION_H
#define __ROTELASTICINTERACTION_H

#include "RotPairInteraction.h"
#include "RotParticle.h"
#include "IGParam.h"


/*!
  \class CRotElasticIGP
  \brief Interaction group parameters for CRotElasticInteractionGroups

  \author Shane Latham
  $Revision$
  $Date$
*/
class CRotElasticIGP : public AIGParam
{
 protected:
 public:
  double m_kr;

  virtual void  packInto(CVarMPIBuffer*) const;
  void setSpringConst(double k){m_kr=k;};
  double getSpringConst() const{return m_kr;};

  friend ostream& operator<<(ostream&,const CRotElasticIGP&);
};

CRotElasticIGP* extractRotElasticIGP(AMPIBuffer*);
CRotElasticIGP* extractRotElasticIGP_p(AMPIBuffer*);

/*!
  \class CRotElasticInteraction
  \brief Elastic Interaction between free rotational particles
  \author Shane Latham, Steffen Abe
  $Revision$
  $Date$
*/
class CRotElasticInteraction : public  ARotPairInteraction
{
public:
  typedef double (CRotElasticInteraction::* ScalarFieldFunction)() const;
  typedef pair<bool,double> (CRotElasticInteraction::* CheckedScalarFieldFunction)() const;
  typedef Vec3 (CRotElasticInteraction::* VectorFieldFunction)() const;

  static ScalarFieldFunction getScalarFieldFunction(const string&);
  static CheckedScalarFieldFunction getCheckedScalarFieldFunction(const string&);
  static VectorFieldFunction getVectorFieldFunction(const string&);

private:
  double m_kr;//!< spring constant
  Vec3 m_cpos;

public:
  typedef CRotElasticIGP ParameterType;

  CRotElasticInteraction(CRotParticle*,CRotParticle*,double); //!<obsolete
  CRotElasticInteraction(CRotParticle*,CRotParticle*,const CRotElasticIGP&);
  virtual ~CRotElasticInteraction(){};

  virtual Vec3 getPos(){return m_cpos;};

  virtual void calcForces();

  Vec3 getForce() const;
  double getPotentialEnergy() const;

  friend ostream& operator<<(ostream&,const CRotElasticInteraction&);
};
#endif //__ELASTICINTERACTION_H
