#ifndef __ROTFRICTIONINTERACTION_H
#define __ROTFRICTIONINTERACTION_H

// -- project includes --
#include "Model/RotPairInteraction.h"
#include "Model/RotParticle.h"
#include "Model/InteractionParam.h"
#include "Foundation/vec3.h"

// -- I/O includes --
#include <iostream>
using std::ostream;


//double calc_angle( double , double ) ;


/*!
  \struct CRotFrictionIGP
  \brief Interaction parameters for frictional interaction between rotational particles
  \author Shane Latham, Steffen Abe
  $Revision$
  $Date$
*/
struct CRotFrictionIGP
{
  double k;
  double mu_d;    // sliding frictional coefficient
  double mu_s;     // max static frictional coefficient
  double k_s;
  double dt;
};

/*!
   \class CRotFrictionInteraction
   \brief Frictional+Elastic interaction between particles between rotational particles
   \author Shane Latham, Steffen Abe
   $Revision$
   $Date$
*/
class CRotFrictionInteraction : public ARotPairInteraction
{
 public: // types
  typedef CRotFrictionIGP ParameterType;

  typedef double (CRotFrictionInteraction::* ScalarFieldFunction)() const;
  typedef pair<bool,double> (CRotFrictionInteraction::* CheckedScalarFieldFunction)() const;  
  typedef Vec3 (CRotFrictionInteraction::* VectorFieldFunction)() const;

  static CheckedScalarFieldFunction getCheckedScalarFieldFunction(const string&);
  static ScalarFieldFunction getScalarFieldFunction(const string&);
  static VectorFieldFunction getVectorFieldFunction(const string&);

// protected:
 private:

  double m_k;     //!< spring constant
  double m_r0;    //!< equilibrium distance
  double m_mu_d;    //!< coefficient of friction
  double m_mu_s;
  double m_ks;    //!< shear stiffness (Cundall)
  double m_dt;    //!< time step
  Vec3 m_Ffric;   //!< current frictional force
  Vec3 m_force_deficit; //!< difference between fric. force & force necessary for slip
  Vec3 m_cpos; //!< contact position
  Vec3 m_normal_force; //!< current normal force
  bool m_is_slipping; //!< static/dynamic status of the interaction
  bool m_is_touching; //!< contact status of the interaction
  
  //Quaternion m_init_q1, m_init_q2;
  //Vec3 m_init_pos1 , m_init_pos2;

 public:
  CRotFrictionInteraction();
  CRotFrictionInteraction(CRotParticle*,CRotParticle*);
  CRotFrictionInteraction(CRotParticle*,CRotParticle*,const CRotFrictionIGP&);
  virtual ~CRotFrictionInteraction();

  virtual void calcForces();
  virtual bool isPersistent();

  void calcNormalForce();
  double getAbsForceDeficit()const;
  double getPotentialEnergy()const;
  double getSlipping()const;
  double getTouching()const;
  virtual Vec3 getPos(){return m_cpos;};
  Vec3 getForce() const;

  friend ostream& operator<<(ostream&,const CRotFrictionInteraction&);
  friend class TML_PackedMessageInterface;
};

#endif //__ROTFRICTIONINTERACTION_H
