#ifndef __ROTPAIRINTERCTION_H
#define __ROTPAIRINTERCTION_H

// -- project includes --
#include "Model/RotParticle.h"
#include "Model/Interaction.h"

/*!
  \class ARotPairInteraction
  \brief Abstract base class interactions between 2 rotational particles

  \author Steffen Abe
  $Revision$
  $Date$
*/
class ARotPairInteraction : public AInteraction
{
 protected:
  CRotParticle *m_p1,*m_p2;

 public:
  // functions 
  ARotPairInteraction();
  ARotPairInteraction(CRotParticle*,CRotParticle*);
  virtual ~ARotPairInteraction();

  inline const CParticle* first()const {return m_p1;}; 
  inline const CParticle* second()const {return m_p2;};
  inline CRotParticle* first() {return m_p1;};
  inline CRotParticle* second() {return m_p2;};
  inline pair<int,int> getPairID() const {return make_pair(m_p1->getID(),m_p2->getID());}
  virtual Vec3 getPos()=0;
  virtual void calcForces()=0;
  void checkIDs();
  virtual bool hasTag(int,int) const;
  virtual Vec3 getPosFirst() const {return m_p1->getPos();}
  virtual Vec3 getPosSecond() const {return m_p2->getPos();}
  void setPP(CRotParticle*,CRotParticle*);
  void setPP(const vector<CRotParticle*>);
};

#endif // __ROTPAIRINTERCTION_H
