/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "Model/SimpleParticleData.h"

namespace esys
{
  namespace lsm
  {
//===============================================================================
    SimpleParticleData::SimpleParticleData()
      : m_id(-1),
        m_tag(-1),
        m_position(),
        m_radius(0.0),
        m_mass(0.0)
    {
    }

    SimpleParticleData::SimpleParticleData(Id id, Tag tag, const Vec3 &position, double radius)
      : m_id(id),
        m_tag(tag),
        m_position(position),
        m_radius(radius),
        m_mass(0.0)
    {
    }

    bool SimpleParticleData::operator==(
      const SimpleParticleData &particleData
    ) const
    {
      return
        (
          (getId()       == particleData.getId())
          &&
          (getPosition() == particleData.getPosition())
          &&
          (getRadius()   == particleData.getRadius())
          &&
          (getTag()      == particleData.getTag())
        );
    }

    const SimpleParticleData::Id &SimpleParticleData::getId() const
    {
      return m_id;
    }

    const SimpleParticleData::Id &SimpleParticleData::getID() const
    {
      return getId();
    }
    
    const Vec3 &SimpleParticleData::getPosition() const
    {
      return m_position;
    }

    const SimpleParticleData::Tag &SimpleParticleData::getTag() const
    {
      return m_tag;
    }

    const double &SimpleParticleData::getRadius() const
    {
      return m_radius;
    }

    void SimpleParticleData::setMass(double mass)
    {
      m_mass = mass;
    }
    
    const double &SimpleParticleData::getMass() const
    {
      return m_mass;
    }

    double SimpleParticleData::get2dMass() const
    {
      return m_radius*m_radius;
    }

    double SimpleParticleData::get3dMass() const
    {
      return m_radius*m_radius*m_radius;
    }

    void SimpleParticleData::read(std::istream &istream)
    {
      istream
        >> m_position
        >> m_radius
        >> m_id
        >> m_tag;
    }

    void SimpleParticleData::write(std::ostream &oStream) const
    {
      const char delim = ' ';
      oStream
        << getPosition() << delim
        << getRadius()   << delim
        << getId()       << delim
        << getTag();
    }

    std::istream &operator>>(std::istream &iStream, SimpleParticleData &particleData)
    {
      particleData.read(iStream);
      return iStream;
    }

    std::ostream &operator<<(std::ostream &oStream, const SimpleParticleData &particleData)
    {
      particleData.write(oStream);
      return oStream;
    }
  };
};
