/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef ESYS_LSMSIMPLEPARTICLEDATA_H
#define ESYS_LSMSIMPLEPARTICLEDATA_H

#include "Foundation/vec3.h"

#include <iostream>

namespace esys
{
  namespace lsm
  {
    /**
     * Container class for particle Id, Tag, position and radius.
     */
    class SimpleParticleData
    {
    public:
      typedef int Id;
      typedef int Tag;

      SimpleParticleData();

      SimpleParticleData(Id id, Tag tag, const Vec3 &position, double radius);

      bool operator==(const SimpleParticleData &particleData) const;

      const Id &getId() const;
      
      const Id &getID() const;

      const Vec3 &getPosition() const;

      const Tag &getTag() const;

      const double &getRadius() const;

      void setMass(double mass);

      const double &getMass() const;

      double get2dMass() const;

      double get3dMass() const;

      void read(std::istream &istream);

      void write(std::ostream &write) const;

    private:
      Id     m_id;
      Tag    m_tag;
      Vec3   m_position;
      double m_radius;
      double m_mass;
    };
    std::istream &operator>>(std::istream &iStream, SimpleParticleData &particleData);
    std::ostream &operator<<(std::ostream &oStream, const SimpleParticleData &particleData);

  };
};

#endif
