#ifndef __SOFTBWALLINTERACTIONGROUP_H
#define __SOFTBWALLINTERACTIONGROUP_H

//--- project includes ---
#include "SoftBWallInteraction.h"
#include "WallIG.h"
#include "BWallInteractionGroup.h"
#include "Field.h"

//--- STL includes ---
#include <map>

using std::map;

template <class T> class ParallelParticleArray;

/*!
  \class CSoftBWallIGP
  \brief Interaction group parameters for CSoftBWallInteractionGroups

  \author Steffen Abe
  $Revision$
  $Date$
*/
class CSoftBWallIGP : public CBWallIGP
{
 protected:
  double m_ky,m_kz;
 public:
  virtual void  packInto(CVarMPIBuffer*) const;
  void setAllK(double kx,double ky,double kz){m_k=kx;m_ky=ky;m_kz=kz;};
  double getKx()const{return m_k;};
  double getKy()const{return m_ky;};
  double getKz()const{return m_kz;};

  friend ostream& operator<<(ostream&,const CSoftBWallIGP&);
};

CSoftBWallIGP* extractSoftBWallIGP(AMPIBuffer*);


/*!
  \class CSoftBBWallInteractionGroup
  \brief Class for a group of bonded, elastic interactions with per-direction spring constants
	between particles and a wall

  \author Steffen Abe
  $Revision$.
  $Date$
*/
template<class T>
class CSoftBWallInteractionGroup : public AWallInteractionGroup<T>
{
 protected:
  vector<CSoftBondedWallInteraction<T> > m_interactions;
  double m_kx,m_ky,m_kz; //!< spring constants for each direction
  int m_tag;

 public:
  CSoftBWallInteractionGroup(TML_Comm*);
  CSoftBWallInteractionGroup(TML_Comm*,const CSoftBWallIGP*);
  virtual ~CSoftBWallInteractionGroup(){if (m_wall!=NULL) delete m_wall;};

  virtual void calcForces();
  virtual void Update(ParallelParticleArray<T>*);

  void addToFields(map<string,AField*>*);

  friend ostream& operator<< <>(ostream&,const CSoftBWallInteractionGroup&);
};

#include  "SoftBWallInteractionGroup.hpp"

#endif //__BSOFTWALLINTERACTIONGROUP_H
