//----------------------------------------------
//       CSoftBWallInteractionGroup functions
//----------------------------------------------

#include "WallForceField.h"
#include "console.h"

template<class T>
CSoftBWallInteractionGroup<T>::CSoftBWallInteractionGroup(TML_Comm* comm):AWallInteractionGroup<T>(comm)
{}

template<class T>
CSoftBWallInteractionGroup<T>::CSoftBWallInteractionGroup(TML_Comm* comm,const CSoftBWallIGP* I):AWallInteractionGroup<T>(comm)
{
  console.XDebug() << "making CSoftBWallInteractionGroup pos : " << I->getWallPos() << " norm : " << I->getWallNormal() << "\n";

  m_kx=I->getKx();
  m_ky=I->getKy();
  m_kz=I->getKz();
  m_wall=new CWall(I->getWallPos(),I->getWallNormal());
  m_tag=I->getTag();
}

template<class T>
void CSoftBWallInteractionGroup<T>::calcForces()
{

  console.XDebug() << "calculating " << m_interactions.size() << " soft bonded wall forces\n" ;

  for(
    typename vector<CSoftBondedWallInteraction<T> >::iterator it=m_interactions.begin();
    it != m_interactions.end();
    it++
  ){
    it->calcForces();
  }
}

template<class T>
void CSoftBWallInteractionGroup<T>::Update(ParallelParticleArray<T>* PPA)
{

  console.XDebug() << "CSoftBWallInteractionGroup::Update()\n" ;

  // empty particle list first
  m_interactions.erase(m_interactions.begin(),m_interactions.end());
  // build new particle list
  typename ParallelParticleArray<T>::ParticleListHandle plh=
    PPA->getParticlesAtPlane(m_wall->getOrigin(),m_wall->getNormal());
  for(typename ParallelParticleArray<T>::ParticleListIterator iter=plh->begin();
      iter!=plh->end();
      iter++){
    if((*iter)->getTag()==m_tag){
      m_interactions.push_back(CSoftBondedWallInteraction<T>(*iter,m_wall,m_kx,m_ky,m_kz));
    }
  }
  console.XDebug() << "end CSoftBWallInteractionGroup::Update()\n";
}

template<class T>
void CSoftBWallInteractionGroup<T>::addToFields(map<string,AField*>* fm)
{
  console.XDebug()<<"CSoftBWallInteractionGroup<T>::addToFields\n";
  map<string,AField*>::iterator iter=fm->find("WallForces");
  if(iter==fm->end()){
    console.XDebug()<<"No wall force field yet, generating it\n";
    CWallForceField *wallf=new CWallForceField;
    wallf->addWall(m_wall);
    fm->insert(pair<string,AField*>("WallForces",wallf));
  }else{
    console.XDebug()<<"Have wall force field, adding wall\n";
    //CWallForceField *wallf=dynamic_cast<CWallForceField*>(iter->second);
    CWallForceField *wallf=(CWallForceField*)(iter->second);
    if(wallf!=NULL){
      wallf->addWall(m_wall);
    } else { // can't happen
      console.Error() << "dynamic cast failed\n";
    }
  }
  console.XDebug()<<"end  CSoftBWallInteractionGroup<T>::addToFields\n";
}

template<class T>
ostream& operator<<(ostream& ost,const CSoftBWallInteractionGroup<T>& IG)
{
  ost << "CBWallInteractionGroup" << endl << flush;
  ost << *(IG.m_wall) << endl << flush;
 
  return ost;
}
