#ifndef __VWFRICTION_H
#define __VWFRICTION_H

// -- project includes --
#include "FrictionInteraction.h"

/*!
  \struct VWFrictionIGP
  \brief Interaction parameters for velocity weakening frictional interaction

  \author Steffen Abe
  $Revision$
  $Date$
*/
struct VWFrictionIGP : public CFrictionIGP
{
  double m_alpha;

  //operator CFrictionIGP();
};

/*!
  \class CVWFriction
  \brief Frictional+Elastic interaction between particles with velocity 
  weakening friction

  \author Steffen Abe
  $Revision$
  $Date$
*/
class CVWFriction : public CFrictionInteraction
{
 public: // types
  typedef VWFrictionIGP ParameterType;

  typedef double (CVWFriction::* ScalarFieldFunction)() const; 
  typedef Vec3 (CVWFriction::* VectorFieldFunction)() const; 
	typedef pair<bool,double> (CVWFriction::* CheckedScalarFieldFunction)() const;

 private:
  double m_alpha;

 public:
  CVWFriction();
  CVWFriction(CParticle*,CParticle*,const VWFrictionIGP&);
  virtual ~CVWFriction();

  virtual void calcForces();

	pair<bool,double> getCurrentMu() const;

  static ScalarFieldFunction getScalarFieldFunction(const string&);
  static VectorFieldFunction getVectorFieldFunction(const string&);
	static CheckedScalarFieldFunction getCheckedScalarFieldFunction(const string&);

  friend class TML_PackedMessageInterface;
};
#endif //__VWFRICTION_H
