#ifndef __VISCWALLIG_H
#define __VISCWALLIG_H

//--- project includes ---
#include "Model/ViscWallInteraction.h"
#include "Model/EWallInteraction.h"
#include "Model/WallIG.h"
#include "Model/EWallInteractionGroup.h"
#include "Fields/Field.h"

//--- STL includes ---
#include <map>

using std::map;

template <class T> class ParallelParticleArray;

/*!
  \class CVWallIGP
  \brief Interaction group parameters for CBWallInteractionGroups

  \author Steffen Abe
  $Revision$
  $Date$
*/
class CVWallIGP : public CEWallIGP
{
 protected:
  int m_tag;
  double m_nu;

 public:
  virtual void  packInto(CVarMPIBuffer*) const;
  void setTag(int tag){m_tag=tag;};
  int getTag()const{return m_tag;};
  void setNu(double nu){m_nu=nu;};
  double getNu()const{return m_nu;};

  friend ostream& operator<<(ostream&,const CVWallIGP&);
};

CVWallIGP* extractVWallIGP(AMPIBuffer*);

/*!
  \class CViscWallIG
  \brief Class for a group of viscous and elastic interactions between particles and a wall

  \author Steffen Abe
  $Revision$.
  $Date$
*/
template<class T>
class CViscWallIG : public AWallInteractionGroup<T>
{
 protected:
  vector<CViscWallInteraction<T> >    m_visc_interactions;    //!< visc interactions for tagged particles
  vector<CElasticWallInteraction<T> > m_elastic_interactions; //!< elastic interactions all particles
  double m_k; //!< spring constant
  double m_nu;
  int m_tag;

 public:
  CViscWallIG(TML_Comm*);
  CViscWallIG(TML_Comm*,const CVWallIGP*);
  virtual ~CViscWallIG(){if (m_wall!=NULL) delete m_wall;};

  virtual void calcForces();
  virtual void applyForce(const Vec3&);
  virtual void setVelocity(const Vec3&);
  virtual void Update(ParallelParticleArray<T>*);

  void addToFields(map<string,AField*>*);

  friend ostream& operator<< <>(ostream&,const CViscWallIG&);
};

#include  "Model/ViscWallIG.hpp"

#endif // __VISCWALLIG_H
