#ifndef __AWALLINTERACTIONGROUP_H
#define __AWALLINTERACTIONGROUP_H

//--- project includes ---
#include "Wall.h"
#include "InteractionGroup.h"

//--- IO includes ---
#include <iostream>

//--- TML includes ---
#include "tml/comm/comm.h"

using std::cerr;
using std::endl;
using std::flush;

/*!
  \class AWallInteractionGroup
  \brief Abstract Base class for a group of interactions between particles and a wall

  \author Steffen Abe
  $Revision$.
  $Date$
*/
template<class T>
class AWallInteractionGroup : public AInteractionGroup<T>
{
 protected:
  CWall* m_wall; //!< the wall
  TML_Comm* m_comm; //!< MPI communicator
  int m_inner_count;

 public:
  AWallInteractionGroup(TML_Comm* comm){m_comm=comm;};
  virtual ~AWallInteractionGroup(){if (m_wall!=NULL) delete m_wall;};

  virtual void calcForces()=0;
  virtual void applyForce(const Vec3&){ cerr << "calling unimplemented function AWallInteractionGroup::applyForce" << endl << flush;}; //!< apply force to wall   
  virtual void setVelocity(const Vec3&){ cerr << "calling unimplemented function AWallInteractionGroup::setVelocity" << endl << flush;}; //!< apply force to wall 
  //virtual void Update(CNTable*)=0;
  inline double getDisplacement(){return m_wall->getDisplacement();};
  inline void resetDisplacement(){m_wall->resetDisplacement();};
  inline void moveWallBy(const Vec3& mv){m_wall->moveBy(mv);};
  inline void zeroForce(){m_wall->zeroForce();};
  virtual void addToFields(map<string,AField*>*){};
};


#endif // __AWALLINTERACTIONGROUP_H
