#include "BondedTMITest.h"

// -- test includes --
#include "Model/Particle.h"
#include "Geometry/Triangle.h"
#include "Model/BTriangleInteraction.h"
#include "Model/BTriMeshIP.h"

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( BondedTMITest );

  /*! 
    constructor - do nothing
  */
  BondedTMITest::BondedTMITest() : CppUnit::TestCase()
  {}

  /*!
    destructor - do nothing
  */
  BondedTMITest::~BondedTMITest()
  {
  }
    
  /*!
   */
  void BondedTMITest::setUp()
  {
    CppUnit::TestCase::setUp();
  }
  
  /*!
   */ 
  void BondedTMITest::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }

  /*!
    test ETriangleInteraction::calcForce
  */
  void BondedTMITest::testTriangleForce()
  {
    // setup triangle
    Vec3 p1=Vec3(0.0,1.0,0.0);
    Vec3 p2=Vec3(1.0,1.0,0.0);
    Vec3 p3=Vec3(0.0,1.0,1.0);
    Triangle Tr1=Triangle(1,2,3,p1,p2,p3,0,0);
    // some test particles & interactions
    Vec3 vpos=Vec3(0.4,1.0,0.6);
    CParticle P1=CParticle(1.0,1.0,vpos,Vec3(0.0,0.0,0.0),Vec3(0.0,0.0,0.0),1);
    BTriMeshIP param;
    param.k=1.0;
    param.brk=1.02;
    BTriangleInteraction I1=BTriangleInteraction(&P1,&Tr1,param);
    // calc & check forces -> initial force should be 0
    I1.calcForces();
    Vec3 force1=P1.getForce();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,force1.X(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,force1.Y(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,force1.Z(),1e-10);
    // move particle
    P1.zeroForce();
    P1.moveToRel(Vec3(0.2,0.0,0.0));
    I1.calcForces();
    force1=P1.getForce();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.2,force1.X(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,force1.Y(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,force1.Z(),1e-10);
    // move node of triangle
    P1.zeroForce();
    Tr1.moveNode(3,Vec3(0.2,0.0,0.0));
    I1.calcForces();
    force1=P1.getForce();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.08,force1.X(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,force1.Y(),1e-10);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,force1.Z(),1e-10);
  }
};
