/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef LSMTESTINTEGRATOR_H
#define LSMTESTINTEGRATOR_H

namespace LsmTest
{
  template <typename TmplInteraction>
  class Integrator
  {
  public:
    Integrator(TmplInteraction &interaction)
      : m_pInteraction(&interaction)
    {
    }

    void integrate(int numTimeSteps, double dt)
    {
      for (int i = 0; i < numTimeSteps; i++) {
        m_pInteraction->calcForces();
        m_pInteraction->first()->integrate(dt);
        m_pInteraction->second()->integrate(dt);
        m_pInteraction->first()->resetDisplacement();
        m_pInteraction->second()->resetDisplacement();
        m_pInteraction->first()->zeroForce();
        m_pInteraction->second()->zeroForce();
        m_pInteraction->first()->rescale();
        m_pInteraction->second()->rescale();
      }
    }

    void initialiseInteractionForces()
    {
      m_pInteraction->calcForces();
    }

    void resetParticleForces()
    {
      m_pInteraction->first()->zeroForce();
      m_pInteraction->second()->zeroForce();
    }

    double getTotalEnergy()
    {
      initialiseInteractionForces();
      const double energy =
        m_pInteraction->getPotentialEnergy()
        +
        m_pInteraction->first()->getKineticEnergy()
        +
        m_pInteraction->second()->getKineticEnergy();
      resetParticleForces();
      
      return energy;
    }

    const TmplInteraction &getInteraction() const
    {
      return *m_pInteraction;
    }

    TmplInteraction &getInteraction()
    {
      return *m_pInteraction;
    }

  private:
    TmplInteraction *m_pInteraction;
  };
};

#endif // LSMTESTINTEGRATOR_H
