/*=============================================================================
??
??******************************************************************************
??*                                                                            *
??*       COPYRIGHT  ???????  -  All Rights Reserved                           *
??*                                                                            *
??* This software is the property of ??????????????.  No part of this code     *
??* may be copied in any form or by any means without the expressed written    *
??* consent of ???????????.  Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that                          *
??* person has a software license agreement with ?????????????.                *
??*                                                                            *
??******************************************************************************
??
*********************************************************************************/

#include "ModelTest/TriMeshTest.h"

// -- IO includes --
#include <iostream>
using std::cout;
using std::endl;

//using namespace esys::lsm;

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( TriMeshTest );

  /*!
    constructor - setup mesh data
  */
  TriMeshTest::TriMeshTest()
  {
    MeshNodeData nd;
    MeshTriData td;

    // node 1
    nd.x=0;
    nd.y=0;
    nd.z=0;
    nd.id=0;
    nd.tag=0;
    m_mnd.push_back(nd);
    // node 2
    nd.x=1;
    nd.y=0;
    nd.z=0;
    nd.id=1;
    nd.tag=0;
    m_mnd.push_back(nd);
    // node 3
    nd.x=1;
    nd.y=1;
    nd.z=0;
    nd.id=2;
    nd.tag=1;
    m_mnd.push_back(nd);
    // node 4
    nd.x=0;
    nd.y=1;
    nd.z=0;
    nd.id=3;
    nd.tag=1;
    m_mnd.push_back(nd);
    // triangle 1
    td.p1=0;
    td.p2=1;
    td.p3=2;
    td.id=0;
    td.tag=0;
    m_mtd.push_back(td);
    // triangle 2
    td.p1=0;
    td.p2=2;
    td.p3=3;
    td.id=1;
    td.tag=1;
    m_mtd.push_back(td);
    m_tm.LoadMesh(m_mnd,m_mtd);
   }
  
  /*!
    destructor
  */
  TriMeshTest::~TriMeshTest()
  {}
  
  /*!
   */
  void TriMeshTest::setUp()
  {
    CppUnit::TestCase::setUp();
  }
  
  /*!
   */
  void TriMeshTest::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }
  
  /*!
    test TriMesh::LoadMesh
  */
  void TriMeshTest::testLoadMesh()
  {
    int tricount=0;
    for(TriMesh::triangle_iterator iter=m_tm.triangles_begin();
	iter!=m_tm.triangles_end();
	iter++){
      cout << *iter << endl;
      tricount++;
    }
    CPPUNIT_ASSERT_EQUAL(2,tricount);
    int edgecount=0;
    for(TriMesh::edge_iterator iter=m_tm.edges_begin();
	iter!=m_tm.edges_end();
	iter++){
      cout << *iter << endl;
      edgecount++;
    }
    CPPUNIT_ASSERT_EQUAL(5,edgecount);
    int nodecount=0;
    for(TriMesh::corner_iterator iter=m_tm.corners_begin();
	iter!=m_tm.corners_end();
	iter++){
       nodecount++;
    }
    CPPUNIT_ASSERT_EQUAL(4,nodecount);
  }

  /*!
    test TriMesh::getTriangleById
  */
  void TriMeshTest::testGetTriangleById()
  {
    Triangle* tp=m_tm.getTriangleById(1);
    CPPUNIT_ASSERT(tp!=NULL);
    cout << *tp << endl;
    CPPUNIT_ASSERT_EQUAL(1,tp->getID());
    tp=m_tm.getTriangleById(5);
    CPPUNIT_ASSERT_EQUAL((Triangle*)NULL,tp);
  }
};
