#ifndef __ASUBLATTICE_H
#define __ASUBLATTICE_H


// -- system includes --
#include <string>

using std::string;

// -- project includes --
#include "shake_math.h"
#include "EWallInteractionGroup.h"
#include "BWallInteractionGroup.h"
#include "SoftBWallInteractionGroup.h"
#include "Model/ViscWallIG.h"
#include "Parallel/CheckPointable.h"

class MpiWTimers;
class TML_Comm;

/*!
  \class ASubLattice
  \brief Abstract base class for sublattices

  \author Steffen Abe
  $Revision$
  $Date$
*/
class ASubLattice : public esys::lsm::CheckPointable
{
protected:

  // -- neighbortable --

public:
  virtual ~ASubLattice();
  void setNTSize(int);
  virtual vector<int> getCommCoords() const=0;
  virtual vector<int> getCommDims() const=0;
  virtual void receiveParticles(TML_Comm*)=0;
  virtual void addOwnedParticleByParam(double,double,const Vec3&,int)=0;
  virtual void addWall(const CEWallIGP*)=0;
  virtual void initNeighborTable(const Vec3&,const Vec3&)=0;
  virtual void initNeighborTable(const Vec3&,const Vec3&,const vector<bool>&)=0;
  virtual void addBondedWall(const CBWallIGP*)=0;
  virtual void addViscWall(const CVWallIGP*)=0;
  virtual void addSoftBondedWall(const CSoftBWallIGP*)=0;
  virtual void addPairIG()=0;
  virtual void addRotPairIG()=0;
  virtual void addTriMesh()=0;
  virtual void addTriMeshIG()=0;
  virtual void addBondedTriMeshIG()=0;
  virtual void addSingleIG()=0;
  virtual void addBondedIG()=0;
  virtual void addRotBondedIG()=0;
  virtual void addDamping()=0;
  //virtual void addRotDamping()=0;
  virtual void setExIG()=0;
  virtual void initComplex();

  virtual const MPI_Comm &getWorkerComm() const = 0;
  
  virtual void rebuildParticleArray()=0;
  virtual void rebuildInteractions()=0;
  virtual void searchNeighbors()=0;
  virtual void checkNeighbors()=0;
  
  virtual void updateInteractions()=0;
  

  virtual void oneStep()=0;
  virtual void exchangePos()=0;

  // moving stuff around
  virtual void moveParticleTo()=0;
  virtual void moveWallBy()=0;
  virtual void applyForceToWall()=0;
  virtual void setVelocityOfWall()=0;
  virtual void setParticleVelocity()=0;
  virtual void setParticleNonDynamic()=0;
  virtual void tagParticleNearestTo()=0;
  virtual void moveSingleNode()=0;
  virtual void moveTaggedNodes()=0;

  virtual void setTimer(MpiWTimers &timers) = 0;
  
  virtual void do2dCalculations() = 0;

  // field functions
  virtual void makeFields()=0;
  virtual void countFieldData()=0;
  virtual void collectFieldData()=0;
  virtual void sendDataToMaster()=0;
  virtual void countParticles()=0;

  // "new" field functions
  virtual void addScalarParticleField()=0;
  virtual void addVectorParticleField()=0;
  virtual void addScalarInteractionField()=0;
  virtual void addVectorInteractionField()=0;
  virtual void addVectorTriangleField()=0;
  virtual void sendFieldData()=0;

  // output
  virtual void printStruct()=0;
  virtual void printData()=0;
  virtual void printTimes()=0;
  
  virtual void printField(const string&)=0; //!< field debugging
  virtual void printAllFields()=0; 
};

#endif //__ASUBLATTICE_H
