/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef ESYS_LSMCHECKPOINTLOADER_H
#define ESYS_LSMCHECKPOINTLOADER_H

#include "Parallel/IterativeReader.h"
#include "Model/Particle.h"
#include "Model/BondedInteraction.h"

#include <vector>
#include <string>

namespace esys
{
  namespace lsm
  {
    typedef std::vector<std::string> StringVector;
    
    /**
     * Objects of this class load particle and interaction check-point
     * data from file and initialise a specified object via the
     * CheckPointLoader::loadInto method.
     */
    class CheckPointLoader
    {
    public:
      class ParticleData : public CParticle
      {
      public:
        ParticleData() : CParticle()
        {
        }

        void read(std::istream &iStream)
        {
          loadCheckPointData(iStream);
        }
      };
      
      class ConnectionData : public BondedInteractionCpData
      {
      public:
        ConnectionData() : BondedInteractionCpData()
        {
        }
        
        void read(std::istream &iStream)
        {
          loadCheckPointData(iStream);
        }
      };
      
      class ParticleReader : public IterativeReader<IStreamIterator<ParticleData> >
      {
      public:
        typedef IterativeReader<IStreamIterator<ParticleData> >::Iterator Iterator;
        
        ParticleReader(std::istream &iStream) : IterativeReader<IStreamIterator<ParticleData> >(iStream)
        {
        }
        
        virtual void initialise()
        {
          int numParticles = 0;
          getIStream() >> numParticles;
          setNumElements(numParticles);
          IterativeReader<IStreamIterator<ParticleData> >::initialise();
        }
      };

      class ConnectionReader : public IterativeReader<IStreamIterator<ConnectionData> >
      {
      public:
        ConnectionReader(std::istream &iStream) : IterativeReader<IStreamIterator<ConnectionData> >(iStream)
        {
        }
        
        virtual void initialise()
        {
          int numConnections = 0;
          getIStream() >> numConnections;
          setNumElements(numConnections);
          IterativeReader<IStreamIterator<ConnectionData> >::initialise();
        }
      };

      CheckPointLoader(const StringVector &fileNames) : m_fileNames(fileNames)
      {
      }

      template<class TmplLsmData>
      void loadInto(TmplLsmData &lsmData)
      {
        for (
          StringVector::const_iterator it = m_fileNames.begin();
          it != m_fileNames.end();
          it++
        )
        {
          std::ifstream iStream(it->c_str());
          ParticleReader pReader(iStream);
          lsmData.template addParticles<ParticleReader::Iterator,CParticle>(pReader.getIterator());

          ConnectionReader cReader(iStream);
          lsmData.addConnections(cReader.getIterator());
        }
      }

    private:
      StringVector m_fileNames;
    };
  };
};

#endif
