/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "Parallel/CheckPointParams.h"
#include "Parallel/MpiInfo.h"
#include "Parallel/mpibuf.h"

#include <sstream>
#include <memory>

CheckPointParams::CheckPointParams() : m_fileNamePrefix(), m_time(-1), m_rank(-1)
{
}

CheckPointParams::CheckPointParams
(
  const std::string &fileNamePrefix,
  int time,
  int rank
) : m_fileNamePrefix(fileNamePrefix),
    m_time(time),
    m_rank(rank)
{
}

CheckPointParams::~CheckPointParams()
{
}

std::string CheckPointParams::getFileName() const
{
  std::stringstream name;
  name << m_fileNamePrefix << "_t=" << m_time << "_" << m_rank << ".txt";

  return name.str();
}

void CheckPointParams::packInto(AMPIBuffer *pMpiBuff) const
{
  pMpiBuff->append(m_fileNamePrefix.c_str());
  pMpiBuff->append(m_time);
}

CheckPointParams CheckPointParams::unpackFrom(AMPIBuffer *pMpiBuff)
{
  CheckPointParams prms;

  prms.m_fileNamePrefix = pMpiBuff->pop_string();
  prms.m_time = pMpiBuff->pop_int();
  prms.m_rank = MpiInfo().rank();

  return prms;
}
