/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef CHECKPOINTPARAMS_H
#define CHECKPOINTPARAMS_H

#include <string>
#include <iostream>

class AMPIBuffer;

/**
 * Parameters associated with check-point file creation.
 */
class CheckPointParams
{
public:
  CheckPointParams(const std::string &fileNamePrefix, int time, int rank);

  ~CheckPointParams();

  /**
   * Returns full check-point file (prefix plus time and rank identifier).
   */
  std::string getFileName() const;

  void packInto(AMPIBuffer *mpiBuff) const;
  static CheckPointParams unpackFrom(AMPIBuffer *mpiBuff);

  //friend std::ostream& operator<<(std::ostream &oStream, const CheckPointParams &checkPointParams);

protected:
  CheckPointParams();

private:
  /**
   * Prefix of check-point file names.
   */
  std::string m_fileNamePrefix;
  
  /**
   * The simulation time at which checkpoint is created.
   */
  int m_time;
  
  /**
   * Rank of mpi process.
   */
  int m_rank;
};

#endif
