/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "Parallel/LatticeParam.h"
#include "Parallel/mpibuf.h"

namespace esys
{
  namespace lsm
  {
    //----------------------------------------------
    //  CLatticeParam functions
    //----------------------------------------------
    CLatticeParam::CLatticeParam(const std::string& s,double r,double a,double t, const ProcessDims &dims)
    {
      m_particle_type=s;
      m_nrange=r;
      m_alpha=a;
      m_dt=t;
      m_dims = dims;
    }

    void CLatticeParam::packInto(AMPIBuffer *B) const
    {
      B->append(m_nrange);
      B->append(m_alpha);
      B->append(m_dt);
      B->append(m_particle_type.c_str());
      B->append(static_cast<int>(m_dims[0]));
      B->append(static_cast<int>(m_dims[1]));
      B->append(static_cast<int>(m_dims[2]));
    }

    CLatticeParam CLatticeParam::extractLatticeParam(AMPIBuffer *B)
    {
      double nrange       = B->pop_double();
      double alpha        = B->pop_double();
      double dt           = B->pop_double();
      std::string particleType = B->pop_string();
      CLatticeParam::ProcessDims dims = CLatticeParam::ProcessDims(3, 0);
      dims[0]             = B->pop_int();
      dims[1]             = B->pop_int();
      dims[2]             = B->pop_int();
    
      return CLatticeParam(particleType, nrange, alpha, dt, dims);
    }

    ostream& operator<<(ostream& ost,const CLatticeParam& CP)
    {
      ost << "CLatticeParam\n";
      ost << "ptype  : " << CP.m_particle_type << endl;
      ost << "nrange : " << CP.m_nrange << endl;
      ost << "alpha  : " << CP.m_alpha << endl;
      ost << "dt     : " << CP.m_dt << endl;

      return ost;
    }
  };
};
