/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef ESYS_LSMLATTICEPARAM_H
#define ESYS_LSMLATTICEPARAM_H

class AMPIBuffer;

#include <vector>
#include <string>
#include <iostream>

namespace esys
{
  namespace lsm
  {
    /**
     * Parameters for the sublattice.
     */
    class CLatticeParam
    {
     public:
      typedef std::vector<unsigned int> ProcessDims;
      CLatticeParam(
        const std::string &particleType,
        double nrange,
        double alpha,
        double dt,
        const ProcessDims &dims = ProcessDims(3, 0)
      );
      void setSearchRange(double nr,double a){m_nrange=nr;m_alpha=a;};
      void setTimeStep(double t){m_dt=t;};
      void setParticleType(const std::string& pt){m_particle_type=pt;}
      void packInto(AMPIBuffer*) const;
      double nrange()const {return m_nrange;};
      double alpha()const {return m_alpha;};
      double dt()const {return m_dt;};
      const ProcessDims &processDims() const {return m_dims;}
      const std::string &particle_type() const {return m_particle_type;};
      friend std::ostream& operator<<(std::ostream &oStream,const CLatticeParam &prms);

      static CLatticeParam extractLatticeParam(AMPIBuffer *pBuff);

     protected:
      std::string m_particle_type;
      double      m_alpha;  //!< pair creation cutoff
      double      m_nrange; //!< searchrange
      double      m_dt;     //!< time step  
      ProcessDims m_dims;
    };
  };
};

#endif
