/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "Parallel/ParserLmInitialiser.h"
#include "Parallel/LatticeMaster.h"

// -- Parser Functions ----

class LatticeMastererPredefFn : public PredefinedFunction
{
public:
  LatticeMastererPredefFn(
    int argc,
    OpValue::OType *itype,
    CLatticeMaster &latticeMaster
  ) : PredefinedFunction(argc, itype),
      m_pLatticeMaster(&latticeMaster)
  {
  }
  
  LatticeMastererPredefFn(
    int argc,
    OpValue::OType itype,
    CLatticeMaster &latticeMaster
  ) : PredefinedFunction(argc, itype),
      m_pLatticeMaster(&latticeMaster)
  {
  }

protected:
  CLatticeMaster &getLatticeMaster()
  {
    return *m_pLatticeMaster;
  }

  const CLatticeMaster &getLatticeMaster() const
  {
    return *m_pLatticeMaster;
  }

private:
  CLatticeMaster *m_pLatticeMaster;
};

class PF_MakeLattice: public LatticeMastererPredefFn 
{
  static OpValue::OType T[4];
public:
  PF_MakeLattice(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4,T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; char* type = args->Get()->Content.str ;
    args->Next() ; double dt = args->Get()->Content.d;
    args->Next() ; double rmax = args->Get()->Content.d;
    args->Next() ; double alpha = args->Get()->Content.d;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().makeLattice(type,rmax,alpha,dt);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  };
};

OpValue::OType PF_MakeLattice::T[4] = {OpValue::STR,OpValue::REAL,OpValue::REAL,OpValue::REAL};

//---------

class PF_AddBondedIG : public LatticeMastererPredefFn {
  static OpValue::OType T[4] ;
public:
  PF_AddBondedIG(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; int tag = args->Get()->Content.l;
    args->Next() ; char* name = args->Get()->Content.str ;
    args->Next() ; double k = args->Get()->Content.d ;
    args->Next() ; double break_dist = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addBondedIG(tag,string(name),k,break_dist);	
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddBondedIG::T[4] = { OpValue::INT,OpValue::STR,OpValue::REAL,OpValue::REAL} ;

//---------

class PF_AddBondedTriMeshIG : public LatticeMastererPredefFn {
  static OpValue::OType T[4] ;
public:
  PF_AddBondedTriMeshIG(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(UNLIMITED_ARGS,OpValue::NONE, latticeMaster) {};
  OpValue fn(ExpressionNode *args) {
    list<OpValue> buildparams;
   
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string name=string(args->Get()->Content.str);
    args->Next() ; string meshname=string(args->Get()->Content.str);
    args->Next() ; double k = args->Get()->Content.d ;
    args->Next() ; double break_dist = args->Get()->Content.d ;
    args->Next() ; string build=string(args->Get()->Content.str);
    args->Next() ;
    while (!args->IsEnd()) {
      OpValue OV;
      ExpressionNode *Ptr = args->Get();
      switch (Ptr->ValueType) {
        case OpValue::INT:
          OV=Ptr->Content.l;
          break;
        case OpValue::REAL:
          OV=Ptr->Content.d;
          break; 
        default:
          console.Error()<<"Type error in evaluation of parameter list for AddTMIG:\n only int and double allowed\n";
          break;
      }
      buildparams.push_back(OV);
      args->Next() ;
    }
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addBondedTriMeshIG(name,meshname,k,break_dist,build,buildparams);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

//---------

class PF_SetProcessDims : public LatticeMastererPredefFn {
  static OpValue::OType T[3] ;
public:
  PF_SetProcessDims(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(3,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues();
    esys::lsm::CLatticeParam::ProcessDims processDims(3, 0);
    args->First(); 
    args->Next();
    processDims[0] = args->Get()->Content.l;
    args->Next();
    processDims[1] = args->Get()->Content.l;
    args->Next();
    processDims[2] = args->Get()->Content.l;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().setProcessDims(processDims);
    SParserAccess::Interface()->SetAllValues();
    return OpValue(1L);
  }
};

OpValue::OType PF_SetProcessDims::T[3] =
  {
    OpValue::INT,
    OpValue::INT,
    OpValue::INT
  };

//---------

class PF_AddPIG :  public LatticeMastererPredefFn
{
public:
  PF_AddPIG(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(UNLIMITED_ARGS,OpValue::NONE, latticeMaster) {};  
  OpValue fn(ExpressionNode *args) {
    string type;
    string name;
    list<OpValue> params;
   
    args->First() ;
    args->Next() ; type=string(args->Get()->Content.str);
    args->Next() ; name=string(args->Get()->Content.str);
    args->Next() ; 
    while (!args->IsEnd()) {
      OpValue OV;
      ExpressionNode *Ptr = args->Get();
      switch (Ptr->ValueType) {
        case OpValue::INT:
          OV=Ptr->Content.l;
          break;
        case OpValue::REAL:
          OV=Ptr->Content.d;
          break; 
        case OpValue::STR:
          OV=Ptr->Content.str;
          break;
        default:
          console.Error()<<"Type error in evaluation of parameter list for AddPIG:\n only int, double and string allowed\n";
          break;
      }
      params.push_back(OV);
      args->Next() ;
    }
    getLatticeMaster().addPairIG(type,name,params);	
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

//---------

class PF_AddTMIG  :  public LatticeMastererPredefFn
{
public:
  PF_AddTMIG(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(UNLIMITED_ARGS,OpValue::NONE, latticeMaster) {};  
  OpValue fn(ExpressionNode *args) {
    list<OpValue> params;
   
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; string type=string(args->Get()->Content.str);
    args->Next() ; string name=string(args->Get()->Content.str);
    args->Next() ; string meshname=string(args->Get()->Content.str);
    args->Next() ; 
    while (!args->IsEnd()) {
      OpValue OV;
      ExpressionNode *Ptr = args->Get();
      switch (Ptr->ValueType) {
        case OpValue::INT:
          OV=Ptr->Content.l;
          break;
        case OpValue::REAL:
          OV=Ptr->Content.d;
          break; 
        default:
          console.Error()<<"Type error in evaluation of parameter list for AddTMIG:\n only int and double allowed\n";
          break;
      }
      params.push_back(OV);
      args->Next() ;
    }
    getLatticeMaster().addTriMeshIG(type,name,meshname,params);	
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

//---------

class PF_AddTriMesh : public LatticeMastererPredefFn {
  static OpValue::OType T[2] ;
public:
  PF_AddTriMesh(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(2,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; string name = string(args->Get()->Content.str) ;
    args->Next() ; string file = string(args->Get()->Content.str) ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addTriMesh(name,file);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddTriMesh::T[2] = { OpValue::STR, OpValue::STR} ;

//---------

class PF_AddSIG :  public LatticeMastererPredefFn
{
public:
  PF_AddSIG(CLatticeMaster &latticeMaster): LatticeMastererPredefFn(UNLIMITED_ARGS,OpValue::NONE, latticeMaster) {};  
  OpValue fn(ExpressionNode *args) {
    string type;
    string name;
    list<OpValue> params;
   
    args->First() ;
    args->Next() ; type=string(args->Get()->Content.str);
    args->Next() ; name=string(args->Get()->Content.str);
    args->Next() ; 
    while (!args->IsEnd()) {
      OpValue OV;
      ExpressionNode *Ptr = args->Get();
      switch (Ptr->ValueType) {
        case OpValue::INT:
          OV=Ptr->Content.l;
          break;
        case OpValue::REAL:
          OV=Ptr->Content.d;
          break; 
        case OpValue::STR:
          OV=Ptr->Content.str;
          break;
        default:
          console.Error()<<"Type error in evaluation of parameter list for AddSIG:\n only int, double and string allowed\n";
          break;
      }
      params.push_back(OV);
      args->Next() ;
    }
    getLatticeMaster().addSingleIG(type,name,params);	
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

//---------

class PF_setNumSteps : public LatticeMastererPredefFn {
public:
  PF_setNumSteps(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(1,OpValue::INT, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; int nsteps = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().setNumSteps(nsteps); 
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

//---------

class PF_runInit : public LatticeMastererPredefFn {
public:
  PF_runInit(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(0,OpValue::NONE, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().runInit() ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  };
};

//---------

class PF_getTimeStep : public LatticeMastererPredefFn {
public:
  PF_getTimeStep(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(0,OpValue::NONE, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues();
    ISVALID_PTR(m_pLatticeMaster);
    const long int timeStep = getLatticeMaster().getTimeStep();
    SParserAccess::Interface()->SetAllValues();
    return OpValue(timeStep);
  };
};

//---------

class PF_getTimeStepSize : public LatticeMastererPredefFn {
public:
  PF_getTimeStepSize(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(0,OpValue::NONE, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    ISVALID_PTR(m_pLatticeMaster) ;
    const double timeStepSize = getLatticeMaster().getTimeStepSize();
    SParserAccess::Interface()->SetAllValues();
    return OpValue(timeStepSize) ;
  };
};

//---------

class PF_initWall : public LatticeMastererPredefFn {
public:
  PF_initWall(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(7, OpValue::REAL, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; real arg1 = args->Get()->Content.d ;
    args->Next() ; real arg2 = args->Get()->Content.d ;
    args->Next() ; real arg3 = args->Get()->Content.d ;
    args->Next() ; real arg4 = args->Get()->Content.d ;
    args->Next() ; real arg5 = args->Get()->Content.d ;
    args->Next() ; real arg6 = args->Get()->Content.d ;
    args->Next() ; real arg7 = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().initWall(Vec3(arg1,arg2,arg3),Vec3(arg4,arg5,arg6),arg7);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

//---------

class PF_initBondedWall : public LatticeMastererPredefFn {
  static OpValue::OType T[8] ;
public:
  PF_initBondedWall(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(8,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; real arg1 = args->Get()->Content.d ;
    args->Next() ; real arg2 = args->Get()->Content.d ;
    args->Next() ; real arg3 = args->Get()->Content.d ;
    args->Next() ; real arg4 = args->Get()->Content.d ;
    args->Next() ; real arg5 = args->Get()->Content.d ;
    args->Next() ; real arg6 = args->Get()->Content.d ;
    args->Next() ; real arg7 = args->Get()->Content.d ;
    args->Next() ; int arg8 = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().initBondedWall(Vec3(arg1,arg2,arg3),Vec3(arg4,arg5,arg6),arg7,arg8);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_initBondedWall::T[8] = { OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::INT} ;

//---------

class PF_initViscWall : public LatticeMastererPredefFn {
  static OpValue::OType T[9] ;
public:
  PF_initViscWall(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(9,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; real ox = args->Get()->Content.d ;
    args->Next() ; real oy = args->Get()->Content.d ;
    args->Next() ; real oz = args->Get()->Content.d ;
    args->Next() ; real nx = args->Get()->Content.d ;
    args->Next() ; real ny = args->Get()->Content.d ;
    args->Next() ; real nz = args->Get()->Content.d ;
    args->Next() ; real k = args->Get()->Content.d ;
    args->Next() ; real nu = args->Get()->Content.d ;
    args->Next() ; int tag = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().initViscWall(Vec3(ox,oy,oz),Vec3(nx,ny,nz),k,nu,tag);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_initViscWall::T[9] = { OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::INT} ;
//---------
//---------

class PF_initSoftBondedWall : public LatticeMastererPredefFn {
  static OpValue::OType T[10] ;
public:
  PF_initSoftBondedWall(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(10,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; real px = args->Get()->Content.d ;
    args->Next() ; real py = args->Get()->Content.d ;
    args->Next() ; real pz = args->Get()->Content.d ;
    args->Next() ; real nx = args->Get()->Content.d ;
    args->Next() ; real ny = args->Get()->Content.d ;
    args->Next() ; real nz = args->Get()->Content.d ;
    args->Next() ; real kx = args->Get()->Content.d ;
    args->Next() ; real ky = args->Get()->Content.d ;
    args->Next() ; real kz = args->Get()->Content.d ;
    args->Next() ; int tag = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().initSoftBondedWall(Vec3(px,py,pz),Vec3(nx,ny,nz),kx,ky,kz,tag);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_initSoftBondedWall::T[10] = { OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::REAL, OpValue::INT} ;

//---------

class PF_moveWall : public LatticeMastererPredefFn {
    static OpValue::OType T[4] ;
public:
  PF_moveWall(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; integer arg1 = args->Get()->Content.l ;
    args->Next() ; real arg2 = args->Get()->Content.d ;
    args->Next() ; real arg3 = args->Get()->Content.d ;
    args->Next() ; real arg4 = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().moveWallBy(arg1,Vec3(arg2,arg3,arg4)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_moveWall::T[4] = { OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------

class PF_applyForceToWall : public LatticeMastererPredefFn {
    static OpValue::OType T[4] ;
public:
  PF_applyForceToWall(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; integer arg1 = args->Get()->Content.l ;
    args->Next() ; real arg2 = args->Get()->Content.d ;
    args->Next() ; real arg3 = args->Get()->Content.d ;
    args->Next() ; real arg4 = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().applyForceToWall(arg1,Vec3(arg2,arg3,arg4)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_applyForceToWall::T[4] = { OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------

class PF_setVelocityOfWall : public LatticeMastererPredefFn {
    static OpValue::OType T[4] ;
public:
  PF_setVelocityOfWall(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; integer tag = args->Get()->Content.l ;
    args->Next() ; real vx = args->Get()->Content.d ;
    args->Next() ; real vy = args->Get()->Content.d ;
    args->Next() ; real vz = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().setVelocityOfWall(tag,Vec3(vx,vy,vz)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_setVelocityOfWall::T[4] = { OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------

class PF_setPVel : public LatticeMastererPredefFn {
    static OpValue::OType T[4] ;
public:
  PF_setPVel(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4,T,latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; integer arg1 = args->Get()->Content.l ;
    args->Next() ; real arg2 = args->Get()->Content.d ;
    args->Next() ; real arg3 = args->Get()->Content.d ;
    args->Next() ; real arg4 = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().setParticleVel(arg1,Vec3(arg2,arg3,arg4)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_setPVel::T[4] = { OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------

class PF_setPPos : public LatticeMastererPredefFn {
    static OpValue::OType T[4] ;
public:
  PF_setPPos(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4,T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; integer arg1 = args->Get()->Content.l ;
    args->Next() ; real arg2 = args->Get()->Content.d ;
    args->Next() ; real arg3 = args->Get()->Content.d ;
    args->Next() ; real arg4 = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().moveParticleTo(arg1,Vec3(arg2,arg3,arg4)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_setPPos::T[4] = { OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------
// move trimesh node
//---------
class PF_moveNode : public LatticeMastererPredefFn {
    static OpValue::OType T[5] ;
public:
  PF_moveNode(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(5,T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string name = string(args->Get()->Content.str) ;
    args->Next() ; int id = args->Get()->Content.l ;
    args->Next() ; real x = args->Get()->Content.d ;
    args->Next() ; real y = args->Get()->Content.d ;
    args->Next() ; real z = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().moveSingleNodeBy(name,id,Vec3(x,y,z)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_moveNode::T[5] = { OpValue::STR, OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------
// move tagged trimesh nodes
//---------
class PF_moveTaggedNodes : public LatticeMastererPredefFn {
    static OpValue::OType T[5] ;
public:
  PF_moveTaggedNodes(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(5,T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string name = string(args->Get()->Content.str) ;
    args->Next() ; int tag = args->Get()->Content.l ;
    args->Next() ; real x = args->Get()->Content.d ;
    args->Next() ; real y = args->Get()->Content.d ;
    args->Next() ; real z = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().moveSingleNodeBy(name,tag,Vec3(x,y,z)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_moveTaggedNodes::T[5] = { OpValue::STR, OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------

class PF_tagPPos : public LatticeMastererPredefFn {
    static OpValue::OType T[5] ;
public:
  PF_tagPPos(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(5,T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; integer tag = args->Get()->Content.l ;
    args->Next() ; integer mask = args->Get()->Content.l ;
    args->Next() ; real x = args->Get()->Content.d ;
    args->Next() ; real y = args->Get()->Content.d ;
    args->Next() ; real z = args->Get()->Content.d ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().tagParticleNearestTo(tag,mask,Vec3(x,y,z)) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_tagPPos::T[5] = { OpValue::INT,OpValue::INT, OpValue::REAL, OpValue::REAL, OpValue::REAL} ;

//---------

class PF_setPNonDynamic : public LatticeMastererPredefFn {
    static OpValue::OType T[1] ;
public:
  PF_setPNonDynamic(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(1, T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; integer arg1 = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().setParticleNonDynamic(arg1) ;
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_setPNonDynamic::T[1] = { OpValue::INT} ;


//---------

class PF_AddExIG : public LatticeMastererPredefFn {
public:
  PF_AddExIG(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(2, OpValue::STR, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string arg1 = string(args->Get()->Content.str) ;
    args->Next() ; string arg2 = string(args->Get()->Content.str) ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addExIG(arg1,arg2);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

//---------

class PF_AddDamping :  public LatticeMastererPredefFn
{
public:
  PF_AddDamping(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(UNLIMITED_ARGS, OpValue::NONE, latticeMaster) {};  
  OpValue fn(ExpressionNode *args) {
    string type;
    string name;
    list<OpValue> params;
  
    args->First() ;
    args->Next() ; type=string(args->Get()->Content.str);
    args->Next() ; 
    while (!args->IsEnd()) {
      OpValue OV;
      ExpressionNode *Ptr = args->Get();
      switch (Ptr->ValueType) {
        case OpValue::INT:
          OV=Ptr->Content.l;
          break;
        case OpValue::REAL:
          OV=Ptr->Content.d;
          break; 
        case OpValue::STR:
          OV=Ptr->Content.str;
          break;
        default:
          console.Error()<<"Type error in evaluation of parameter list for AddDamping:\n only int, double and string allowed\n";
          break;
      }
      params.push_back(OV);
      args->Next() ;
    }
    getLatticeMaster().addDamping(type,params);	
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

//---------

class PF_InitComplexFF : public LatticeMastererPredefFn {
  static OpValue::OType T[4] ;
public:
  PF_InitComplexFF(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4, T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string arg1 = string(args->Get()->Content.str) ;
    args->Next() ; int arg2 = args->Get()->Content.l ;
    args->Next() ; int arg3 = args->Get()->Content.l ;
    args->Next() ; int arg4 = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().initComplexFromFile(arg1,arg2,arg3,arg4);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_InitComplexFF::T[4]={ OpValue::STR, OpValue::INT, OpValue::INT, OpValue::INT};

//---------

class PF_ReadGeometryFile : public LatticeMastererPredefFn {
public:
  PF_ReadGeometryFile(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(1,OpValue::STR, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str) ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().readGeometryFile(filename);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

//---------

class PF_SetSpatialDomain : public LatticeMastererPredefFn {
  static OpValue::OType T[6] ;
public:
  PF_SetSpatialDomain(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(6, T, latticeMaster) { } ;
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First(); 
    args->Next();
    double x = args->Get()->Content.d;
    args->Next();
    double y = args->Get()->Content.d;
    args->Next();
    double z = args->Get()->Content.d;
    Vec3 minPt(x, y, z);

    args->Next();
    x = args->Get()->Content.d;
    args->Next();
    y = args->Get()->Content.d;
    args->Next();
    z = args->Get()->Content.d;

    getLatticeMaster().setSpatialDomain(minPt, Vec3(x, y, z));
    SParserAccess::Interface()->SetAllValues();
    return OpValue(1L);
  };
};

//---------

OpValue::OType PF_SetSpatialDomain::T[6] = 
  {
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL
  };

//---------

class PF_AddSaveField :  public LatticeMastererPredefFn{
  static OpValue::OType T[4] ;
public:
  PF_AddSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string fieldname = string(args->Get()->Content.str) ;
    args->Next() ; string filename = string(args->Get()->Content.str) ;
    args->Next() ; integer dt = args->Get()->Content.l ;
    args->Next() ; string type= string(args->Get()->Content.str) ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addSaveField(fieldname,filename,dt,type);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  };
};

OpValue::OType PF_AddSaveField::T[4]={ OpValue::STR, OpValue::STR, OpValue::INT, OpValue::STR};

//---------

class PF_SaveAsDX :  public LatticeMastererPredefFn{
  static OpValue::OType T[2] ;
public:
  PF_SaveAsDX(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(2, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; integer dt = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().setDXSaving(filename,dt);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_SaveAsDX::T[2]={ OpValue::STR, OpValue::INT };

//--------------

class PF_AddScalarParticleSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[6] ;
public:
  PF_AddScalarParticleSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(6, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addScalarParticleSaveField(filename,fieldname,savetype,t0,te,dt);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddScalarParticleSaveField::T[6]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT, OpValue::INT };

//--------------

class PF_AddTaggedScalarParticleSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[8] ;
public:
  PF_AddTaggedScalarParticleSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(8, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    args->Next() ; int tag = args->Get()->Content.l ;
    args->Next() ; int mask = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addTaggedScalarParticleSaveField(filename,fieldname,savetype,t0,te,dt,tag,mask);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddTaggedScalarParticleSaveField::T[8]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT, OpValue::INT,OpValue::INT, OpValue::INT};

//--------------
class PF_AddTaggedScalarParticleFieldDistribution :   public LatticeMastererPredefFn{
  static OpValue::OType T[12] ;
public:
  PF_AddTaggedScalarParticleFieldDistribution(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(12, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt_coll = args->Get()->Content.l ;
    args->Next() ; int dt_save = args->Get()->Content.l ;
    args->Next() ; double x0 = args->Get()->Content.d ;
    args->Next() ; double xm = args->Get()->Content.d ;
    args->Next() ; int nx = args->Get()->Content.l ;
    args->Next() ; int tag = args->Get()->Content.l ;
    args->Next() ; int mask = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addTaggedScalarParticleDistributionSaver(filename,fieldname,savetype,t0,te,dt_coll,dt_save,tag,mask,x0,xm,nx);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddTaggedScalarParticleFieldDistribution::T[12]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT,OpValue::INT,OpValue::INT,OpValue::REAL,OpValue::REAL,OpValue::INT,OpValue::INT,OpValue::INT};

//--------------
class PF_AddScalarInteractionSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[8] ;
public:
  PF_AddScalarInteractionSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(8, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string igtype = string(args->Get()->Content.str);
    args->Next() ; string igname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addScalarInteractionSaveField(filename,fieldname,igtype,igname,savetype,t0,te,dt,false);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddScalarInteractionSaveField::T[8]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT, OpValue::INT };

//--------------
class PF_AddVectorInteractionSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[8] ;
public:
  PF_AddVectorInteractionSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(8, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string igtype = string(args->Get()->Content.str);
    args->Next() ; string igname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addVectorInteractionSaveField(filename,fieldname,igtype,igname,savetype,t0,te,dt,false);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddVectorInteractionSaveField::T[8]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT, OpValue::INT };

//--------------

class PF_AddCheckedScalarInteractionSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[8] ;
public:
  PF_AddCheckedScalarInteractionSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(8, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string igtype = string(args->Get()->Content.str);
    args->Next() ; string igname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addScalarInteractionSaveField(filename,fieldname,igtype,igname,savetype,t0,te,dt,true);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddCheckedScalarInteractionSaveField::T[8]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT, OpValue::INT };

//--------------

class PF_AddTaggedScalarInteractionSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[10] ;
public:
  PF_AddTaggedScalarInteractionSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(10, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string igtype = string(args->Get()->Content.str);
    args->Next() ; string igname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    args->Next() ; int tag = args->Get()->Content.l ;
    args->Next() ; int mask = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addTaggedScalarInteractionSaveField(filename,fieldname,igtype,igname,savetype,t0,te,dt,tag,mask,false);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddTaggedScalarInteractionSaveField::T[10]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT, OpValue::INT, OpValue::INT, OpValue::INT };
//--------------

class PF_AddTaggedCheckedScalarInteractionSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[10] ;
public:
  PF_AddTaggedCheckedScalarInteractionSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(10, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ;
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string igtype = string(args->Get()->Content.str);
    args->Next() ; string igname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    args->Next() ; int tag = args->Get()->Content.l ;
    args->Next() ; int mask = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addTaggedScalarInteractionSaveField(filename,fieldname,igtype,igname,savetype,t0,te,dt,tag,mask,true);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddTaggedCheckedScalarInteractionSaveField::T[10]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR,OpValue::INT, OpValue::INT, OpValue::INT, OpValue::INT, OpValue::INT };

//--------------

class PF_AddVectorParticleSaveField :   public LatticeMastererPredefFn{
  static OpValue::OType T[6] ;
public:
  PF_AddVectorParticleSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(6, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addVectorParticleSaveField(filename,fieldname,savetype,t0,te,dt);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddVectorParticleSaveField::T[6]={OpValue::STR,OpValue::STR,OpValue::STR, OpValue::INT, OpValue::INT, OpValue::INT };

class PF_PerformCheckPoints :   public LatticeMastererPredefFn{
  static OpValue::OType T[4] ;
public:
  PF_PerformCheckPoints(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(4, T, latticeMaster)
  {
  };
  
  OpValue fn(ExpressionNode *args)
  {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next();
    string filename = string(args->Get()->Content.str);
    
    args->Next() ;
    int beginTime = args->Get()->Content.l;
    
    args->Next();
    int endTime = args->Get()->Content.l;
    
    args->Next();
    int timeInterval = args->Get()->Content.l;
    
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().performCheckPoints(filename, beginTime, endTime, timeInterval);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

OpValue::OType PF_PerformCheckPoints::T[4] = 
  {
    OpValue::STR,
    OpValue::INT,
    OpValue::INT,
    OpValue::INT
  };
  
//--------------

class PF_SaveTimingDataToFile :   public LatticeMastererPredefFn{
  static OpValue::OType T[1] ;
public:
  PF_SaveTimingDataToFile(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(1, T, latticeMaster)
  {
  };
  
  OpValue fn(ExpressionNode *args)
  {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next();
    string fileNamePrefix = string(args->Get()->Content.str);

    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().saveTimingDataToFile(fileNamePrefix);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

OpValue::OType PF_SaveTimingDataToFile::T[1] = 
  {
    OpValue::STR
  };
//----------
class PF_AddTaggedVectorParticleSaveField :   public LatticeMastererPredefFn
{
  static OpValue::OType T[8] ;
public:
  PF_AddTaggedVectorParticleSaveField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(8, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    args->Next() ; int tag = args->Get()->Content.l ;
    args->Next() ; int mask = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addTaggedVectorParticleSaveField(filename,fieldname,savetype,t0,te,dt,tag,mask);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddTaggedVectorParticleSaveField::T[8]={OpValue::STR,OpValue::STR,OpValue::STR, OpValue::INT, OpValue::INT, OpValue::INT, OpValue::INT, OpValue::INT };

//--------------
class PF_AddVectorTriangleField :   public LatticeMastererPredefFn
{
  static OpValue::OType T[7] ;
public:
  PF_AddVectorTriangleField(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(7, T, latticeMaster){};
  OpValue fn(ExpressionNode *args) {
    SParserAccess::Interface()->GetAllValues() ;
    args->First() ; 
    args->Next() ; string filename = string(args->Get()->Content.str);
    args->Next() ; string fieldname = string(args->Get()->Content.str);
    args->Next() ; string meshname = string(args->Get()->Content.str);
    args->Next() ; string savetype = string(args->Get()->Content.str);
    args->Next() ; int t0 = args->Get()->Content.l ;
    args->Next() ; int te = args->Get()->Content.l ;
    args->Next() ; int dt = args->Get()->Content.l ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().addVectorTriangleSaveField(filename,fieldname,meshname,savetype,t0,te,dt);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  } ;
} ;

OpValue::OType PF_AddVectorTriangleField::T[7]={OpValue::STR,OpValue::STR,OpValue::STR,OpValue::STR, OpValue::INT, OpValue::INT, OpValue::INT };

//--------
 
class PF_AddRotBondedIG : public LatticeMastererPredefFn {
  static OpValue::OType T[10] ;
public:
    PF_AddRotBondedIG(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(10, T, latticeMaster)
    {
    };

    OpValue fn(ExpressionNode *args) {
        SParserAccess::Interface()->GetAllValues() ;
        args->First() ;
        args->Next() ; int tag = args->Get()->Content.l;
        args->Next() ; char* name = args->Get()->Content.str ;
        args->Next() ; double kr = args->Get()->Content.d ;
        args->Next() ; double ks = args->Get()->Content.d ;
        args->Next() ; double kt = args->Get()->Content.d ;
        args->Next() ; double kb = args->Get()->Content.d ;
        args->Next() ; double max_nForce = args->Get()->Content.d ;
        args->Next() ; double max_shFroce = args->Get()->Content.d ;
        args->Next() ; double max_tMoment = args->Get()->Content.d ;
        args->Next() ; double max_bMoment = args->Get()->Content.d ;

        ISVALID_PTR(&(getLatticeMaster())) ;
        getLatticeMaster().addRotBondedIG(tag,string(name),kr,ks,kt,kb,max_nForce, max_shFroce,max_tMoment, max_bMoment);
        SParserAccess::Interface()->SetAllValues() ;
        return OpValue(1L) ;
    } ;
} ;

OpValue::OType PF_AddRotBondedIG::T[10] =
  {
    OpValue::INT,
    OpValue::STR,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL,
    OpValue::REAL
  };
//---------

class PF_AddRotPIG :  public LatticeMastererPredefFn
{
public:
  PF_AddRotPIG(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(UNLIMITED_ARGS, OpValue::NONE, latticeMaster)
  {
  };
  OpValue fn(ExpressionNode *args) {
    string type;
    string name;
    list<OpValue> params;

    args->First() ;
    args->Next() ; type=string(args->Get()->Content.str);
    args->Next() ; name=string(args->Get()->Content.str);
    args->Next() ;
    while (!args->IsEnd()) {
      OpValue OV;
      ExpressionNode *Ptr = args->Get();
      switch (Ptr->ValueType) {
      case OpValue::INT:
        OV=Ptr->Content.l;
        break;
      case OpValue::REAL:
        OV=Ptr->Content.d;
        break;
      case OpValue::STR:
        OV=Ptr->Content.str;
        break;
      default:
        console.Error()<<"Type error in evaluation of parameter list for AddPIG:\n only int, double and string allowed\n";
        break;
      }
      params.push_back(OV);
      args->Next() ;
    }
    getLatticeMaster().addRotPairIG(type,name,params);
    console.XDebug() << "after TheLatticeMaster->addRotPairIG\n";
    SParserAccess::Interface()->SetAllValues() ;
    console.XDebug() << "##\n";
    return OpValue(1L) ;
  }
};

/*

//---------
                                                                                
class PF_AddRotDamping :  public LatticeMastererPredefFn
{
public:
  PF_AddRotDamping(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(UNLIMITED_ARGS,OpValue::NONE, latticeMaster) {};
  OpValue fn(ExpressionNode *args) {
    string type;
    string name;
    list<OpValue> params;
                                                                                
    args->First() ;
    args->Next() ; type=string(args->Get()->Content.str);
    args->Next() ;
    while (!args->IsEnd()) {
      OpValue OV;
      ExpressionNode *Ptr = args->Get();
      switch (Ptr->ValueType) {
      case OpValue::INT:
        OV=Ptr->Content.l;
        break;
      case OpValue::REAL:
        OV=Ptr->Content.d;
        break;
      case OpValue::STR:
        OV=Ptr->Content.str;
        break;
      default:
        console.Error()<<"Type error in evaluation of parameter list for AddDamping:\n only int, double and string allowed\n";
        break;
      }
      params.push_back(OV);
      args->Next() ;
    }
    TheLatticeMaster->addRotDamping(type,params);
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

*/

//--------------

class PF_do2dCalculations :   public LatticeMastererPredefFn{
  static OpValue::OType T[1] ;
public:
  PF_do2dCalculations(CLatticeMaster &latticeMaster) : LatticeMastererPredefFn(0, T, latticeMaster)
  {
  };
  
  OpValue fn(ExpressionNode *args)
  {
    SParserAccess::Interface()->GetAllValues() ;
    ISVALID_PTR(m_pLatticeMaster) ;
    getLatticeMaster().do2dCalculations();
    SParserAccess::Interface()->SetAllValues() ;
    return OpValue(1L) ;
  }
};

OpValue::OType PF_do2dCalculations::T[1] = 
  {
    OpValue::INT
  };

namespace esys
{
  namespace lsm
  {
    class ParserLmInitialiser::Impl
    {
    public:
      Impl(CLatticeMaster &latticeMaster) : m_pLatticeMaster(&latticeMaster)
      {
      }
      CLatticeMaster *m_pLatticeMaster;
    };

    ParserLmInitialiser::ParserLmInitialiser(
      CLatticeMaster &latticeMaster,
      CParserBridge &bridge
    ) : m_pImpl(new Impl(latticeMaster)) 
    {
      bridge.addLocalInterface(this);
    }

    ParserLmInitialiser::~ParserLmInitialiser()
    {
      delete m_pImpl;
    }

    CLatticeMaster &ParserLmInitialiser::getLatticeMaster()
    {
      return *(m_pImpl->m_pLatticeMaster);
    }

    const CLatticeMaster &ParserLmInitialiser::getLatticeMaster() const
    {
      return *(m_pImpl->m_pLatticeMaster);
    }

    void ParserLmInitialiser::AddVariablesTo(InternVFTable &P) {
      ADDPFN(PF_AddBondedIG(getLatticeMaster()),"AddBondedInteractions");
      ADDPFN(PF_AddPIG(getLatticeMaster()),"AddPairInteractions");
      ADDPFN(PF_AddSIG(getLatticeMaster()),"AddSingleInteractions");
      ADDPFN(PF_AddDamping(getLatticeMaster()),"AddDamping");
      ADDPFN(PF_AddExIG(getLatticeMaster()),"AddExclusion");
      ADDPFN(PF_MakeLattice(getLatticeMaster()),"makeLattice");
      ADDPFN(PF_setNumSteps(getLatticeMaster()),"setNumSteps") ;
      ADDPFN(PF_initWall(getLatticeMaster()),"initWall") ;
      ADDPFN(PF_initBondedWall(getLatticeMaster()),"initBondedWall") ;
      ADDPFN(PF_initSoftBondedWall(getLatticeMaster()),"initSoftBondedWall") ;
      ADDPFN(PF_initViscWall(getLatticeMaster()),"initViscWall") ;
      ADDPFN(PF_runInit(getLatticeMaster()),"runInit");
      ADDPFN(PF_getTimeStep(getLatticeMaster()),"getTimeStep");
      ADDPFN(PF_getTimeStepSize(getLatticeMaster()),"getTimeStepSize");
      ADDPFN(PF_moveWall(getLatticeMaster()),"moveWall") ;
      ADDPFN(PF_applyForceToWall(getLatticeMaster()),"applyForceToWall") ;
      ADDPFN(PF_setVelocityOfWall(getLatticeMaster()),"setVelocityOfWall") ;
      ADDPFN(PF_setPVel(getLatticeMaster()),"SetParticleVelocity") ;
      ADDPFN(PF_setPPos(getLatticeMaster()),"MoveParticleTo") ;
      ADDPFN(PF_setPNonDynamic(getLatticeMaster()),"SetParticleNonDynamic");
      ADDPFN(PF_AddSaveField(getLatticeMaster()),"AddSaveField") ;
      ADDPFN(PF_InitComplexFF(getLatticeMaster()),"initComplexFromFile") ;
      ADDPFN(PF_SaveAsDX(getLatticeMaster()),"saveAsDX") ;
      ADDPFN(PF_ReadGeometryFile(getLatticeMaster()),"readGeometryFile") ;
      ADDPFN(PF_AddScalarParticleSaveField(getLatticeMaster()),"addScalarParticleFieldSaver");
      ADDPFN(PF_AddTaggedScalarParticleSaveField(getLatticeMaster()),"addTaggedScalarParticleFieldSaver");
      ADDPFN(PF_AddVectorParticleSaveField(getLatticeMaster()),"addVectorParticleFieldSaver");
      ADDPFN(PF_PerformCheckPoints(getLatticeMaster()),"performCheckPoints");  
      ADDPFN(PF_AddTaggedVectorParticleSaveField(getLatticeMaster()),"addTaggedVectorParticleFieldSaver");
      ADDPFN(PF_AddScalarInteractionSaveField(getLatticeMaster()),"addScalarInteractionFieldSaver");
      ADDPFN(PF_AddVectorInteractionSaveField(getLatticeMaster()),"addVectorInteractionFieldSaver");
      ADDPFN(PF_AddCheckedScalarInteractionSaveField(getLatticeMaster()),"addCheckedScalarInteractionFieldSaver");
      ADDPFN(PF_AddTaggedScalarInteractionSaveField(getLatticeMaster()),"addTaggedScalarInteractionFieldSaver");
      ADDPFN(PF_AddTaggedCheckedScalarInteractionSaveField(getLatticeMaster()),"addTaggedCheckedScalarInteractionFieldSaver");
      ADDPFN(PF_AddTaggedScalarParticleFieldDistribution(getLatticeMaster()),"addTaggedScalarParticleFieldDistribution");
      ADDPFN(PF_AddVectorTriangleField(getLatticeMaster()),"addVectorTriangleFieldSaver");
      ADDPFN(PF_SetSpatialDomain(getLatticeMaster()),"setSpatialDomain");
      ADDPFN(PF_SetProcessDims(getLatticeMaster()),"setProcessDims");
      ADDPFN(PF_tagPPos(getLatticeMaster()),"tagParticleNearestTo");
      ADDPFN(PF_SaveTimingDataToFile(getLatticeMaster()),"saveTimingDataToFile");
      ADDPFN(PF_AddRotBondedIG(getLatticeMaster()),"AddRotBondedInteractions");
      ADDPFN(PF_AddRotPIG(getLatticeMaster()),"AddRotPairInteractions");
      ADDPFN(PF_AddTriMesh(getLatticeMaster()),"readMeshFile");
      ADDPFN(PF_AddTMIG(getLatticeMaster()),"addTriMeshInteractions");
      ADDPFN(PF_moveNode(getLatticeMaster()),"moveTriMeshNodeBy");
      ADDPFN(PF_moveTaggedNodes(getLatticeMaster()),"moveTaggedTriMeshNodesBy");
      ADDPFN(PF_do2dCalculations(getLatticeMaster()),"do2dCalculations");
      ADDPFN(PF_AddBondedTriMeshIG(getLatticeMaster()),"addBondedTriMeshInteractions");
    }

    void ParserLmInitialiser::DelVariablesFrom(InternVFTable &P) {
      P.DelFnc("makeLattice");
      P.DelFnc("initWall");
      P.DelFnc("initBondedWall");
      P.DelFnc("initSoftBondedWall");
      P.DelFnc("initViscWall");
      P.DelFnc("setNumSteps") ;
      P.DelFnc("runInit") ;
      P.DelFnc("getTimeStep") ;
      P.DelFnc("getTimeStepSize") ;
      P.DelFnc("moveWall") ;
      P.DelFnc("applyForceToWall") ;
      P.DelFnc("setVelocityOfWall") ;
      P.DelFnc("AddBondedInteractions");
      P.DelFnc("AddPairInteractions");
      P.DelFnc("AddSingleInteractions");
      P.DelFnc("AddDamping");
      P.DelFnc("AddExclusion");
      P.DelFnc("SetParticleVelocity") ;
      P.DelFnc("MoveParticleTo") ;
      P.DelFnc("AddSaveField") ;
      P.DelFnc("InitComplexFromFile") ;
      P.DelFnc("SaveAsDX");
      P.DelFnc("readGeometryFile") ;
      P.DelFnc("addScalarParticleFieldSaver");
      P.DelFnc("addTaggedScalarParticleFieldSaver");
      P.DelFnc("addVectorParticleFieldSaver");
      P.DelFnc("performCheckPoints");
      P.DelFnc("addTaggedVectorParticleFieldSaver");
      P.DelFnc("addScalarInteractionFieldSaver");
      P.DelFnc("addVectorInteractionFieldSaver");
      P.DelFnc("addCheckedScalarInteractionFieldSaver");
      P.DelFnc("addTaggedScalarInteractionFieldSaver");
      P.DelFnc("addTaggedCheckedScalarInteractionFieldSaver");
      P.DelFnc("addTaggedScalarParticleFieldDistribution");
      P.DelFnc("addVectorTriangleFieldSaver");
      P.DelFnc("SetParticleNonDynamic");
      P.DelFnc("setSpatialDomain");
      P.DelFnc("setProcessDims");
      P.DelFnc("tagParticleNearestTo");
      P.DelFnc("saveTimingDataToFile");
      P.DelFnc("AddRotBondedInteractions"); 
      P.DelFnc("AddRotPairInteractions");
      P.DelFnc("addTriMeshInteractions");
      P.DelFnc("readMeshFile");
      P.DelFnc("moveTriMeshNodeBy");
      P.DelFnc("moveTaggedTriMeshNodesBy");
      P.DelFnc("do2dCalculations");
      P.DelFnc("addBondedTriMeshInteractions");
    }
  };
};
