#ifndef __SUBLATTICECONTROLER_H
#define __SUBLATTICECONTROLER_H

//--- MPI --- 
#include <mpi.h>

//--- TML includes ---
#include "tml/comm/comm.h"
#include "tml/comm/comm_world.h"

#include "Parallel/SubLattice.h"
#include "Parallel/CheckPointable.h"
#include "Foundation/Timer.h"

#include <boost/shared_ptr.hpp>

class CheckPointer;

/*!
  \class CSubLatticeControler
  \brief class for control of a SubLattice

  Does initialisation and control of a TSubLattice and comunicates with the TLatticeMaster
 
  \author Steffen Abe
  $Revision$
  $Date$
*/
class CSubLatticeControler : esys::lsm::CheckPointable
{
 private:
  int m_global_rank;
  int m_global_size;

  TML_CommWorld m_world_comm; // global MPI communicator

  ASubLattice*  m_lattice;
  CheckPointer* m_pCheckPointer;
  std::string   m_timingFileName;
  typedef boost::shared_ptr<MpiWTimers> MpiWTimersPtr;
  MpiWTimersPtr m_timersPtr;

 public:
  CSubLatticeControler();
  ~CSubLatticeControler();

  void initMPI();
  void makeLattice();
  void initLattice();
  void initLatticeCirc();
  void init2DTriangularLocal();  
  void init3DTriangularLocal();
  void getWall();
  void getBondedWall();
  void getViscWall();
  void getSoftBondedWall();
  void searchNeighbors();
  void performTiming();
  void saveTimingData();
  void setTimingFileName(const std::string &timingFileName) {m_timingFileName=timingFileName;}
  const std::string &getTimingFileName() const {return m_timingFileName;}
  void do2dCalculations();
  void run();

  virtual void saveCheckPointData(std::ostream &oStream);

  virtual void loadCheckPointData(std::istream &iStream);
  
};

#endif //__SUBLATTICECONTROLER_H
