/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "ParallelTest/BondedInteractionCheckPointTest.h"
#include "Model/BondedInteraction.h"

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( BondedInteractionCheckPointTest );

  BondedInteractionCheckPointTest::BondedInteractionCheckPointTest()
      : CppUnit::TestCase()
  {
  }

  BondedInteractionCheckPointTest::~BondedInteractionCheckPointTest()
  {
  }

  void BondedInteractionCheckPointTest::setUp()
  {
    CppUnit::TestCase::setUp();
  }

  void BondedInteractionCheckPointTest::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }

  class TestBondedInteractionData : public BondedInteractionCpData
  {
  public:
  
    TestBondedInteractionData() : BondedInteractionCpData()
    {
    }
    
    TestBondedInteractionData(ParticleId p1Id, ParticleId p2Id, InteractionTag tag)
      : BondedInteractionCpData(p1Id, p2Id, tag)
    {
    }
    
    bool operator==(const TestBondedInteractionData &data) const
    {
      return
        (
          (getP1Id() == data.getP1Id())
          &&
          (getP2Id() == data.getP2Id())
          &&
          (getTag() == data.getTag())
        );
    }
  };
  
  std::ostream &operator<<(std::ostream &oStream, const TestBondedInteractionData &interaction)
  {
    TestBondedInteractionData i = interaction;
    i.saveCheckPointData(oStream);
    return oStream;
  }
  
  void BondedInteractionCheckPointTest::testCheckPointable()
  {
    TestBondedInteractionData data1(1, 2, 100);
    TestBondedInteractionData data2;
    
    CPPUNIT_ASSERT(!(data1 == data2));

    std::stringstream sStream;
    
    data1.saveCheckPointData(sStream);
    data2.loadCheckPointData(sStream);
    
    CPPUNIT_ASSERT_EQUAL(data1, data2);
  }
};
