/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef GEOMETRYREADERTEST_H
#define GEOMETRYREADERTEST_H

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestCase.h>

#include <stdexcept>

namespace LsmTest
{
  /**
   * Test cases for esys::lsm::GeometryReader, esys::lsm::GeometryInfo,
   * esys::lsm::ParticleReader and esys::lsm::ConnectionReader
   */
  class GeometryReaderTest : public CppUnit::TestCase
  {
  
  public:
    /**
     *
     */
    GeometryReaderTest();

    ~GeometryReaderTest();
  
    /**
     * Sets up common data.
     */
    virtual void setUp();
  
    /**
     * Cleans up common data.
     */
    virtual void tearDown();

    CPPUNIT_TEST_SUITE( GeometryReaderTest );

    CPPUNIT_TEST( testGeometryInfoEquality );
    CPPUNIT_TEST( testGeometryInfoReadWrite );
    CPPUNIT_TEST( testParticleReader );
    CPPUNIT_TEST( testParticleDataReadWrite );
    CPPUNIT_TEST( testParticleReaderMultiParticles );
    CPPUNIT_TEST( testConnectionReader );
    CPPUNIT_TEST( testGeometryReaderVersion1_1 );
    CPPUNIT_TEST( testGeometryReaderVersion1_2 );
    CPPUNIT_TEST_EXCEPTION(testParticleReaderThrow, std::runtime_error);    
    CPPUNIT_TEST_EXCEPTION(testConnectionReaderThrow, std::runtime_error);

    CPPUNIT_TEST_SUITE_END();

    void testGeometryInfoEquality();
    void testGeometryInfoReadWrite();

    void testParticleDataReadWrite();
    void testParticleReader();
    void testParticleReaderThrow();
    void testParticleReaderMultiParticles();

    void testConnectionReaderThrow();
    void testConnectionReader();
    void testGeometryReaderVersion1_1();
    void testGeometryReaderVersion1_2();
  };
}; // namespace LsmTest

#endif
