/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "ParallelTest/MeshReaderTest.h"
#include "Parallel/MeshReader.h"
#include "Model/MeshData.h"

// -- STL includes --
#include <sstream>
using std::stringstream;

using namespace esys::lsm;

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( MeshReaderTest );
  
  /*!
    constructor - do nothing
  */
  MeshReaderTest::MeshReaderTest() : FileTestCase()
  {
  }

  /*!
    destructor - do nothing
  */
  MeshReaderTest::~MeshReaderTest()
  {
  }
  
  /*!
   */
  void MeshReaderTest::setUp()
  {
    // Create directory for temporary files.
    FileTestCase::setUp();
  }

  /*!
   */
  void MeshReaderTest::tearDown()
  {
    // Clean up temporary files.
    FileTestCase::tearDown();
  }

  /*!
    test NodeReader 
  */
  void MeshReaderTest::testNodeReader()
  {
    stringstream st;
    
    // generate stream
    st << "\n3D-Nodes 2\n";
    st << "0  42 69 1.0 2.0 3.0 \n"; 
    st << "1  42 0 0.5 0.0 0.0 \n";

    // setup NodeReader
    NodeReader nodeReader(st);
    nodeReader.initialise();

    // tests
    CPPUNIT_ASSERT_EQUAL(2,nodeReader.getNumElements());
    NodeReader::Iterator  &readerIt = nodeReader.getIterator();
    CPPUNIT_ASSERT(readerIt.hasNext());
    MeshNodeData mnd=readerIt.next();
    CPPUNIT_ASSERT_EQUAL(0,mnd.id);
    CPPUNIT_ASSERT_EQUAL(69,mnd.tag);
    CPPUNIT_ASSERT_EQUAL(1.0,mnd.x);
    CPPUNIT_ASSERT_EQUAL(2.0,mnd.y);
    CPPUNIT_ASSERT_EQUAL(3.0,mnd.z);
    CPPUNIT_ASSERT(readerIt.hasNext());
    readerIt.next();
    CPPUNIT_ASSERT(!readerIt.hasNext());
  }

  /*!
    test TriReader
  */
  void MeshReaderTest::testTriReader()
  {
    stringstream st;
    
    // generate stream
    st << "Tri3 2\n";
    st << "0 1 1 2 3\n";
    st << "1 2 3 4 5\n";

    // setup TriReader
    TriReader triReader(st);
    triReader.initialise();
    CPPUNIT_ASSERT_EQUAL(2,triReader.getNumElements());
    TriReader::Iterator  &readerIt = triReader.getIterator();
    CPPUNIT_ASSERT(readerIt.hasNext());
    MeshTriData mnd=readerIt.next();
    CPPUNIT_ASSERT_EQUAL(0,mnd.id);
    CPPUNIT_ASSERT_EQUAL(1,mnd.tag);
    CPPUNIT_ASSERT_EQUAL(1,mnd.p1);
    CPPUNIT_ASSERT_EQUAL(2,mnd.p2);
    CPPUNIT_ASSERT_EQUAL(3,mnd.p3);
    CPPUNIT_ASSERT(readerIt.hasNext());
    readerIt.next();
    CPPUNIT_ASSERT(!readerIt.hasNext());
  }

  const std::string meshDataString =
    "Triangle \n"
    "3D-Nodes 8\n"
    "0 0  0 0.000000 0.000000 0.000000\n"
    "1 1  0 0.500000 0.000000 0.000000\n"
    "2 2  0 1.000000 0.000000 0.000000\n"
    "3 3  0 1.000000 0.500000 0.000000\n"
    "4 4  0 1.000000 1.000000 0.000000\n"
    "5 5  0 0.500000 1.000000 0.000000\n"
    "6 6  0 0.000000 1.000000 0.000000\n"
    "7 7  0 0.000000 0.500000 0.000000\n"
    "Tri3 6\n"
    "0 0 0 1 7\n"
    "1 0 5 6 7\n"
    "2 0 7 1 3\n"
    "3 0 3 5 7\n"
    "4 0 4 5 3\n"
    "5 0 3 1 2\n"
    "Line2 8\n"
    "0  10  0  1\n"
    "2  1  7  0\n"
    "3  20  5  6\n"
    "4  1  6  7\n"
    "9  20  4  5\n"
    "10  2  3  4\n"
    "11  10  1  2\n"
    "12  2  2  3\n"
    "Tri3Face_Contact 0\n"
    "Point1 0";
  /*!
    test reading mesh from file
  */
  void MeshReaderTest::testMeshReader()
  {
    boost::filesystem::path filePath = getTestDirPath() / "testmesh.msh";
    writeFile(filePath, meshDataString);

    MeshReader meshReader(filePath.string());

    // get node,tri and edge iterators
    MeshReader::NodeIterator &niter=meshReader.getNodeIterator();
    MeshReader::TriIterator &titer=meshReader.getTriIterator();

    // check sizes
    CPPUNIT_ASSERT_EQUAL(8,niter.getNumRemaining());
    CPPUNIT_ASSERT_EQUAL(6,titer.getNumRemaining());
  }
};
