/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "ParallelTest/NTableTest.h"
#include "ntable/src/ntable.h"
#include "ntable/src/nt_block.h"
#include "ntable/src/nt_slab.h"
#include "Model/BasicParticle.h"
#include "Foundation/vec3.h"
#include "Geometry/Triangle.h"

#include <iostream>
using std::cout;
using std::endl;

using namespace esys::lsm;

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( NTableTest );
  
   
  /*! 
    constructor - setup NTable
  */
  NTableTest::NTableTest(): CppUnit::TestCase()
  {  
    // setup table
    m_NT=new NeighborTable<CBasicParticle>(3,3,4,1.0,0.2,Vec3(0.0,0.0,0.0),0,0,0);
    // fill with particles
    m_NT->insert(CBasicParticle(Vec3(0.50,1.50,2.50),1.0,0));
    m_NT->insert(CBasicParticle(Vec3(0.54,0.82,2.34),1.0,1));
    m_NT->insert(CBasicParticle(Vec3(0.46,2.00,0.14),1.0,2));
    m_NT->insert(CBasicParticle(Vec3(4.00,2.00,1.00),1.0,3)); // outside
    m_NT->insert(CBasicParticle(Vec3(1.08,1.38,3.19),1.0,4));
    m_NT->insert(CBasicParticle(Vec3(0.49,0.31,2.49),1.0,5));
    m_NT->insert(CBasicParticle(Vec3(1.26,1.49,1.40),1.0,6));
    m_NT->insert(CBasicParticle(Vec3(1.26,1.49,2.40),0.4,7));
  }

  /*!
    destructor
  */
  NTableTest::~NTableTest()
  {
    delete m_NT;
  }

  /*!
   */
  void NTableTest::setUp()
  {
    CppUnit::TestCase::setUp();
  }

  /*!
   */
  void NTableTest::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }

  /*!
    test getParticlesNearTriangle
  */
  void NTableTest::TestTriangle()
  {
    Vec3 p1=Vec3(1.1,1.1,1.1);
    Vec3 p2=Vec3(2.1,2.1,1.1);
    Vec3 p3=Vec3(1.1,2.1,2.1);
    Triangle Tr=Triangle(1,2,3,p1,p2,p3,0,0);
    T_Handle<NeighborTable<CBasicParticle>::particlelist> parlist=m_NT->getParticlesNearTriangle(Tr);

    CPPUNIT_ASSERT_EQUAL(size_t(2),parlist->size());
    NeighborTable<CBasicParticle>::particlelist::iterator iter=parlist->begin();
    CPPUNIT_ASSERT_EQUAL(6,(*iter)->getID());
  }
};
