/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "ParallelTest/ParticleCheckPointTest.h"
#include "Model/Particle.h"

namespace LsmTest
{
  CPPUNIT_TEST_SUITE_REGISTRATION( ParticleCheckPointTest );

  ParticleCheckPointTest::ParticleCheckPointTest()
      : CppUnit::TestCase()
  {
  }


  ParticleCheckPointTest::~ParticleCheckPointTest()
  {
  }

  void ParticleCheckPointTest::setUp()
  {
    CppUnit::TestCase::setUp();
  }

  void ParticleCheckPointTest::tearDown()
  {
    CppUnit::TestCase::tearDown();
  }

  class TestParticle : public CParticle
  {
  public:
    TestParticle() : CParticle()
    {
    }

    TestParticle(
      double rad,
      double mass,
      const Vec3& pos,
      const Vec3& vel,
      const Vec3& force,
      int id
    ) : CParticle(rad, mass, pos, vel, force, id)
    {
    }

    virtual ~TestParticle()
    {
    }

    bool operator==(const TestParticle &particle) const
    {
      return
        (
          (getPos()    == particle.getPos())
          &&
          (getRad()    == particle.getRad())
          &&
          (getID()     == particle.getID())
          &&
          (getTag()    == particle.getTag())
          &&
          (getMass()   == particle.getMass())
          &&
          (getInitPos()== particle.getInitPos())
          &&
          (getOldPos() == particle.getOldPos())
          &&
          (getVel()    == particle.getVel())
          &&
          (getForce()  == particle.getForce())
        );
    }
  };

  void ParticleCheckPointTest::testCheckPointable()
  {
    TestParticle particle1(1.0, 2.0, Vec3(1.0, 2.0, 3.0), Vec3(10.0, 20.0, 30.0), Vec3(0.1, 0.2, 0.3), 1);
    TestParticle particle2;
    
    std::stringstream sStream;
    
    CPPUNIT_ASSERT(!(particle1 == particle2));
    
    particle1.saveCheckPointData(sStream);
    particle2.loadCheckPointData(sStream);
    
    CPPUNIT_ASSERT_EQUAL(particle1, particle2);
  }
};
