/********************************************************************************/
/* GLOBALS.H		 							*/
/* 										*/
/* Description:	  Parser predefined functions and globals varibale		*/
/* Author(s):     D. Place (QUAKES)						*/
/* Creation date: March 1999							*/
/*										*/
/********************************************************************************/
#include "stdafx.h"
#include "globals.h"
#include "functions.h"
#include <string.h>

InternVFTable::InternVFTable() : List<InternVariable> (), ParserStatics () {
    Init() ;
}

void InternVFTable::Destroy() {
    First() ;
    while (!IsEnd()) {
	DB_DELETE(Get(),"InternVariable") ;
//	if (VT) VT->DelSymbol(Get()->Sym->name) ;
	delete Get() ;
	Clear() ;
    }
}
  
int InternVFTable::DelFnc(char *name) {
  DB_ASSERT(PF) ;
  PF->RemoveFunction(name) ;
  return true ;
}

int InternVFTable::DelVar(char* name) { 
    First() ;
    while (!IsEnd()) {
      InternVariable *GV = Get() ;
      if (!strcmp(GV->Sym->name,name)) {
	VT->DelSymbol(name) ;
	DB_DELETE(GV,"InternVariable") ;
	delete GV ;
	Clear() ;
      } else {
	Next() ;
      }
    }
  return true ;
}

void InternVFTable::Init() {
}
    
int InternVFTable::AddVar(char* VarName, char **PtrValue) {
  Symbol *Sym = VT->AddGlobalSymbol(VarName, OpValue::STR) ;
  if (!Sym) return false ;
  InternVariable *GV = new InternVariable ;
  DB_NEWALLOC(GV,"InternVariable") ;
  GV->Sym = Sym ;
  GV->Update = true ;
  GV->Content.str = PtrValue ;
  *this << GV ;
  return true ;
}

int InternVFTable::AddVar(char* VarName, integer *PtrValue) {
  Symbol *Sym = VT->AddGlobalSymbol(VarName, OpValue::INT) ;
  if (!Sym) return false ;
  InternVariable *GV = new InternVariable ;
  DB_NEWALLOC(GV,"InternVariable") ;
  GV->Sym = Sym ;
  GV->Update = true ;
  GV->Content.l = PtrValue ;
  *this << GV ;
  return true ;
}

int InternVFTable::AddVar(char* VarName, real *PtrValue) {
  Symbol *Sym = VT->AddGlobalSymbol(VarName, OpValue::REAL) ;
  if (!Sym) return false ;
  InternVariable *GV = new InternVariable ;
  DB_NEWALLOC(GV,"InternVariable") ;
  GV->Sym = Sym ;
  GV->Update = true ;
  GV->Content.d = PtrValue ;
  *this << GV ;
  return true ;
}

int InternVFTable::AddArray(char* VarName, integer *PtrValue, int NDim, int *Dims) {
  Symbol *Sym = VT->AddGlobalSymbol(VarName, OpValue::PTR) ;
  if (!Sym) return false ;
  InternVariable *GV = new InternVariable ;
  DB_NEWALLOC(GV,"InternVariable") ;
  GV->Sym = Sym ;
  GV->Update = false ;
  GV->Content.l = PtrValue ;
  *this << GV ;
  return Sym->value->Allocate(OpValue::INT, 1, NDim, Dims) ;
}

int InternVFTable::AddArray(char* VarName, real *PtrValue, int NDim, int *Dims) {
  Symbol *Sym = VT->AddGlobalSymbol(VarName, OpValue::PTR) ;
  if (!Sym) return false ;
  InternVariable *GV = new InternVariable ;
  DB_NEWALLOC(GV,"InternVariable") ;
  GV->Sym = Sym ;
  GV->Update = false ;
  GV->Content.d = PtrValue ;
  *this << GV ;
  return Sym->value->Allocate(OpValue::REAL, 1, NDim, Dims) ;
}

int InternVFTable::AddFnc(char *name, PredefinedFunction *Fn) {
  DB_ASSERT(PF) ;
  PF->AddFunction(name, Fn) ;
  return true ;
  }

int InternVFTable::AddArray(char* VarName, char **PtrValue, int NDim, int *Dims) {
  Symbol *Sym = VT->AddGlobalSymbol(VarName, OpValue::PTR) ;
  if (!Sym) return false ;
  InternVariable *GV = new InternVariable ;
  DB_NEWALLOC(GV,"InternVariable") ;
  GV->Sym = Sym ;
  GV->Update = false ;
  GV->Content.str = PtrValue ;
  *this << GV ;
  return Sym->value->Allocate(OpValue::STR, 1, NDim, Dims) ;
}

