#include "stdafx.h"
#include "mainparser.h"
#include "console.h"
#include "functions.h"
#include "parse.h"

//-- IO includes --
#include <fstream>
using std::ifstream;


/********************************************************************************/
/* class									*/
/* 										*/
/* Description:									*/
/* 										*/
/* 										*/
/********************************************************************************/

VariableTable *ParserStatics::VT=NULL ;
PredefFunctionTable *ParserStatics::PF=NULL ;

void ParserStatics::Recycle() {
    if (VT) {
      VT->Destroy() ;
      VT->Init() ;
    } else {
	VT = new VariableTable ;
        DB_NEWALLOC(VT,"VariableTable") ;
    }
    if (PF) {
      PF->Destroy() ;
      PF->Init() ;
    } else {
	PF = new PredefFunctionTable ;
        DB_NEWALLOC(PF,"PredefFunctionTable") ;
    }
}

void ParserStatics::Create() {
    if (!VT) {
	VT = new VariableTable ;
        DB_NEWALLOC(VT,"VariableTable") ;
    }
    if (!PF) {
	PF = new PredefFunctionTable ;
        DB_NEWALLOC(PF,"PredefFunctionTable") ;
    }
}

void ParserStatics::Destroy() {
    if (VT) { 
	VT->Destroy() ;
	DB_DELETE(VT,"?") ;
	delete VT ;
	VT=NULL ;
    }
    if (PF) {
	PF->Destroy() ; 
	DB_DELETE(PF,"?") ;
	delete PF ;
	PF=NULL ;
    }
}


/********************************************************************************/
/* class									*/
/* 										*/
/* Description:									*/
/* 										*/
/* 										*/
/********************************************************************************/

char *ParserError::filename ;
void ParserError::Error(char *msg, char *msg2) {
    console.Error() << "--------------------------------------------------------\n" ;
    console.Error() << msg << msg2 << "\n\n" ;
}

void ParserError::Error(char *msg, yyltype *loc) 
{
    char line[10000] ;
    integer i ;
    integer nline ;
    char *thefile ;

    if (!loc->text) {
       thefile = filename ;
    } else {
       thefile = loc->text ;
    }
    ifstream Input(thefile) ;
    
    nline = 1 ;
    //    console.Error() << "--------------------------------------------------------\n" ;
    console.Error() << "*** Parsing error in file \"" << thefile << "\"\n" ;
    console.Error() << "*** " << msg << " at line " << loc->first_line << ":\n" ;
    while (Input.good()) {
	Input.getline(line,10000) ;
	if ( nline == loc->first_line) {
	  console.Error() << "    " << line << "\n" ;
	  console.Error() << " ---" ; 
	    for (i=0; i<loc->first_column-1; i++) console.Error(false) << "-" ;
	    console.Error(false) << "^\n" ;
	}
	nline ++ ;
    }
}

/********************************************************************************/
/* class									*/
/* 										*/
/* Description:									*/
/* 										*/
/* 										*/
/********************************************************************************/
InternVFTable *AParserInterface::VFTable = NULL;

int AParserInterface::SetAllValues() {
    int res = true ;
    VFTable->First() ;
    while (!VFTable->IsEnd()) {
        ISVALID_PTR(VFTable->Get()) ;
	if (VFTable->Get()->Update)  {	  
	  ISVALID_PTR(VFTable->Get()->Sym) ;
	  ISVALID_PTR(VFTable->Get()->Sym->value) ;
	  res = res & VFTable->Get()->Sym->value->GetValueFrom(VFTable->Get()->Content.WhatEver) ;
	}
	VFTable->Next() ;
    }
    return res ;
}

int AParserInterface::GetAllValues() {
    int res = true ;
    VFTable->First() ;
    while (!VFTable->IsEnd()) {
	if (VFTable->Get()->Update) 
	res = res & VFTable->Get()->Sym->value->CopyValueTo(VFTable->Get()->Content.WhatEver) ;
	VFTable->Next() ;
    }
    return res ;
}

void AParserInterface::Create() {
    if (!VFTable) {
	VFTable = new InternVFTable ;
	DB_NEWALLOC(VFTable,"InternVFTable") ;
    }
}


int AParserInterface::IsCreated() {
  return (VFTable!=NULL) ;
}

void AParserInterface::Destroy() {
    if (VFTable) {
	VFTable->Destroy() ;
	DB_DELETE(VFTable,"InternVFTable") ;
	delete VFTable ;
	VFTable = NULL ;
    }
}


OpValue * AParserInterface::Value(char *name) {
    VFTable->First() ;
    while (!VFTable->IsEnd()) {
	if (!strcmp(VFTable->Get()->Sym->name,name)) 
	return VFTable->Get()->Sym->value ;
	VFTable->Next() ;
    }
    return NULL ;
}

int AParserInterface::GetValue(char *name) {
    int res = true ;
    VFTable->First() ;
    while (!VFTable->IsEnd()) {
	if (!strcmp(VFTable->Get()->Sym->name,name)) 
	res = res & VFTable->Get()->Sym->value->CopyValueTo(VFTable->Get()->Content.WhatEver) ;
	VFTable->Next() ;
    }
    return res ;
}

int AParserInterface::SetValue(char *name) {
    int res = true ;
    if (!VFTable) return false ;
    VFTable->First() ;
    while (!VFTable->IsEnd()) {
	if (!strcmp(VFTable->Get()->Sym->name,name)) 
	res = res & VFTable->Get()->Sym->value->GetValueFrom(VFTable->Get()->Content.WhatEver) ;
	VFTable->Next() ;
    }
    return res ;
}

InternVFTable* AParserInterface::GetVarTable() {
    return VFTable ;
}



/********************************************************************************/
/* class									*/
/* 										*/
/* Description:									*/
/* 										*/
/* 										*/
/********************************************************************************/

AParser::AParser() : ParserStatics(), ParserError() {
} ;

AParser::AParser(AParserInterface *iPI) : ParserStatics(), ParserError() {
    if (iPI) PI = iPI ;
} ;


AParserInterface * AParser::Interface() { 
  ISVALID_PTR(PI) ;
  return PI ; 
} 

int AParser::Eval(char *fn) {
    if (PF) { // If no function table then no evaluation...
      int res ;
//      console.Debug() << "Evaluating " << fn << "\n" ;
      OpValue FnName(fn) ;
      FunctionCall FC(FnName) ;
      res = FC.Eval() ;
      if (res) res = Interface()->GetAllValues() ;
      return res ;
    } else {
      return false ; 
    }
}

int AParser::Parse(char *file) {
    extern FILE *yyin,*yyout ;
    extern void lexerinit(void) ;
    int res;

    // Open the file and sets stdion/out for yyparse
    filename = file ;
    yyin = fopen(filename,"r+") ;
    yyout = stdout ;
    
    // Initialise pasers global variables
    if (!Interface()->IsCreated()) {
       Create() ;			// Make sure static variables are allocated
//       Interface()->GetAllModelVariables() ;	// allocate variables in parser and link them with the model
//       Interface()->GetModelFunctions() ;
    }
    Interface()->SetAllValues() ;	// set the values
    PF->RemoveFunction("main") ;// remove pre-existing function "main" if any
    
    // encode/parse the file
    lexerinit() ;
    res = yyparse() ;
    fclose(yyin) ;
    console.flush() ;
//    if (res) {
//      console.Error() << "Syntax Error \n" ;
//	return false;
//    }
    
    // eval "main" function
/*
    res = Eval("main") ;
    if (!res) {
      console.Error() << "Incorrect evaluation\n" ;
    }
*/
    return !res ;

}

void AParser::Destroy() {
    Interface()->Destroy() ;
    ParserStatics::Destroy() ;
}

void AParser::Create() {
    ParserStatics::Create() ;
    Interface()->Create() ;
}

