/********************************************************************************/
/* MAINPARSER.H		 							*/
/* 										*/
/* Description:	  Parser evaluation functions					*/
/* Author(s):     D. Place (QUAKES)						*/
/* Creation date: March 1999							*/
/*										*/
/********************************************************************************/
#ifndef _MAINPARSER_H_
#define _MAINPARSER_H_

#include "mtypes.h"

#define ADDFN(fn,str) { fn *FN = new fn ; \
    DB_NEWALLOC(FN,str) ; \
    P.AddFnc(str,FN) ; \
}
                        
#define ADDPFN(fn,str) { PredefinedFunction *FN = new fn ; \
    DB_NEWALLOC(FN,str) ; \
    P.AddFnc(str,FN) ; \
}

class PredefFunctionTable ;

/*!
  \class ParserStatics
  \brief Singleton for table for variable & functions
*/
class ParserStatics {
protected:
  static VariableTable *VT;          //!< Varible table use in script
  static PredefFunctionTable *PF;    //!< Table of all defined functions (model function & intrinsic function)

public:
    ParserStatics() {} ;
    
    static void Create() ;    //!< Create varible table and Function table
    static void Destroy() ;   //!< Destroy varible table and Function table
    static void Recycle() ;   //!< Destroy & Create varible table and Function table
} ;


/*!
  \class ParserError
  \brief Singleton for handling error in script
*/
class ParserError {
protected:
    static char *filename ;
public:
    ParserError() {} ;
    static void Error(char *msg, yyltype *loc) ;
    static void Error(char *msg, char *msg2) ;
}; 


class InternVFTable ;

/*!
  \class AParserInterface
  \brief Parser Interface, access to parser evaluation
*/
class AParserInterface {
protected: 
    
  static InternVFTable *VFTable ; //!< Intern variable table link with program variable
    
public:
    AParserInterface() {} ;
    
    virtual int SetAllValues() ;
    virtual int GetAllValues() ; 
    virtual void Create() ;
    virtual int IsCreated() ;
    virtual void Destroy() ;
    OpValue * Value(char *name) ;
    virtual int GetValue(char *) ; 
    virtual int SetValue(char *) ;
    InternVFTable* GetVarTable() ;
} ;


/*!
  \class AParser
  \brief Creation of parser etc...
*/
class AParser: virtual public ParserStatics, virtual public ParserError {
protected: 
    AParserInterface *PI ;
public:
    virtual AParserInterface * Interface() ;
    AParser() ;
    AParser(AParserInterface *) ;
    virtual int Eval(char *fn) ;	//!< Eval a specific function (no args) 
    virtual int Parse(char *file) ;     //!< Analyse & encode file then evaluate "main"
    void Create() ;                     //!< Create Parser
    virtual void Destroy() ;		//!< Clear all global parsers variables
    
} ;


#endif



