/********************************************************************************/
/* STATEMENTS.CPP	 							*/
/* 										*/
/* Description:	  Parser evaluation functions					*/
/* Author(s):     D. Place (QUAKES)						*/
/* Creation date: March 1999							*/
/*										*/
/********************************************************************************/
#include "stdafx.h"
#include "statements.h"
#include <string.h>
#include "parse.h"


/********************************************************************************/
/* Statement classes		 						*/
/* 										*/
/* Description:									*/
/*   Statement for, do and while (cf constructor for the child values)		*/
/* 										*/
/********************************************************************************/

WhileStatement::WhileStatement(ExpressionNode *Predicat, ExpressionNode *Block) {
    First() ;
    *this << Predicat << Block ;
    ExpressionType="WhileStatement";
}

int WhileStatement::Eval() {
    First() ;
    ExpressionNode *Predicat=Get();
    Next() ;
    ExpressionNode *Block=Get() ;
    if (!Predicat->Eval()) return false ;
    while ( Predicat->GetIntVal() != 0L ) {
	if (!Block->Eval()) return false ;
	if (return_has_been_reached) return true ;
	if (!Predicat->Eval()) return false ;
    }
    return true;
}

/********************************************************************************/

DoStatement::DoStatement(ExpressionNode *Predicat, ExpressionNode *Block) {
    First() ;
    *this << Predicat << Block ;
    ExpressionType="DoStatement";
}

int DoStatement::Eval() {
    First() ;
    ExpressionNode *Predicat=Get();
    Next() ;
    ExpressionNode *Block=Get() ;
    if (!Block->Eval()) return false ;
    if (!Predicat->Eval()) return false ;
    while ( Predicat->GetIntVal() != 0L ) {
	if (!Block->Eval()) return false ;
	if (return_has_been_reached) return true ;
	if (!Predicat->Eval()) return false ;
    }
    return true;
}

/********************************************************************************/

ForStatement::ForStatement(ExpressionNode *Pre, 
			   ExpressionNode *Predica, 
			   ExpressionNode *Post, 
			   ExpressionNode *Block) {
    First() ;
    *this << Pre << Predica << Post << Block ;
    ExpressionType="ForStatement";
}

int ForStatement::Eval() {
    First() ;
    ExpressionNode *Pre=Get();
    Next() ;
    ExpressionNode *Predicat=Get();
    Next() ;
    ExpressionNode *Post=Get();
    Next() ;
    ExpressionNode *Block=Get() ;
    if (!Pre->Eval()) return false ;
    if (!Predicat->Eval()) return false ;
    while ( Predicat->GetIntVal() != 0L ) {
	if (!Block->Eval()) return false ;
	if (return_has_been_reached) return true ;
	if (!Post->Eval()) return false ;
        if (!Predicat->Eval()) return false ;
    }
    return true;
}

/********************************************************************************/
/* class IfStatement		 						*/
/* 										*/
/* Description:									*/
/*   structure of a if (Px is an expression or predicat, Bx is a block) :	*/
/*   if (P1) B1 elseif (P2) B2 elseif (P3) B3 ... else Bn:			*/
/*   the list contains: P1 B1 P2 B2 P3 B3 ... 1 Bn				*/
/* 										*/
/********************************************************************************/

IfStatement::IfStatement(ExpressionNode *Predicat, ExpressionNode *Block) {
    First() ;
    *this << Predicat << Block ;
    ExpressionType="IfStatement";
}    

int IfStatement::Eval() {
    ExpressionNode *Predicat,*Block ;
    First() ;
    while (!IsEnd()) {
	Predicat=Get();
	Next() ;
	Block=Get() ;
	Next() ;
	if (!Predicat->Eval()) return false ;
	if ( Predicat->GetIntVal() != 0L ) {
	    return Block->Eval() ;
	}
    }
    return true;
}    

/********************************************************************************/
/* class block			 						*/
/* 										*/
/* Description:									*/
/*   A list of expression							*/
/* 										*/
/********************************************************************************/

Block::Block(ExpressionNode *instruction) {
    First() ;
    *this << instruction ;
    ExpressionType="Block";
}

Block::Block() {
}
    
int Block::Eval() {
    int res ;
    First() ;
    while (!IsEnd()) {
	res = Get()->Eval() ;
	if ( ( (res) && (return_has_been_reached) ) || (!res) ) {
	    return res ;
	} 
	Next() ;
    }
    return true;
}

