#include "stdafx.h"
#include "console.h"
#include "LSMSocket.h"
#include <string.h>
#ifndef _WINDOWS
#include <unistd.h>
#endif

/********************************************************************************/
/* class CShakePacket   	 						*/
/* 							     			*/
/* Description:	     								*/
/*   send/receive data through sockets                                          */
/********************************************************************************/

CShakePacket::CShakePacket() {
	m_data = NULL ;
}

CShakePacket::CShakePacket(void *data, unsigned long length, char *type) {
  init(data,length,type) ;
}

CShakePacket::CShakePacket(Cxdr &cxdr, char *type) {
  init(cxdr.data(),cxdr.len(),type) ;
}

void CShakePacket::init(void *data, unsigned long length, char *type) {
	m_data = data ;
	m_length = length ;
	if (type==NULL) {
		strcpy(&m_rtype[0],"<RAW>") ;
	} else {
		strcpy(&m_rtype[0],"<   >") ;
		if (strlen(type)<3) {
			tconsole.Error() << "Type for socket packet sould be at least 3 characters\n" ;
			strcpy(&m_rtype[0],"<RAW>") ;
		} else {
			m_rtype[1]=type[0] ;		
			m_rtype[2]=type[1] ;		
			m_rtype[3]=type[2] ;		
		}
	}
	m_type[0]=m_rtype[1] ;
	m_type[1]=m_rtype[2] ;
	m_type[2]=m_rtype[3] ;
	m_type[3]='\0' ;
}

bool CShakePacket::sendTo(CSocket &s) {
  Cxdr xdr_length ;
  Cxdr xdr_THIS((char *)m_data,m_length) ;

  s.send((void *)"[PKG]") ;
  s.send((void *)(&m_rtype[0])) ;
  xdr_length << m_length ;
  s.send(xdr_length) ;
  s.send(xdr_THIS) ;
  s.send((void *)"[END]") ;
  //tconsole.Debug() << m_length << "bytes sent.\n" ;
  return true ;

	/* -- we don't do ACK in asynchronous msg... */
	/*
	char ackMark[6] ;
	s.recv((void *)ackMark,5) ;
	ackMark[5]='\0' ;
	if (!strcmp(ackMark,"[ACK]")) {
		return true ;
	} else {
		tconsole.Error() << "Packet error, didn't receive [ACK]\n" ;
		return false ;
	}
	*/
}

bool CShakePacket::receiveFrom(CSocket &s) {
	char beginMark[6] ;
	char endMark[6] ;
	void *data ;
	unsigned long thelength ;
	int val ;
	bool blk ;
	Cxdr xdr_length;

	val = s.recv((void *)beginMark,5) ;
        //tconsole.Debug() << "returned val="<<val<<" err="<<WSAGetLastError()<<"\n" ;
	if ((val<=0) && s.wouldBlock() ) {
		return false ;
	}
	blk = s.isNonBlocking() ;
	s.nonBlocking(false) ;
	beginMark[5]='\0' ;
	if (!strcmp(beginMark,"[PKG]")) {
		s.recv((void *)(&m_rtype[0]),5) ;
		m_rtype[5]='\0' ;
		m_type[0]=m_rtype[1] ;
		m_type[1]=m_rtype[2] ;
		m_type[2]=m_rtype[3] ;
		m_type[3]='\0' ;
		s.recv(xdr_length) ;
		xdr_length >> thelength ;
		//tconsole.Debug() << "Length of packet is " << thelength << "\n" ;
		if (m_data==NULL) {
			m_data = (void *)(new char[thelength]) ;
			data = m_data ;
		} else {
			if ((m_length < thelength) && (m_length !=0)) {
				tconsole.Error() << "buffer too small to receive data\n" ;
				data = (void *)(new char[thelength]) ;
			} else {
				data = m_data ;
			}
		}
		m_length = thelength ;
		//tconsole.Debug() << "Receiving " << m_length << " bytes ...\n" ;
		Cxdr xdr_THIS((char *)data,m_length) ;
		s.recv(xdr_THIS) ;
		s.recv((void *)endMark,5) ;
		endMark[5]='\0' ;
		s.nonBlocking(blk) ;
		if (!strcmp(endMark,"[END]")) {
		  //tconsole.Debug() << "Packet received.\n" ;
		        // no ACK ...  s.send((void *)"[ACK]") ;
			return true ;
		}
		tconsole.Error() << "Packet error, missing [END] marker\n" ;
		return false ;
	} else {
		s.nonBlocking(blk) ;
		tconsole.Error() << "Packet error, missing [PKG] marker\n" ;
		return false ;
	}
}

int CShakePacket::getSize() { 
	return m_length; 
} 

char *CShakePacket::getType() { 
	return &m_type[0] ; 
} 

void *CShakePacket::getData() {
	return m_data ;
}


/********************************************************************************/
/* class CShakeSocket                                                           */
/*                                                                              */
/* Description:                                                                 */
/*   bas class for client/server socket application                             */
/********************************************************************************/

CShakeSocket::CShakeSocket(char *Id, int startPORT, int endPORT) {
	m_minPort=startPORT ;
	m_maxPort=endPORT ; 
	m_thePort=startPORT ;
	m_Id = new char[strlen(Id)+1] ;
	strcpy(m_Id,Id) ;
        m_connected = false ;
}

CShakeSocket::~CShakeSocket() {
  if (m_connected) {
    m_s.close() ;
    //m_s.shutdown() ;
    m_connected = false ;
  }
  delete m_Id ;
}

bool CShakeSocket::close() {
  m_connected = false ;
  return getSocket().close() ;  
}

//int recv(char *data, int bufsize=0) ; 
//int send(char *data, int nLength=0) ; 

int CShakeSocket::recv(CShakePacket &packet) {
	if (packet.receiveFrom(getSocket())) {
		return packet.getSize() ;
	} else {
		return -1 ;
	}
}

bool CShakeSocket::wouldBlock() {
  return getSocket().wouldBlock() ;
}


int CShakeSocket::recv(char *data, int bufsize) {
	bool res ;

	CShakePacket packet((void *)data,bufsize) ;
	res = packet.receiveFrom(getSocket()) ;
	if (!res) {
		return -1 ;
	} else {
		return packet.getSize() ;
	}
}


int CShakeSocket::send(CShakePacket &packet) {
	if (packet.sendTo(getSocket())) {
		return packet.getSize() ;
	} else {
		return -1 ;
	}
}


int CShakeSocket::send(char *data, int nLength) {
	bool res ;

	int len = nLength;
	if (len == 0) {
		len = strlen(data)+1 ;
	}
	CShakePacket packet((void *)data,len) ;
	res = packet.sendTo(getSocket()) ;
	if (!res) {
		return -1 ;
	} else {
		return packet.getSize() ;
	}
}


void CShakeSocket::nonBlocking(bool flag) {
	getSocket().nonBlocking(flag) ;
}

CSocket & CShakeSocket::getSocket() {
	return m_s ;
}

bool CShakeSocket::connected() {
	return m_connected ;
}


/********************************************************************************/
/* class CShakeSocketClient   							*/
/*                                                                              */
/* Description:                                                                 */
/*   client socket                                                              */
/********************************************************************************/

CShakeSocketClient::CShakeSocketClient(char *Id) : CShakeSocket(Id) {
}

CShakeSocketClient::~CShakeSocketClient() {
  if (m_connected) {
    m_s.close();
    //m_s.shutdown() ;
    m_connected = false ;
  }
}


bool CShakeSocketClient::connectTo(char *host, int startPORT, int endPORT, char *Id) {
	bool res ;
	char *msg = new char[1024] ;
	
	if (startPORT!=0) {
	  m_minPort=startPORT ;
	  if (endPORT!=0) {
	    m_maxPort=endPORT ; 
	  } else {
	    m_maxPort=startPORT+10 ; 
	  }
	  m_thePort=startPORT ;
	}

	m_s.create();	
	res=false ;

	m_thePort=m_minPort-1 ;
	while ((!res) && (m_thePort<m_maxPort)) {
		m_thePort++ ;
		//tconsole.Debug() << m_Id << " trying to connect at " << host << ":" << m_thePort << " \n" ;
		res = m_s.connect(m_thePort, host);
		if (res) {
#ifndef _WINDOWS
		  //		        sleep(1) ;
#endif
			tconsole.Debug() << m_Id << " connected to " << host << ":" << m_thePort << ", sending Id..\n" ;
			send(m_Id) ;
			tconsole.Debug() << " receiving Id..\n" ;
			recv(msg,1024) ;
			if (Id!=NULL) {
				if (!strcmp(msg,Id)) { // we got the requested server
					m_ServerId = new char[strlen((char *)(msg))+1] ;
					strcpy(m_ServerId,msg) ;
					delete msg ;
					send("ACCEPT\0") ;
				} else { // oops, wrong server
					send("DENY\0") ;
					m_s.close() ;
					m_s.create() ;
					res = false ;
				}
			} else {
			  m_ServerId = new char[strlen((char *)(msg))+1] ;
			  strcpy(m_ServerId,msg) ;
			  delete msg ;
			  send("ACCEPT\0") ;
			}
		} else {
		  m_s.close() ;
		  m_s.create() ;
		}
	}
	
	if (!res) {
		m_s.close();
		delete msg ;
		return false ;
	}
	
	tconsole.Debug() << m_Id << " Connected to " << m_ServerId << " \n" ;
        m_connected = true ;
	return true ;
}

/********************************************************************************/
/* class CShakeSocketServer                                                     */
/*                                                                              */
/* Description:                                                                 */
/*   server socket                                                              */
/********************************************************************************/

CShakeSocketServer::CShakeSocketServer(char *Id, int startPORT, int endPORT) : CShakeSocket(Id,startPORT,endPORT) {
  m_created = false ;
}

CShakeSocketServer::~CShakeSocketServer() {
  if (m_connected) {
    m_client.close() ;
    //m_client.shutdown() ;
    m_connected = false ;
  }
  m_s.close() ;
  //m_s.shutdown() ;
}

bool CShakeSocketServer::create() {
  bool res=false ;

  m_s.create();
  m_thePort=m_minPort-1 ;
  while ((!res) && (m_thePort<=m_maxPort)) {
    m_thePort++ ;
    //tconsole.Debug() << m_Id << " trying to alloc port " << m_thePort << " \n" ;
    res = m_s.bind(m_thePort);
  }
  if (!res) {
    return false ;
  }

  res = m_s.listen();
  m_created = true ;
  return res ;
}

bool CShakeSocketServer::isCreated() {
  return m_created ;
}

bool CShakeSocketServer::accept() {
	char *msg = new char[1024] ;

	bool res=false;
	
	while(!res) {
		// wait for a connection ....
		res = m_s.accept(m_client); 		
		if ((!res)&&(m_s.wouldBlock())) {
		  // tconsole.Debug() << m_Id << " would block...\n" ;
		  delete msg ;
		  return false ;
		} else if (!res) {
		  tconsole.Warning() << m_Id << " cannot connect...\n" ;
		  delete msg ;
		  return false ;
		} else {
		  tconsole.Debug() << m_Id << " got a connection...\n" ;
		}

		m_s.nonBlocking(false) ; // set to blocking to negociate
		m_client.nonBlocking(false) ; // set to blocking to negociate

		// get Client Id.
		if (recv(msg,1024)==-1) {
		  m_client.close() ;
		  tconsole.Debug() << m_ClientId << " Error while receiving Client Id \n" ;
		  return false ;
		}
		m_ClientId = new char[strlen((char *)(msg))+1] ;
		strcpy(m_ClientId,msg) ;

		// send My Id
		if (send(m_Id)==-1) {
		  m_client.close() ;
		  tconsole.Debug() << m_ClientId << " Error while sending Id \n" ;
		  return false ;
		}
		
		// Is the client accept my Id?
		recv(msg, 1024) ;
		if (!strcmp(msg,"ACCEPT")) {
			tconsole.Debug() << m_Id << " connected to " << m_ClientId << " \n" ;
			m_connected = true ;
			res = true ;
		} else if (!strcmp(msg,"DENY")) {
			tconsole.Debug() << m_ClientId << " denied connection to " << m_Id << " \n" ;
			m_client.close() ;
			res=false ;
			delete msg ;
			return res  ;
		}
	}
	delete msg ;
	return res  ;
}

void CShakeSocketServer::serverNonBlocking(bool flag) {
  m_s.nonBlocking(flag) ;
}

CSocket & CShakeSocketServer::getSocket() {
  //if (m_connected) 
    return m_client ;
  //else return m_s ;
}

bool CShakeSocketServer::inboundClose() {
  m_created = false ;
    return m_s.close() ;
}

