#ifndef _LSMSOCKET_H_
#define _LSMSOCKET_H_
#include <stdlib.h>
#include "tcpsocket.h"
#include "shake_xdr.h"

class CShakeSocket ;

/********************************************************************************/
/* class CShakePacket                                                           */
/*                                                                              */
/* Description:	                                                                */
/*   send/receive data through sockets                                          */
/********************************************************************************/

#define DEFAULTMINPORT 5500
#define DEFAULTMAXPORT DEFAULTMINPORT+50

class CShakePacket {
private:
	char m_rtype[6] ;
        void init(void *data, unsigned long length, char *type) ;
protected:
	unsigned long m_length ;
	void *m_data ;
	char m_type[4] ;
public:
	CShakePacket() ;
	// the type of the packet : only the first 3 characters are significatif, by default type is "RAW"
	CShakePacket(void *data, unsigned long length, char *type=NULL) ; // supply a buffer to receive data
	CShakePacket(Cxdr &cxdr, char *type=NULL) ; // supply a buffer to receive data

	bool sendTo(CSocket &s) ;
	bool receiveFrom(CSocket &s) ;
	int getSize() ;
	char *getType() ;
	void *getData() ;
} ;

/********************************************************************************/
/* class CShakeSocket                                                           */
/*                                                                              */
/* Description:                                                                 */
/*   bas class for client/server socket application                             */
/********************************************************************************/

class CShakeSocket {
protected:
	int m_minPort,m_maxPort ; 
	int m_thePort ; // The port through which the server/client is connected
        bool m_connected ;

	CSocket m_s ; // The socket
	char *m_Id ; // the ID

	virtual CSocket & getSocket() ;
  	int recv(char *data, int bufsize=0) ; 
  	int send(char *data, int nLength=0) ; 
public:
	CShakeSocket(char *Id , int startPORT=5500, int endPORT=5600) ;
	virtual ~CShakeSocket() ;

	int recv(CShakePacket &packet) ;
	int send(CShakePacket &packet) ;

        bool wouldBlock() ;
	void nonBlocking(bool flag) ;
        bool connected() ;
        bool close() ;
        char *getId() { return m_Id ; } ;
        int  getPort() { return m_thePort ; } ;
};

/********************************************************************************/
/* class CShakeSocketClient                                                     */
/*                                                                              */
/* Description:                                                                 */
/*   client socket                                                              */
/********************************************************************************/

class CShakeSocketClient : public CShakeSocket {
protected:
	char *m_ServerId ; // a unique Id  
public:
	CShakeSocketClient(char *Id) ;
	virtual ~CShakeSocketClient() ;

	bool connectTo(char *host, int startPORT=0, int endPORT=0, char *Id=NULL) ; // create and connect to host 

};

/********************************************************************************/
/* class CShakeSocketServer                                                     */
/*                                                                              */
/* Description:                                                                 */
/*   server socket (accept 1 client)                                            */
/********************************************************************************/

class CShakeSocketServer : public CShakeSocket {
protected:
	CSocket m_client ;
	char *m_ClientId ; // a unique Id 
        bool m_created ;

	virtual CSocket & getSocket() ;
public:
	CShakeSocketServer(char *Id , int startPORT=DEFAULTMINPORT, int endPORT=DEFAULTMAXPORT) ;
	virtual ~CShakeSocketServer() ;

	void serverNonBlocking(bool flag) ;
        bool inboundClose() ;
	bool create() ; // create socket and listen (non-blocking) 
	bool accept() ; // wait for a connection.

        bool isCreated() ; // return true if created or open.
};


#endif


