#include "stdafx.h"
#include "SocketLoop.h"
#include "console.h"

CSocketSvc *sock = new CSocketSvc ;

int CSocketSvc::m_maxPort = DEFAULTMAXPORT ;
int CSocketSvc::m_minPort = DEFAULTMINPORT ;

CSocketSvc::CSocketSvc() { 
  m_Loop = NULL ;
  m_SocketClient = NULL ;
  m_SocketServer = NULL ;
  m_Socket = NULL ;
  m_MustExit = false ; 
}

void CSocketSvc::initAsClient(CSocketLoop *Loop,CShakeSocketClient *SocketClient) {
  m_Loop = Loop ;
  m_SocketClient = SocketClient ;
  m_Socket = SocketClient ;
}

void CSocketSvc::initAsServer(CSocketLoop *Loop,CShakeSocketServer *SocketServer) {
  m_Loop = Loop ;
  m_SocketServer = SocketServer ;
  m_Socket = SocketServer ;
}

CSocketLoop *CSocketSvc::loop() {
  return m_Loop ;
}

CShakeSocketClient *CSocketSvc::client() {
  return m_SocketClient ;
}

CShakeSocketServer *CSocketSvc::server() {
  return m_SocketServer ;
}

CShakeSocket *CSocketSvc::socket() {
  return m_Socket ;
}
 

bool CSocketSvc::msgloop() {
  CShakePacket *Packet=NULL ;

  if (!m_Socket) return false ;

  if (!m_Socket->connected()) {
    return notConnectedSW() ;
  } else {
    sock->loop()->getAllMessages() ;

    Packet = sock->loop()->getMessage() ;
    while (Packet) {
      bool res = m_packetSW(*Packet) ;
      if (!res) {
	sock->loop()->putMessage(Packet) ; // console.Warning() << "Packet lost of type " << Packet->getType() << "\n" ;
	Packet = NULL ;
      } else {
	delete Packet ;
	Packet = sock->loop()->getMessage() ;
      }
    }    
  }
  return true ;
}

bool CSocketSvc::mustExit() {
  return m_MustExit; 
}

bool CSocketSvc::closeInboundSW() {
  if (!sock->server()) {
    return false ;
  } else {
    return sock->server()->inboundClose() ;
  }
}


bool CSocketSvc::m_packetSW(CShakePacket &Packet) {
  if (!strcmp(Packet.getType(),"END")) {  // End connection (should return an ECK)
    return closeConnectionSW() ;
  } else if (!strcmp(Packet.getType(),"ECK")) { // End connection (terminal)
    return connectionEndedSW() ;
  } else if (!strcmp(Packet.getType(),"CLS")) { // Exit
    return closeInboundSW() ;
  } else if (!strcmp(Packet.getType(),"EVL")) { // Exit
    return reqEvaluateSW() ;
  } else if (!strcmp(Packet.getType(),"XIT")) { // Exit
    return exitSW() ;
  } else if (!strcmp(Packet.getType(),"PNG")) { // Exit
    return pingSW() ;
  } else if (!strcmp(Packet.getType(),"QRY")) { // Query
    return querySW() ;
  } else if (!strcmp(Packet.getType(),"QDT")) { // Reply for query
    return queryReplySW(Packet) ;
  } else {
    return false ;
  }
}

bool CSocketSvc::reqEvaluateSW() {
  sendTag("WAT") ;
  CShakePacket *pack = waitFor("LSM") ;
  char *script=(char *)(pack->getData()) ;
  evaluateSW(script) ;
  return true ;
}

bool CSocketSvc::queryReplySW(CShakePacket &Packet) {
  char Id[1024] ;
  unsigned short Port ; 
  char name[1024] ;

  Cxdr xdata((char *)(Packet.getData()),Packet.getSize()) ;
  xdata >> &Id[0] >> Port >> &name[0] ;
  queryReadSW(&Id[0],Port,&name[0]) ;
  return true ;
}

bool CSocketSvc::querySW() {
  Cxdr xdata ;

  char *Id ;
  unsigned short Port ; 
  char *name="unknown\0" ;
  Id = m_Socket->getId() ;
  Port = m_Socket->getPort() ;
  
  //tconsole.Debug() << "Processing query...\n" ;
  xdata << Id << Port << name ;
  CShakePacket QDT(xdata, "QDT") ; 
  m_Socket->send(QDT) ;
  //tconsole.Debug() << "Processing query: reply sent.\n" ;
  return true ; 
}

bool CSocketSvc::closeConnectionSW() {
    CShakePacket ECK((void *)"nothing", 1 , "ECK") ;
    m_Socket->send(ECK) ;
    m_Socket->close() ;
    return true ;
}

bool CSocketSvc::connectionEndedSW() {
    m_Socket->close() ;
    m_MustExit = true ;
    return true ;
}

bool CSocketSvc::exitSW() { 
  CShakePacket ECK((void *)"nothing", 1 , "ECK") ;
  m_Socket->send(ECK) ;
  m_Socket->close() ;
  m_MustExit = true ;
  return true ;
}

bool CSocketSvc::pingSW() { 
  CShakePacket ECK((void *)"nothing", 1 , "ACK") ;
  m_Socket->send(ECK) ;
  return true ;
}

CShakePacket *CSocketSvc::waitFor(char *Type) {
    CShakePacket *Eck=NULL ; 
    if (!m_Socket->connected()) {
      return NULL ;
    }
    while(!Eck) {
      loop()->getAllMessages() ;
      Eck = loop()->getMessage(Type) ;
    }
    return Eck ;
}

int CSocketSvc::sendTag(char *Type) {
    if (!m_Socket->connected()) {
      return -1 ;
    }
    CShakePacket P((void *)"nothing", 1 , Type) ; // ask server to end
    return m_Socket->send(P) ;
}

CSocketLoop::CSocketLoop(CShakeSocket &s) {
    m_s = &s ;
}

CSocketLoop::~CSocketLoop() {
    // empty m_PB...
}

bool CSocketLoop::checkMessages() {
    if (!m_s->connected()) return false ;
    CShakePacket *newPacket = new CShakePacket;
    int res ;
    m_s->nonBlocking(true) ;
    res = m_s->recv(*newPacket) ;
    m_s->nonBlocking(false) ;
    if (res<0) {
      if (m_s->wouldBlock()) {
        delete newPacket ;
        return false;
      } else {
	tconsole.Error() << "Error recieving message in msg_loop, closing connection\n" ;
	m_s->close() ;
        delete newPacket ;
        return false;	
      }
    } else { //receive a packet
        tconsole.Debug() << "receive one message of type " << newPacket->getType() << " ...\n" ;
        m_PB.Append(newPacket) ;
        return true ;
    }
} 


void CSocketLoop::getAllMessages() {
    while (checkMessages()) ;
}

CShakePacket *CSocketLoop::getMessage(char *Type) {
    CShakePacket *res=NULL, *cur ;
    m_PB.First() ;
    while ((! m_PB.IsEnd()) && (!res)) {
        cur = m_PB.Get() ;
        if (Type==NULL) {
            res = cur ;
            m_PB.Clear() ;
        } else if (!strcmp(cur->getType(),Type)) {
            res = cur ;
            m_PB.Clear() ;
        } else {
            m_PB.Next() ;
        }
    }
    return res ;
}

void CSocketLoop::putMessage(CShakePacket *P) {
  m_PB.Append(P) ;
}
