// --- System includes ---
#include <string>
#include <sstream>
#include <iostream>
#include <map>
#include <cstdlib>
#include <utility>
#include <vector>

// --- Project includes ---
#include "frame.h"
#include "mesh.h"
#include "colormap.h"
#include "geocolormap.h"
#include "campos.h"
#include "../../Foundation/vec3.h"

using std::cout;
using std::cerr;
using std::endl;
using std::string;
using std::ostringstream;
using std::atoi;
using std::vector;

int main(int argc,char** argv)
{
  vector<string> includefilename;
  string infilename;
  string ofilename;
  string basefilename;
  int t0_frame=0;
  int tstart;
  int nt;
  int dt;
  int camdir=0;
  bool options_valid=true;
  Vec3 campos,lookat;
  bool cam_set=false;
  bool with_mesh=false;
  ColorMap* color_map=NULL;
  int field=-1;
  string camerafilename;
  bool with_campos=false;
  CameraPos* camera_descr;

  // process args
  int args_read=1;  
  while(args_read<argc){
    string option=string(argv[args_read]);
    if(option=="-i"){
      if(argc>args_read){
	basefilename=string(argv[args_read+1]);
	args_read+=2;
      } else {
	options_valid=false;
      }
    } else if(option=="-o"){
      if(argc>args_read){
	ofilename=argv[args_read+1];
	args_read+=2;
      } else {
	options_valid=false;
      }
    } else if(option=="-t0"){
      if(argc>args_read){
	t0_frame=atoi(argv[args_read+1]);
	args_read+=2;
      } else {
	options_valid=false;
      }
    } else if(option=="-inc"){
      if(argc>args_read){
	includefilename.push_back(string(argv[args_read+1]));
	args_read+=2;
      } else {
	options_valid=false;
      }
    } else if(option=="-campos"){
      if(argc>args_read){
	camerafilename=string(argv[args_read+1]);
	with_campos=true;
	cam_set=true;
	args_read+=2;
      } else {
	options_valid=false;
      }
    } else if(option=="-t"){
      if(argc>args_read+2){
	tstart=atoi(argv[args_read+1]);
	nt=atoi(argv[args_read+2]);
	dt=atoi(argv[args_read+3]);
	args_read+=4;
      } else {
	options_valid=false;
      }
    } else if(option=="-dir"){
      if(argc>args_read){
	camdir=atoi(argv[args_read+1]);
	args_read+=2;
      } else {
	options_valid=false;
      }
    } else if(option=="-field"){
      if(argc>args_read){
	field=atoi(argv[args_read+1]);
	args_read+=2;
      } else {
	options_valid=false;
      }
    } else if(option=="-camera"){
      if(argc>args_read+5){
	double lookx=atof(argv[args_read+1]);
	double looky=atof(argv[args_read+2]);
	double lookz=atof(argv[args_read+3]);
	double cposx=atof(argv[args_read+4]);
	double cposy=atof(argv[args_read+5]);
	double cposz=atof(argv[args_read+6]);
	cam_set=true;
	lookat=Vec3(lookx,looky,lookz);
	campos=Vec3(cposx,cposy,cposz);
	args_read+=7;
      } else {
	options_valid=false;
      }
    } else if(option=="-color"){
      if(argc>args_read+7){
	double r0=atof(argv[args_read+1]);
	double g0=atof(argv[args_read+2]);
	double b0=atof(argv[args_read+3]);
	double r1=atof(argv[args_read+4]);
	double g1=atof(argv[args_read+5]);
	double b1=atof(argv[args_read+6]);
	double x0=atof(argv[args_read+7]);
	double x1=atof(argv[args_read+8]);
	color_map=new ColorMap(Vec3(r0,g0,b0),Vec3(r1,g1,b1),x0,x1);
	args_read+=9;
      } else {
	options_valid=false;
      } 
    } else if(option=="-geo"){
      if(argc>args_read+3){
	double x0=atof(argv[args_read+1]);
	double x1=atof(argv[args_read+2]);
	int nl=atoi(argv[args_read+3]);
	double rd=atof(argv[args_read+4]);
	color_map=new GeoColorMap(Vec3(0.64,0.48,0.395),Vec3(0.40,0.34,0.265),x0,x1,nl,rd);
	field=9;
	args_read+=5;
      } else {
	options_valid=false;
      }
    } else if(option=="-mesh") {
      with_mesh=true;
      options_valid=true;
      args_read++;
    } else {
      cerr << "Unknown option " << option << endl;
    }
  }

  if(options_valid){
    // read camera description file
    if(with_campos){
      camera_descr=new CameraPos(camerafilename);
    }
    for(int i=0;i<nt;i++){
      // make filename
      ostringstream filename;
      filename << basefilename << "_t=" << tstart+i*dt << "_0.txt"; 
      if(with_mesh){
	// setup filenames 
	// - evil hack - use ???_1.txt as source file -> change if mesh is distributed
	ostringstream mesh_infilename;
	mesh_infilename << basefilename << "_t=" << tstart+i*dt << "_1.txt"; 
	ostringstream mesh_outfilename;
	mesh_outfilename << "mesh_include_t=" << i+t0_frame << "_1.inc"; 
	// write mesh include
	do_mesh(mesh_infilename.str(),mesh_outfilename.str());
	// add mesh to include files
	includefilename.push_back(mesh_outfilename.str());
      }
      // setup camera position
      if(with_campos){
	pair<Vec3,Vec3> cp=camera_descr->getCamPos(tstart+i*dt);
	campos=cp.first;
	lookat=cp.second;
      }
      // write frame
      do_single_frame(filename.str(),ofilename,i+t0_frame,includefilename,camdir,color_map,field,lookat,campos,cam_set);
    }
  }

  if(color_map!=NULL) delete color_map;
  if(camera_descr!=NULL) delete camera_descr;
  return 0;
}



