/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "UnitTest/FileTestCase.h"

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/convenience.hpp>
#include <boost/filesystem/fstream.hpp>

#include <stdexcept>
#include <iostream>

namespace LsmTest
{
  class TmpPath
  {
  public:
    TmpPath() : m_path()
    {
    }

    ~TmpPath()
    {
      if (boost::filesystem::exists(path())) {
        boost::filesystem::remove_all(path());
      }
    }

    const boost::filesystem::path &path() const
    {
      return m_path;
    }
    
    boost::filesystem::path &path()
    {
      return m_path;
    }
    
  private:
    boost::filesystem::path m_path;
  };
  
  class FileTestCase::Impl
  {
  public:
    boost::filesystem::path m_basePath;
    
    static TmpPath s_basePath;    
  };

  TmpPath FileTestCase::Impl::s_basePath;

  FileTestCase::FileTestCase(std::string Name)
    : CppUnit::TestCase(Name),
      m_implPtr(new Impl())
  {
  }

  FileTestCase::FileTestCase()
    : CppUnit::TestCase(),
      m_implPtr(new Impl())
  {
  }

  FileTestCase::~FileTestCase()
  {
  }

  boost::filesystem::path FileTestCase::getUnitTestBasePath()
  {
    if (Impl::s_basePath.path().empty()) {
      boost::filesystem::path basePath("/tmp/Lsm");
      if (!boost::filesystem::exists(basePath)) {
        boost::filesystem::create_directories(basePath);
      }

      char dir[64] = "";
      strcpy(dir, (basePath / "UnitTestsXXXXXX").native_directory_string().c_str());
      if (mkdtemp(dir) != NULL) {
        Impl::s_basePath.path() = boost::filesystem::path(dir);
      }
      else {
        throw std::runtime_error(std::string() + "Could not make temporary directory:" + dir);
      }
    }
    return Impl::s_basePath.path();
  }

  void FileTestCase::setUp()
  {
    try
    {
      CppUnit::TestCase::setUp();
      m_implPtr->m_basePath = getUnitTestBasePath() / getName();
      boost::filesystem::create_directories(m_implPtr->m_basePath);
    }
    catch (std::runtime_error &e)
    {
      std::cerr << e.what() << std::endl;
      throw;
    }
  }

  void FileTestCase::tearDown()
  {
    if (boost::filesystem::exists(m_implPtr->m_basePath)) {
      boost::filesystem::remove_all(m_implPtr->m_basePath);
    }
    CppUnit::TestCase::tearDown();    
  }

  void FileTestCase::writeFile(
    const boost::filesystem::path &filePath,
    const std::string &data
  ) const
  {
    boost::filesystem::ofstream oStream(filePath);
    oStream << data;
  }

  boost::filesystem::path FileTestCase::getTestDirPath() const
  {
    return m_implPtr->m_basePath;
  }

  std::string FileTestCase::getTestDirString() const
  {
    return m_implPtr->m_basePath.string();
  }
}
