/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef LSMTESTFILETESTCASE_H
#define LSMTESTFILETESTCASE_H

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestCase.h>

#include <string>

#include <boost/shared_ptr.hpp>
#include <boost/filesystem/path.hpp>

namespace LsmTest
{
  class FileTestCase : public CppUnit::TestCase
  {
  public:
    FileTestCase(std::string Name);
    FileTestCase();
    virtual ~FileTestCase();
    
    /**
     * Returns parent path under which all test directories are created.
     */
    static boost::filesystem::path getUnitTestBasePath();

    /**
     * Returns the path of an existing (created in setUp())
     * temporary directory to which test data can be written.
     */        
    boost::filesystem::path getTestDirPath() const;

    /**
     * Returns the path of an existing (created in setUp())
     * temporary directory to which test data can be written.
     */
    std::string getTestDirString() const;
    
    /**
     * Creates temporary directory in which test data can be stored.
     */
    virtual void setUp();
    
    /**
     * Removes temporary test directory and all contents.
     */
    virtual void tearDown();
    
    /**
     * Write data-string to the specifed text-file. Existing
     * file is overwritten.
     */
    void writeFile(
      const boost::filesystem::path &filePath,
      const std::string &data
    ) const;

  private:
    class Impl;
    typedef boost::shared_ptr<Impl> ImplPtr;
    ImplPtr m_implPtr;
  };
}
#endif
