/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#ifndef LSMTESTTESTDLL_H
#define LSMTESTTESTDLL_H

#include <cppunit/extensions/HelperMacros.h>
#include <string>
#include <iostream>
#include <boost/limits.hpp>

namespace LsmTest
{
  class MsgStream
  {
  public:
    template <typename Tmpl>
    MsgStream operator<<(const Tmpl &t)
    {
      std::stringstream str;
      str << t;
      m_msg += str.str();
  
      return *this;
    }
  
    const std::string &getMsg() const
    {
      return m_msg;
    }
  
  private:
    std::string m_msg;
  };
  
};
inline std::ostream &operator<<(std::ostream &oStream, const LsmTest::MsgStream &msgStream)
{
  oStream << msgStream.getMsg();
  return oStream;
}

const double DEFAULT_TOL_DBL = std::numeric_limits<double>::epsilon()*1024;

#define LSMUNIT_ASSERT_VEC3_EQUAL2(EXPECTED, RESULT, TOL)                      \
  CPPUNIT_ASSERT_MESSAGE(                                                      \
    (                                                                          \
      LsmTest::MsgStream()                                                     \
        << "expected: " << EXPECTED << "\n"                                    \
        << " but was: " << RESULT << "\n"                                      \
        << " diff = : "                                                        \
        << Vec3(EXPECTED[0] - RESULT[0], EXPECTED[1] - RESULT[1], EXPECTED[2] - RESULT[2])\
    ).getMsg(),                                                                \
    (                                                                          \
      (fabs(EXPECTED[0] - RESULT[0]) <= TOL)                                   \
      &&                                                                       \
      (fabs(EXPECTED[1] - RESULT[1]) <= TOL)                                   \
      &&                                                                       \
      (fabs(EXPECTED[2] - RESULT[2]) <= TOL)                                   \
    )                                                                          \
  )


#define LSMUNIT_ASSERT_VEC3_EQUAL(EXPECTED, RESULT)                            \
  LSMUNIT_ASSERT_VEC3_EQUAL2(EXPECTED, RESULT, DEFAULT_TOL_DBL)


#define LSMUNIT_ASSERT_ENERGY_DISIPATED(INITIAL_ENERGY, COMPUTED_ENERGY, TOL)  \
    CPPUNIT_ASSERT_MESSAGE(                                                    \
      (MsgStream()                                                             \
        << "Energy " << COMPUTED_ENERGY                                        \
        << " not less than or equal to initial energy "                        \
        << INITIAL_ENERGY                                                      \
        << ". Error = "                                                        \
        << (static_cast<int>((fabs(COMPUTED_ENERGY - INITIAL_ENERGY)/INITIAL_ENERGY)*10000.0))/100.0\
        << "%"                                                                 \
      ).getMsg(),                                                              \
      COMPUTED_ENERGY <= INITIAL_ENERGY + TOL                                  \
    );

#define LSMUNIT_ASSERT_DOUBLES_EQUAL2(EXPECTED, RESULT, TOL)                   \
  CPPUNIT_ASSERT_DOUBLES_EQUAL(EXPECTED, RESULT, TOL)

#define LSMUNIT_ASSERT_DOUBLES_EQUAL(EXPECTED, RESULT)                         \
  LSMUNIT_ASSERT_DOUBLES_EQUAL2(EXPECTED, RESULT, DEFAULT_TOL_DBL)
  
#endif
