/*=============================================================================
??
??******************************************************************************
??*                                               *
??* COPYRIGHT ??????? - All Rights Reserved *
??*  *
??* This software is the property of ??????????????. No part of this code   *
??* may be copied in any form or by any means without the expressed written   *
??* consent of ???????????. Copying, use or modification of this software     *
??* by any unauthorised person is illegal unless that    *
??* person has a software license agreement with ?????????????.            *
??*                                                          *
??******************************************************************************
??
*********************************************************************************/

#include "RegistryRunner.h"

#include <iostream>
#include <string>
#include <vector>
#include <stdexcept>

#include <UnitTest/TestDll.h>
#include <libltdl/ltdl.h>

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>

namespace LsmTest
{
  typedef std::vector<std::string> StringVector;
  class RegistryRunner::Impl
  {
    public:
      StringVector m_dlNames;
  };
  
  RegistryRunner::RegistryRunner() : m_pImpl(new Impl)
  {
  }

  RegistryRunner::RegistryRunner(int argc, char *argv[]) : m_pImpl(new Impl)
  {
    for (int i = 1; i < argc; i++)
    {
      m_pImpl->m_dlNames.push_back(argv[i]);
    }
  }

  RegistryRunner::~RegistryRunner()
  {
    delete m_pImpl;
  }

  void RegistryRunner::run()
  {
    if (lt_dlinit() != 0) {
      const char *errMsg = lt_dlerror();
      throw
        std::runtime_error(
          std::string("Error during ltdl initialization: ")
          +
          (errMsg ? errMsg : "unknown")
        );
    }

    for (
      StringVector::const_iterator it = m_pImpl->m_dlNames.begin();
      it != m_pImpl->m_dlNames.end();
      it++
    )
    {
      try {
        LsmTest::TestDll testDll(*it);
        testDll.load();
      }
      catch (std::exception &e) {
        std::cerr << e.what() << std::endl;
      }
    }
    CppUnit::TextUi::TestRunner runner;

    CppUnit::TestFactoryRegistry &registry = CppUnit::TestFactoryRegistry::getRegistry();

    runner.addTest( registry.makeTest() );
    runner.run();
  }
} // namespace LsmTest
