template<typename P,typename I>
ParallelInteractionStorage_ED<P,I>::ParallelInteractionStorage_ED(AParallelParticleArray* ppa,const typename I::ParameterType& param):ParallelInteractionStorage_E<P,I>(ppa,param)
{
  m_exIG=NULL;
  m_update_timestamp=0;
}

template<typename P,typename InteractionType>
void  ParallelInteractionStorage_ED<P,InteractionType>::addExIG(AParallelInteractionStorage* eg)
{
  m_exIG=eg; 
}

/*!
  Update interactions. Do full dynamic search.
*/
template<typename T,typename InteractionType>
bool ParallelInteractionStorage_ED<T,InteractionType>::update()
{
  //  std::cout << "ParallelInteractionStorage_ED::update at node " << m_comm.rank() << std::endl << std::flush;
  int count_l=0;
	bool res=true;

  if(m_update_timestamp!=m_ppa->getTimeStamp()){// m_ppa rebuild since last update 
    console.XDebug() << "node " << m_comm.rank() << " ppa has been rebuilt\n";
    // clean out old interactions if not flagged as persistent
    typename list<InteractionType>::iterator iter=m_interactions.begin();
    while(iter!=m_interactions.end()){
      if(iter->isPersistent()){
	iter++;
	//console.XDebug() << "node " << m_comm.rank() << "persistent interaction\n";
      }else{
	typename list<InteractionType>::iterator er_iter=iter;
	// get particle ids and remove pair from set
	vector<int> rm_pids=iter->getAllID();
	m_set.erase(make_pair(rm_pids[0],rm_pids[1]));
	iter++;
	m_interactions.erase(er_iter);
      }
    }
    // get list  of pairs from m_ppa
    console.XDebug() << "node " << m_comm.rank() << " before full pair list\n";
    typename ParallelParticleArray<T>::PairListHandle plh=((ParallelParticleArray<T>*)m_ppa)->getFullPairList();
    console.XDebug() << "node " << m_comm.rank() << " got full pair list\n";
    // generate interactions from pairs
    for(typename ParallelParticleArray<T>::PairListIterator iter=plh->begin();
	iter!=plh->end();
	iter++){
      // check vs. ExIG
      vector<int> tv;
      tv.push_back(iter->first->getID());
      tv.push_back(iter->second->getID());
      if(m_exIG!=NULL){
	if((!m_exIG->isIn(tv))&&(!isIn(tv))){ 
	  m_interactions.push_back(InteractionType(iter->first,iter->second,m_param));
	  m_set.insert(pair<int,int>(iter->first->getID(),iter->second->getID()));
	  count_l++;
	} 
      } else if (!isIn(tv)) {
	m_interactions.push_back(InteractionType(iter->first,iter->second,m_param));
	m_set.insert(pair<int,int>(iter->first->getID(),iter->second->getID()));
      }
    }
  } else { // m_ppa not rebuild since last update -> just get additional interactions
    console.XDebug() << "node " << m_comm.rank() << " ppa not rebuilt\n";
    // get list  of pairs from m_ppa
    typename ParallelParticleArray<T>::PairListHandle plh=((ParallelParticleArray<T>*)m_ppa)->getNewPairList();
    for(typename ParallelParticleArray<T>::PairListIterator iter=plh->begin();
	iter!=plh->end();
	iter++){
      // check vs. ExIG
      vector<int> tv;
      tv.push_back(iter->first->getID());
      tv.push_back(iter->second->getID());
      if(m_exIG!=NULL){
	if((!m_exIG->isIn(tv))&&(!isIn(tv))){
	  m_interactions.push_back(InteractionType(iter->first,iter->second,m_param));
	  m_set.insert(pair<int,int>(iter->first->getID(),iter->second->getID()));
	  count_l++;
	}
      } else if (!isIn(tv)) {
	m_interactions.push_back(InteractionType(iter->first,iter->second,m_param));
	m_set.insert(pair<int,int>(iter->first->getID(),iter->second->getID()));
      }
    }
  }
  m_update_timestamp=m_ppa->getTimeStamp();

  console.Debug() << "added " << count_l << " pairs to ParallelInteractionStorage_ED\n";
  //  std::cout << "end ParallelInteractionStorage_ED::update at node " << m_comm.rank() << std::endl << std::flush;

	return res;
}
